<?php
namespace MaxAddonsPro\Classes;

use MaxAddonsPro\Classes\Helper;

/**
 * Handles logic for the admin settings page.
 *
 * @since 1.0.0
 */
final class Admin_Settings {
	/**
	 * Holds any errors that may arise from
	 * saving admin settings.
	 *
	 * @since 1.0.0
	 * @var array $errors
	 */
	public static $errors = array();

	public static $settings = array();

	/**
	 * Initializes the admin settings.
	 *
	 * @since 1.0.0
	 * @return void
	 */
	public static function init() {
		self::migrate_settings();

		add_action( 'plugins_loaded', __CLASS__ . '::init_hooks' );

		add_action( 'plugins_loaded', __CLASS__ . '::save_elements' );

		add_action( 'plugins_loaded', __CLASS__ . '::save_woo_builder_elements' );

		add_action( 'plugins_loaded', __CLASS__ . '::save_conditions' );

		add_action( 'plugins_loaded', __CLASS__ . '::save_features' );
	}

	/**
	 * Adds the admin menu and enqueues CSS/JS if we are on
	 * the plugin's admin settings page.
	 *
	 * @since 1.0.0
	 * @return void
	 */
	public static function init_hooks() {
		if ( ! is_admin() ) {
			return;
		}

		add_action( 'admin_menu', __CLASS__ . '::add_menu_page', 601 );
		add_filter( 'all_plugins', __CLASS__ . '::update_branding' );

		if ( isset( $_REQUEST['page'] ) && 'mab-settings' == $_REQUEST['page'] ) {
			//add_action( 'admin_enqueue_scripts', __CLASS__ . '::styles_scripts' );
			self::save();
		}

		add_action( 'admin_init', __CLASS__ . '::refresh_instagram_access_token' );
	}

	/**
	 * Enqueues the needed CSS/JS for the builder's admin settings page.
	 *
	 * @since 1.0.0
	 * @return void
	 */
	public static function styles_scripts() {
		// Styles
		//wp_enqueue_style( 'mab-admin-settings', MAB_PRO_URL . 'assets/css/admin-settings.css', array(), MAB_PRO_VER );
	}

	/**
	 * Get elements.
	 *
	 * @since 1.0.0
	 * @return array
	 */
	public static function get_elements() {
		$elements = array(
			'album'             => esc_html__( 'Album', 'max-addons' ),
			'animated-heading'  => esc_html__( 'Animated Heading', 'max-addons' ),
			'author-list'       => esc_html__( 'Author List', 'max-addons' ),
			'breadcrumbs'       => esc_html__( 'Breadcrumbs', 'max-addons' ),
			'copy-to-clipboard' => esc_html__( 'Copy to Clipboard', 'max-addons' ),
			'cta-button'        => esc_html__( 'CTA Button', 'max-addons' ),
			'content-ticker'    => esc_html__( 'Content Ticker', 'max-addons' ),
			'dynamic-charts'    => esc_html__( 'Dynamic Charts', 'max-addons' ),
			'dynamic-table'     => esc_html__( 'Dynamic Table', 'max-addons' ),
			'flip-box'          => esc_html__( 'Flip Box', 'max-addons' ),
			'icon-list'         => esc_html__( 'Dynamic List', 'max-addons' ),
			'image-accordion'   => esc_html__( 'Image Accordion', 'max-addons' ),
			'image-gallery'     => esc_html__( 'Image Gallery', 'max-addons' ),
			'hotspots'          => esc_html__( 'Hotspots', 'max-addons' ),
			'image-comparison'  => esc_html__( 'Image Comparison', 'max-addons' ),
			'instagram-feed'    => esc_html__( 'Instagram Feed', 'max-addons' ),
			'lottie'            => esc_html__( 'Lottie', 'max-addons' ),
			'multi-heading'     => esc_html__( 'Multi Heading', 'max-addons' ),
			'offcanvas'         => esc_html__( 'Off Canvas', 'max-addons' ),
			'post-tiles'        => esc_html__( 'Post Tiles', 'max-addons' ),
			'random-image'      => esc_html__( 'Random Image', 'max-addons' ),
			'reviews'           => esc_html__( 'Reviews', 'max-addons' ),
			'table'             => esc_html__( 'Table', 'max-addons' ),
			'table-nested'      => esc_html__( 'Table (Nestable)', 'max-addons' ),
			'unfold'            => esc_html__( 'Unfold', 'max-addons' ),
			'slide-menu'        => esc_html__( 'Slide Menu', 'max-addons' ),
			'rating'            => esc_html__( 'Rating', 'max-addons' ),
			'svg-animation'     => esc_html__( 'SVG Animation', 'max-addons' ),
			'toggle'            => esc_html__( 'Toggle', 'max-addons' ),
			'thumbnail-slider'  => esc_html__( 'Thumbnail Slider', 'max-addons' ),
			'timeline'          => esc_html__( 'Timeline', 'max-addons' ),
			'video'             => esc_html__( 'Video', 'max-addons' ),
			'video-gallery'     => esc_html__( 'Video Gallery', 'max-addons' ),
			'modal-popup'       => esc_html__( 'Modal Popup', 'max-addons' ),
			'reusable-block'    => esc_html__( 'Reusable Block', 'max-addons' ),
			'toc'               => esc_html__( 'Table of Contents', 'max-addons' ),
		);

		// Contact Form 7
		if ( function_exists( 'wpcf7' ) ) {
			$elements['cf7-styler'] = esc_html__( 'Contact Form 7 Styler', 'max-addons' );
		}

		// Gravity Forms
		if ( class_exists( 'GFForms' ) ) {
			$elements['gravity-forms-styler'] = esc_html__( 'Gravity Forms Styler', 'max-addons' );
		}

		// Fluent Forms
		if ( function_exists( 'wpFluentForm' ) ) {
			$elements['fluent-forms-styler'] = esc_html__( 'Fluent Forms Styler', 'max-addons' );
		}

		// Formidable Forms
		if ( class_exists( 'FrmForm' ) ) {
			$elements['formidable-forms-styler'] = __( 'Formidable Forms Styler', 'max-addons' );
		}

		asort( $elements );

		return $elements;
	}

	/**
	 * Get woo elements.
	 *
	 * @since 1.7.1
	 * @return array
	 */
	public static function get_woo_elements() {
		$elements = array();

		if ( Helper::is_woocommerce_active() ) {
			$elements['woo-add-to-cart']        = esc_html__( 'Woo - Add to Cart', 'max-addons' );
			$elements['woo-buy-now-button']     = esc_html__( 'Woo - Buy Now Button', 'max-addons' );
			$elements['woo-free-shipping-bar']  = esc_html__( 'Woo - Free Shipping Bar', 'max-addons' );
			$elements['woo-product-badge']      = esc_html__( 'Woo - Product Badges', 'max-addons' );
			$elements['woo-product-gallery']    = esc_html__( 'Woo - Product Gallery', 'max-addons' );
			$elements['woo-product-image']      = esc_html__( 'Woo - Product Image', 'max-addons' );
			$elements['woo-product-reviews']    = esc_html__( 'Woo - Product Reviews', 'max-addons' );
			$elements['woo-empty-cart-button']  = esc_html__( 'Woo - Empty Cart Button', 'max-addons' );
			$elements['woo-stock-progress-bar'] = esc_html__( 'Woo - Stock Progress Bar', 'max-addons' );

			ksort( $elements );
		}

		return $elements;
	}

	/**
	 * Get woo builder elements.
	 *
	 * @since 1.11.0
	 * @return array
	 */
	public static function get_woo_builder_elements() {
		$elements = array();

		if ( Helper::is_woocommerce_active() ) {
			$elements['woo-order-details']     = [
				'category' => esc_html__( 'Thank You Page Builder', 'max-addons' ),
				'title'    => esc_html__( 'Woo - Order Details', 'max-addons' )
			];
			$elements['woo-order-overview']    = [
				'category' => esc_html__( 'Thank You Page Builder', 'max-addons' ),
				'title'    => esc_html__( 'Woo - Order Overview', 'max-addons' )
			];
			$elements['woo-order-billing-address'] = [
				'category' => esc_html__( 'Thank You Page Builder', 'max-addons' ),
				'title'    => esc_html__( 'Woo - Order Billing Address', 'max-addons' )
			];
			$elements['woo-order-shipping-address'] = [
				'category' => esc_html__( 'Thank You Page Builder', 'max-addons' ),
				'title'    => esc_html__( 'Woo - Order Shipping Address', 'max-addons' )
			];
			$elements['woo-checkout-billing-fields'] = [
				'category' => esc_html__( 'Checkout Page Builder', 'max-addons' ),
				'title'    => esc_html__( 'Woo - Billing Fields', 'max-addons' )
			];
			$elements['woo-checkout-shipping-fields'] = [
				'category' => esc_html__( 'Checkout Page Builder', 'max-addons' ),
				'title'    => esc_html__( 'Woo - Shipping Fields', 'max-addons' )
			];
			$elements['woo-checkout-hooks'] = [
				'category' => esc_html__( 'Checkout Page Builder', 'max-addons' ),
				'title'    => esc_html__( 'Woo - Checkout Hooks', 'max-addons' )
			];

			// ksort( $elements );
		}

		return $elements;
	}

	/**
	 * Get conditions.
	 *
	 * @since 1.2.0
	 * @return array
	 */
	public static function get_conditions() {
		$conditions = array(
			'post_category'        => esc_html__( 'Post category', 'max-addons' ),
			'post_tag'             => esc_html__( 'Post tag', 'max-addons' ),
			'post_type'            => esc_html__( 'Post type', 'max-addons' ),
			'post_comments_count'  => esc_html__( 'Post comments count', 'max-addons' ),
			'post_comments_status' => esc_html__( 'Post comments status', 'max-addons' ),
			'author_archive'       => esc_html__( 'Author archive', 'max-addons' ),
			'category_archive'     => esc_html__( 'Category archive', 'max-addons' ),
			'tag_archive'          => esc_html__( 'Tag archive', 'max-addons' ),
			'search_results'       => esc_html__( 'Search', 'max-addons' ),
			'request_parameter'    => esc_html__( 'Request parameter', 'max-addons' ),
			'is_parent'            => esc_html__( 'Is parent?', 'max-addons' ),
			'is_child'             => esc_html__( 'Is child?', 'max-addons' ),
			'page_type'            => esc_html__( 'Page type', 'max-addons' ),
		);

		ksort( $conditions );

		return $conditions;
	}

	/**
	 * Get conditions.
	 *
	 * @since 1.2.0
	 * @return array
	 */
	public static function get_woo_conditions() {
		$conditions = array();

		if ( Helper::is_woocommerce_active() ) {
			$conditions = array(
				'woo_product_category'     => esc_html__( 'Product category', 'max-addons' ),
				'woo_product_price'        => esc_html__( 'Product price', 'max-addons' ),
				'woo_product_rating'       => esc_html__( 'Product rating', 'max-addons' ),
				'woo_product_stock'        => esc_html__( 'Product stock', 'max-addons' ),
				'woo_product_type'         => esc_html__( 'Product type', 'max-addons' ),
				'woo_orders_placed'        => esc_html__( 'Orders placed', 'max-addons' ),
				'woo_customer_total_spent' => esc_html__( 'Customer total spent', 'max-addons' ),
				'woo_cart_products_count'  => esc_html__( 'Cart products count', 'max-addons' ),
				'woo_cart_products'        => esc_html__( 'Cart products', 'max-addons' ),
				'woo_cart_total'           => esc_html__( 'Cart total', 'max-addons' ),
			);

			ksort( $conditions );
		}

		return $conditions;
	}

	/**
	 * Get features.
	 *
	 * @since 1.2.0
	 * @return array
	 */
	public static function get_features() {
		$features = array(
			'background-slideshow' => esc_html__( 'Background Slideshow', 'max-addons' ),
			'mask'                 => esc_html__( 'Mask', 'max-addons' ),
			'tooltip'              => esc_html__( 'Tooltip', 'max-addons' ),
			'wrapper-link'         => esc_html__( 'Wrapper Link', 'max-addons' ),
		);

		ksort( $features );

		return $features;
	}

	/**
	 * Get enabled elements.
	 *
	 * @since 1.0.0
	 * @return array
	 */
	public static function get_enabled_elements() {
		$enabled_modules = get_option( 'max_bricks_elements', 'disabled' );

		if ( is_array( $enabled_modules ) ) {
			if ( ! Helper::is_woocommerce_active() ) {
				$filteredArray = array_filter( $enabled_modules, function($item) {
					return strpos($item, 'woo') !== 0; // Exclude items that start with "woo"
				} );

				return $filteredArray;
			}

			return $enabled_modules;
		}

		if ( 'disabled' == $enabled_modules ) {
			return $enabled_modules;
		}

		return self::get_elements();
	}

	/**
	 * Get enabled woo builder elements.
	 *
	 * @since 1.11.0
	 * @return array
	 */
	public static function get_enabled_woo_builder_elements() {
		$enabled_modules = get_option( 'max_bricks_woo_builder_elements', 'disabled' );

		if ( is_array( $enabled_modules ) ) {
			return $enabled_modules;
		}

		if ( 'disabled' == $enabled_modules ) {
			return $enabled_modules;
		}

		return self::get_woo_builder_elements();
	}

	/**
	 * Get enabled elements.
	 *
	 * @since 1.0.0
	 * @return array
	 */
	public static function get_enabled_conditions() {
		$enabled_conditions = get_option( 'max_bricks_conditions', 'disabled' );

		if ( is_array( $enabled_conditions ) ) {
			return $enabled_conditions;
		}

		if ( 'disabled' == $enabled_conditions ) {
			return $enabled_conditions;
		}

		return self::get_conditions();
	}

	/**
	 * Get enabled elements.
	 *
	 * @since 1.0.0
	 * @return array
	 */
	public static function get_enabled_features() {
		$enabled_features = get_option( 'max_bricks_features', 'disabled' );

		if ( is_array( $enabled_features ) ) {
			return $enabled_features;
		}

		if ( 'disabled' == $enabled_features ) {
			return $enabled_features;
		}

		return self::get_features();
	}

	/**
	 * Get settings.
	 *
	 * @since 1.0.0
	 * @return array
	 */
	public static function get_settings() {
		$default_settings = array(
			'plugin_name'          => '',
			'plugin_short_name'    => '',
			'plugin_desc'          => '',
			'plugin_author'        => '',
			'plugin_uri'           => '',
			'admin_label'          => '',
			'support_link'         => '',
			'hide_support'         => 'off',
			'hide_wl_settings'     => 'off',
			'hide_plugin'          => 'off',
		);

		$settings = get_option( 'mab_elements_settings' );

		if ( ! is_array( $settings ) || empty( $settings ) ) {
			$settings = $default_settings;
		}

		if ( is_array( $settings ) && ! empty( $settings ) ) {
			$settings = array_merge( $default_settings, $settings );
		}

		return apply_filters( 'mab_elements_admin_settings', $settings );
	}

	/**
	 * Get admin label from settings.
	 *
	 * @since 1.0.0
	 * @return string
	 */
	public static function get_admin_label() {
		$settings = self::get_settings();

		$admin_label = $settings['admin_label'];

		return trim( $admin_label ) == '' ? 'Max Addons' : trim( $admin_label );
	}

	/**
	 * Renders the update message.
	 *
	 * @since 1.0.0
	 * @return void
	 */
	public static function render_update_message() {
		if ( ! empty( self::$errors ) ) {
			foreach ( self::$errors as $message ) {
				echo '<div class="error"><p>' . wp_kses_post( $message ) . '</p></div>';
			}
		} elseif ( ! empty( $_POST ) && ! isset( $_POST['email'] ) ) {
			echo '<div class="updated"><p>' . esc_html__( 'Settings updated!', 'max-addons' ) . '</p></div>';
		}
	}

	/**
	 * Adds an error message to be rendered.
	 *
	 * @since 1.0.0
	 * @param string $message The error message to add.
	 * @return void
	 */
	public static function add_error( $message ) {
		self::$errors[] = $message;
	}

	static public function parse_error( $message ) {
		if ( isset( $_GET['mab_debug'] ) ) {
			return $message;
		}
		if ( false !== strpos( $message, 'wpbricksaddons' ) ) {
			return esc_html__( 'Could not connect to the host.', 'max-addons' );
		}

		return;
	}

	/**
	 * Renders the admin settings menu.
	 *
	 * @since 1.0.0
	 * @return void
	 */
	public static function add_menu_page() {
		$admin_label = self::get_admin_label();

		$title = $admin_label;
		$cap   = 'manage_options';
		$slug  = 'mab-settings';
		$func  = __CLASS__ . '::render';

		add_submenu_page( 'bricks', $title, $title, $cap, $slug, $func );
	}

	public static function render() {
		include MAB_PRO_DIR . 'includes/admin/admin-settings.php';
		//include MAB_PRO_DIR . 'includes/modules-manager.php';
	}

	public static function get_tabs() {
		$settings = self::get_settings();

		return apply_filters( 'mab_elements_admin_settings_tabs', array(
			'general'     => array(
				'title'    => esc_html__( 'General', 'max-addons' ),
				'show'     => true,
				'cap'      => 'manage_options',
				'file'     => MAB_PRO_DIR . 'includes/admin/admin-settings-license.php',
				'priority' => 50,
			),
			'elements'    => array(
				'title'    => esc_html__( 'Elements', 'max-addons' ),
				'show'     => true,
				'cap'      => 'edit_posts',
				'file'     => MAB_PRO_DIR . 'includes/admin/admin-settings-elements.php',
				'priority' => 100,
			),
			'conditions'    => array(
				'title'    => esc_html__( 'Conditions', 'max-addons' ),
				'show'     => true,
				'cap'      => 'edit_posts',
				'file'     => MAB_PRO_DIR . 'includes/admin/admin-settings-conditions.php',
				'priority' => 150,
			),
			'features'     => array(
				'title'    => esc_html__( 'Features', 'max-addons' ),
				'show'     => true,
				'cap'      => 'edit_posts',
				'file'     => MAB_PRO_DIR . 'includes/admin/admin-settings-features.php',
				'priority' => 200,
			),
			'woo-builder'     => array(
				'title'    => esc_html__( 'Woo Builder', 'max-addons' ),
				'show'     => ( Helper::is_woocommerce_active() ),
				'cap'      => 'edit_posts',
				'file'     => MAB_PRO_DIR . 'includes/admin/admin-settings-woo-builder.php',
				'priority' => 210,
			),
			'integration' => array(
				'title'    => esc_html__( 'Integration', 'max-addons' ),
				'show'     => ! self::get_option( 'mab_hide_integration_tab' ),
				'cap'      => ! is_network_admin() ? 'manage_options' : 'manage_network_plugins',
				'file'     => MAB_PRO_DIR . 'includes/admin/admin-settings-integration.php',
				'priority' => 250,
			),
			'white-label'   => array(
				'title'     => esc_html__( 'White Label', 'max-addons' ),
				'show'      => ( is_network_admin() || ! is_multisite() ) && ( ! self::get_option( 'mab_hide_wl_settings' ) || self::get_option( 'mab_hide_wl_settings' ) == 0 ),
				'cap'		=> ! is_network_admin() ? 'manage_options' : 'manage_network_plugins',
				'file'      => MAB_PRO_DIR . 'includes/admin/admin-settings-wl.php',
				'priority'  => 300,
			),
		) );
	}

	public static function render_tabs( $current_tab ) {
		$tabs = self::get_tabs();
		$sorted_data = array();

		foreach ( $tabs as $key => $data ) {
			$data['key'] = $key;
			$sorted_data[ $data['priority'] ] = $data;
		}

		ksort( $sorted_data );

		foreach ( $sorted_data as $data ) {
			if ( $data['show'] ) {
				if ( isset( $data['cap'] ) && ! current_user_can( $data['cap'] ) ) {
					continue;
				}
				?>
				<a href="<?php echo self::get_form_action( '&tab=' . $data['key'] ); ?>" class="nav-tab<?php echo ( $current_tab == $data['key'] ? ' nav-tab-active' : '' ); ?>"><span><?php echo $data['title']; ?></span></a>
				<?php
			}
		}
	}

	public static function render_setting_page() {
		$tabs = self::get_tabs();
		$current_tab = self::get_current_tab();

		if ( isset( $tabs[ $current_tab ] ) ) {
			$no_setting_file_msg = esc_html__( 'Setting page file could not be located.', 'max-addons' );

			if ( ! isset( $tabs[ $current_tab ]['file'] ) || empty( $tabs[ $current_tab ]['file'] ) ) {
				echo $no_setting_file_msg;
				return;
			}

			if ( ! file_exists( $tabs[ $current_tab ]['file'] ) ) {
				echo $no_setting_file_msg;
				return;
			}

			$render = ! isset( $tabs[ $current_tab ]['show'] ) ? true : $tabs[ $current_tab ]['show'];
			$cap = 'manage_options';

			if ( isset( $tabs[ $current_tab ]['cap'] ) && ! empty( $tabs[ $current_tab ]['cap'] ) ) {
				$cap = $tabs[ $current_tab ]['cap'];
			} else {
				$cap = ! is_network_admin() ? 'manage_options' : 'manage_network_plugins';
			}

			if ( ! $render || ! current_user_can( $cap ) ) {
				esc_html_e( 'You do not have permission to view this setting.', 'max-addons' );
				return;
			}

			include $tabs[ $current_tab ]['file'];
		}
	}

	/**
	 * Get current tab.
	 */
	public static function get_current_tab() {
		$current_tab = isset( $_GET['tab'] ) ? $_GET['tab'] : 'general';

		// if ( ! isset( $_GET['tab'] ) ) {
		// 	if ( is_multisite() && ! is_network_admin() ) {
		// 		$current_tab = 'elements';
		// 	}
		// }

		return $current_tab;
	}

	/**
	 * Renders the action for a form.
	 *
	 * @since 1.0.0
	 * @param string $type The type of form being rendered.
	 * @return void
	 */
	public static function get_form_action( $type = '' ) {
		if ( is_network_admin() ) {
			return esc_url( network_admin_url( '/admin.php?page=mab-settings' . $type ) );
		} else {
			return esc_url( admin_url( '/admin.php?page=mab-settings' . $type ) );
		}
	}

	public static function get_user_roles() {
		global $wp_roles;

		return $wp_roles->get_names();
	}

	/**
	 * Returns an option from the database for
	 * the admin settings page.
	 *
	 * @since 1.0.0
	 * @param string $key The option key.
	 * @return mixed
	 */
	public static function get_option( $key, $network_override = true ) {
		if ( is_network_admin() ) {
			$value = get_site_option( $key );
		} elseif ( ! $network_override && is_multisite() ) {
			$value = get_site_option( $key );
		} elseif ( $network_override && is_multisite() ) {
			$value = get_option( $key );
			$value = ( false === $value || ( is_array( $value ) && in_array( 'disabled', $value ) && get_option( 'mab_override_ms' ) != 1 ) ) ? get_site_option( $key ) : $value;
		} else {
			$value = get_option( $key );
		}

		return $value;
	}

	/**
	 * Updates an option from the admin settings page.
	 *
	 * @since 1.0.0
	 * @param string $key The option key.
	 * @param mixed $value The value to update.
	 * @return mixed
	 */
	public static function update_option( $key, $value, $network_override = true ) {
		if ( is_network_admin() ) {
			update_site_option( $key, $value );
		}
		// Delete the option if network overrides are allowed and the override checkbox isn't checked.
		elseif ( $network_override && is_multisite() && ! isset( $_POST['mab_override_ms'] ) ) {
			delete_option( $key );
		} else {
			update_option( $key, $value );
		}
	}

	/**
	 * Delete an option from the admin settings page.
	 *
	 * @since 1.0.0
	 * @param string $key The option key.
	 * @param mixed $value The value to delete.
	 * @return mixed
	 */
	public static function delete_option( $key ) {
		if ( is_network_admin() ) {
			delete_site_option( $key );
		} else {
			delete_option( $key );
		}
	}

	/**
	 * Set the branding data to plugin.
	 *
	 * @since 1.0.0
	 * @return array
	 */
	public static function update_branding( $all_plugins ) {
		if ( ! is_array( $all_plugins ) || empty( $all_plugins ) || ! isset( $all_plugins[ MAB_PRO_BASE ] ) ) {
			return $all_plugins;
		}

		$settings = self::get_settings();

		$all_plugins[ MAB_PRO_BASE ]['Name']           = ! empty( self::get_option('mab_plugin_name') ) ? self::get_option('mab_plugin_name') : $all_plugins[ MAB_PRO_BASE ]['Name'];
		$all_plugins[ MAB_PRO_BASE ]['PluginURI']      = ! empty( self::get_option('mab_plugin_uri') ) ? self::get_option('mab_plugin_uri') : $all_plugins[ MAB_PRO_BASE ]['PluginURI'];
		$all_plugins[ MAB_PRO_BASE ]['Description']    = ! empty( self::get_option('mab_plugin_desc') ) ? self::get_option('mab_plugin_desc') : $all_plugins[ MAB_PRO_BASE ]['Description'];
		$all_plugins[ MAB_PRO_BASE ]['Author']         = ! empty( self::get_option('mab_plugin_author') ) ? self::get_option('mab_plugin_author') : $all_plugins[ MAB_PRO_BASE ]['Author'];
		$all_plugins[ MAB_PRO_BASE ]['AuthorURI']      = ! empty( self::get_option('mab_plugin_uri') ) ? self::get_option('mab_plugin_uri') : $all_plugins[ MAB_PRO_BASE ]['AuthorURI'];
		$all_plugins[ MAB_PRO_BASE ]['Title']          = ! empty( self::get_option('mab_plugin_name') ) ? self::get_option('mab_plugin_name') : $all_plugins[ MAB_PRO_BASE ]['Title'];
		$all_plugins[ MAB_PRO_BASE ]['AuthorName']     = ! empty( self::get_option('mab_plugin_author') ) ? self::get_option('mab_plugin_author') : $all_plugins[ MAB_PRO_BASE ]['AuthorName'];

		if ( self::get_option('hide_plugin') == 'on' ) {
			unset( $all_plugins[ MAB_PRO_BASE ] );
		}

		return $all_plugins;
	}

	public static function save() {
		// Only admins can save settings.
		if ( ! current_user_can( 'manage_options' ) ) {
			return;
		}

		self::save_license();
		self::save_elements();
		self::save_integration();
		self::save_white_label();

		do_action( 'mab_elements_admin_settings_save' );
	}

	/**
	 * Saves the license.
	 *
	 * @since 1.0.0
	 * @access private
	 * @return void
	 */
	private static function save_license() {
		if ( isset( $_POST['mab_license_key'] ) ) {

			$old = get_option( 'mab_license_key' );
			$new = $_POST['mab_license_key'];

			if ( $old && $old != $new ) {
				delete_option( 'mab_license_status' ); // new license has been entered, so must reactivate
			}

			update_option( 'mab_license_key', $new );
		}
	}

	/**
	 * Saves integrations.
	 *
	 * @since 1.0.0
	 * @access private
	 * @return void
	 */
	private static function save_integration() {
		if ( ! isset( $_POST['mab_license_deactivate'] ) && ! isset( $_POST['mab_license_activate'] ) ) {
			if ( isset( $_POST['mab_google_places_api_key'] ) ) {

				// Validate Google Places API.
				if ( ! empty( $_POST['mab_google_places_api_key'] ) ) {
					$place_id = 'ChIJjZwurGTlZzkRLVOKFH8rKYc';
					$api_key  = $_POST['mab_google_places_api_key'];

					$parameters = "key=$api_key&placeid=$place_id";

					$url = "https://maps.googleapis.com/maps/api/place/details/json?$parameters";

					$response = wp_remote_post(
						$url,
						array(
							'method'      => 'POST',
							'timeout'     => 60,
							'httpversion' => '1.0',
						)
					);

					$google_status = '';
					$error_message = '';
					if ( ! is_wp_error( $response ) || wp_remote_retrieve_response_code( $response ) === 200 ) {
						$final_result  = json_decode( wp_remote_retrieve_body( $response ) );
						$result_status = $final_result->status;
						$error_message = $final_result->status;

						if ( 'OVER_QUERY_LIMIT' === $result_status ) {
							$google_status = 'exceeded';
						} elseif ( 'OK' === $result_status ) {
							$google_status = 'yes';
						} else {
							$google_status = 'no';
						}
					} else {
						$google_status = 'no';
					}

					if ( 'no' === $google_status ) {
						//$error = sprintf( esc_html__( 'Google Places API Key is invalid. Error: %s', 'max-addons' ), wp_remote_retrieve_response_message( $response ) );
						$error = sprintf( esc_html__( 'Google Places API Key is invalid. Error: %s', 'max-addons' ), $error_message );

						if ( ! empty( $error ) ) {
							wp_die( wp_kses_post( $error ), esc_html__( 'Google Places API', 'max-addons' ), array(
								'back_link' => true,
							) );
						}
					}
				}

				self::update_option( 'mab_google_places_api_key', trim( $_POST['mab_google_places_api_key'] ), false );
			}

			if ( isset( $_POST['mab_yelp_api_key'] ) ) {
				self::update_option( 'mab_yelp_api_key', trim( $_POST['mab_yelp_api_key'] ), false );
			}

			if ( isset( $_POST['mab_instagram_access_token'] ) ) {
				self::update_option( 'mab_instagram_access_token', trim( $_POST['mab_instagram_access_token'] ), false );
			}

			if ( isset( $_POST['mab_youtube_api_key'] ) ) {
				self::update_option( 'mab_youtube_api_key', trim( $_POST['mab_youtube_api_key'] ), false );
			}

			if ( isset( $_POST['mab_google_sheets_api_key'] ) ) {
				self::update_option( 'mab_google_sheets_api_key', trim( $_POST['mab_google_sheets_api_key'] ), false );
			}
		}
	}

	public static function save_elements() {
		if ( ! isset( $_POST['mab-elements-settings-nonce'] ) || ! wp_verify_nonce( $_POST['mab-elements-settings-nonce'], 'mab-elements-settings' ) ) {
			return;
		}

		if ( ! empty( $_POST['mab_enabled_elements'] ) ) {
			update_option( 'max_bricks_elements', $_POST['mab_enabled_elements'] );
		} else {
			update_option( 'max_bricks_elements', 'disabled' );
		}
	}

	public static function save_woo_builder_elements() {
		if ( ! isset( $_POST['mab-woo-builder-elements-settings-nonce'] ) || ! wp_verify_nonce( $_POST['mab-woo-builder-elements-settings-nonce'], 'mab-woo-builder-elements-settings' ) ) {
			return;
		}

		if ( ! empty( $_POST['mab_enabled_woo_builder_elements'] ) ) {
			update_option( 'max_bricks_woo_builder_elements', $_POST['mab_enabled_woo_builder_elements'] );
		} else {
			update_option( 'max_bricks_woo_builder_elements', 'disabled' );
		}
	}

	public static function save_conditions() {
		if ( ! isset( $_POST['mab-conditions-settings-nonce'] ) || ! wp_verify_nonce( $_POST['mab-conditions-settings-nonce'], 'mab-conditions-settings' ) ) {
			return;
		}

		if ( ! empty( $_POST['mab_enabled_conditions'] ) ) {
			update_option( 'max_bricks_conditions', $_POST['mab_enabled_conditions'] );
		} else {
			update_option( 'max_bricks_conditions', 'disabled' );
		}
	}

	public static function save_features() {
		if ( ! isset( $_POST['mab-features-settings-nonce'] ) || ! wp_verify_nonce( $_POST['mab-features-settings-nonce'], 'mab-features-settings' ) ) {
			return;
		}

		if ( ! empty( $_POST['mab_enabled_features'] ) ) {
			update_option( 'max_bricks_features', $_POST['mab_enabled_features'] );
		} else {
			update_option( 'max_bricks_features', 'disabled' );
		}
	}

	/**
	 * Saves the white label settings.
	 *
	 * @since 1.0.0
	 * @access private
	 * @return void
	 */
	private static function save_white_label() {
		if ( ! isset( $_POST['mab-wl-settings-nonce'] ) || ! wp_verify_nonce( $_POST['mab-wl-settings-nonce'], 'mab-wl-settings' ) ) {
			return;
		}

		$settings = self::get_settings();

		if ( isset( $_POST['mab_plugin_name'] ) ) {
			self::update_option( 'mab_plugin_name', sanitize_text_field( $_POST['mab_plugin_name'] ) );
			self::update_option( 'mab_plugin_desc', esc_textarea( $_POST['mab_plugin_desc'] ) );
			self::update_option( 'mab_plugin_author', sanitize_text_field( $_POST['mab_plugin_author'] ) );
			self::update_option( 'mab_plugin_uri', esc_url( $_POST['mab_plugin_uri'] ) );
		}
		$plugin_short_name      = isset( $_POST['mab_plugin_short_name'] ) ? sanitize_text_field( $_POST['mab_plugin_short_name'] ) : '';
		$admin_label            = isset( $_POST['mab_admin_label'] ) ? sanitize_text_field( $_POST['mab_admin_label'] ) : 'Max Addons';
		$support_link           = isset( $_POST['mab_support_link'] ) ? esc_url_raw( $_POST['mab_support_link'] ) : 'httsp://wpbricksaddons.com/contact/';
		$hide_integration_tab   = isset( $_POST['mab_hide_integration_tab'] ) ? absint( $_POST['mab_hide_integration_tab'] ) : 0;
		$hide_maintenance_tab   = isset( $_POST['mab_hide_maintenance_tab'] ) ? absint( $_POST['mab_hide_maintenance_tab'] ) : 0;
		$hide_wl_form           = isset( $_POST['mab_hide_wl_settings'] ) ? absint( $_POST['mab_hide_wl_settings'] ) : 0;
		$hide_plugin            = isset( $_POST['mab_hide_plugin'] ) ? absint( $_POST['mab_hide_plugin'] ) : 0;

		self::update_option( 'mab_plugin_short_name', $plugin_short_name );
		self::update_option( 'mab_admin_label', $admin_label );
		self::update_option( 'mab_support_link', $support_link );
		self::update_option( 'mab_hide_integration_tab', $hide_integration_tab );
		self::update_option( 'mab_hide_maintenance_tab', $hide_maintenance_tab );
		self::update_option( 'mab_hide_wl_settings', $hide_wl_form );
		self::update_option( 'mab_hide_plugin', $hide_plugin );
	}

	public static function migrate_settings() {
		if ( ! is_multisite() ) {
			return;
		}

		if ( 'yes' === get_option( 'mab_multisite_settings_migrated' ) ) {
			return;
		}

		$fields = array(
			'mab_license_status',
			'mab_license_key',
			'mab_elements_settings',
			'max_bricks_elements',
			'max_bricks_woo_builder_elements',
			'max_bricks_conditions',
			'max_bricks_features',
		);

		foreach ( $fields as $field ) {
			$value = get_site_option( $field );
			if ( $value ) {
				update_option( $field, $value );
			}
		}

		update_option( 'mab_multisite_settings_migrated', 'yes' );
	}

	/**
	* Refresh instagram token after 30 days.
	*
	* @since 2.5.2
	*/
	public static function refresh_instagram_access_token( $access_token = '', $widget_id = '' ) {
		if ( empty( $access_token ) ) {
			$access_token = trim( \MaxAddonsPro\Classes\Admin_Settings::get_option( 'mab_instagram_access_token' ) );
		}

		$updated_access_token = "mab_updated_instagram_access_token";
		
		if ( ! empty( $widget_id ) ) {
			$updated_access_token = "mab_updated_instagram_access_token_widget_$widget_id";
		}

		if ( empty( $access_token ) ) {
			return;
		}
	
		$updated = get_transient( $updated_access_token );

		if ( ! empty( $updated ) ) {
			return;
		}
	
		$endpoint_url = add_query_arg(
			[
				'access_token' => $access_token,
				'grant_type'   => 'ig_refresh_token',
			],
			'https://graph.instagram.com/refresh_access_token'
		);
	
		$response = wp_remote_get( $endpoint_url );
	
		if ( ! $response || 200 !== wp_remote_retrieve_response_code( $response ) || is_wp_error( $response ) ) {
			set_transient( $updated_access_token, 'error', DAY_IN_SECONDS );
			return;
		}
	
		$body = wp_remote_retrieve_body( $response );
	
		if ( ! $body ) {
			set_transient( $updated_access_token, 'error', DAY_IN_SECONDS );
			return;
		}
	
		$body = json_decode( $body, true );
	
		if ( empty( $body['access_token'] ) || empty( $body['expires_in'] ) ) {
			set_transient( $updated_access_token, 'error', DAY_IN_SECONDS );
			return;
		}
	
		set_transient( $updated_access_token, 'updated', 30 * DAY_IN_SECONDS );
	}
}

Admin_Settings::init();
