<?php
namespace MaxAddonsPro\Elements;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly
}

class Animated_Heading_Element extends \Bricks\Element {
	// Element properties
	public $category = 'max-addons-elements'; // Use predefined element category 'general'
	public $name     = 'max-animated-heading'; // Make sure to prefix your elements
	public $icon     = 'ti-layout-cta-btn-left max-element'; // Themify icon font class
	public $tag      = 'h3';
	public $scripts  = [ 'mabAnimatedHeading' ]; // Script(s) run when element is rendered on frontend or updated in builder

	// Return localized element label
	public function get_label() {
		return esc_html__( 'Animated Heading', 'max-addons' );
	}

	// Enqueue element styles and scripts
	public function enqueue_scripts() {
		wp_enqueue_style( 'mab-animated-heading' );
		wp_enqueue_script( 'mab-animated-heading' );
	}

	// Set builder control groups
	public function set_control_groups() {
		$this->control_groups['heading'] = [
			'title' => esc_html__( 'Heading', 'max-addons' ),
			'tab'   => 'content',
		];

		$this->control_groups['headingStyle'] = [
			'title' => esc_html__( 'Heading Style', 'max-addons' ),
			'tab'   => 'content',
		];

		$this->control_groups['marker'] = [
			'title' => esc_html__( 'Shape', 'max-addons' ),
			'tab'   => 'content',
		];

		$this->control_groups['settings'] = [
			'title' => esc_html__( 'Settings', 'max-addons' ),
			'tab'   => 'content',
		];
	}

	public function set_controls() {

		$this->set_heading_controls();
		$this->set_heading_style_controls();
		$this->set_marker_controls();
		$this->set_settings_controls();
	}

	// Set heading controls
	public function set_heading_controls() {
		$this->controls['headingStyle'] = [
			'tab'         => 'content',
			'group'       => 'heading',
			'label'       => esc_html__( 'Style', 'max-addons' ),
			'type'        => 'select',
			'options'     => [
				'highlight' => esc_html__( 'Highlighted', 'max-addons' ),
				'rotate'    => esc_html__( 'Rotating', 'max-addons' ),
			],
			'inline'      => true,
			'clearable'   => false,
			'pasteStyles' => false,
			'default'     => 'highlight',
		];

		$this->controls['marker'] = [
			'tab'         => 'content',
			'group'       => 'heading',
			'label'       => esc_html__( 'Shape', 'max-addons' ),
			'type'        => 'select',
			'options'     => [
				'circle'           => _x( 'Circle', 'Shapes', 'max-addons' ),
				'curly'            => _x( 'Curly', 'Shapes', 'max-addons' ),
				'underline'        => _x( 'Underline', 'Shapes', 'max-addons' ),
				'double'           => _x( 'Double', 'Shapes', 'max-addons' ),
				'double_underline' => _x( 'Double Underline', 'Shapes', 'max-addons' ),
				'underline_zigzag' => _x( 'Underline Zigzag', 'Shapes', 'max-addons' ),
				'diagonal'         => _x( 'Diagonal', 'Shapes', 'max-addons' ),
				'strikethrough'    => _x( 'Strikethrough', 'Shapes', 'max-addons' ),
				'rectangle'        => _x( 'Rectangle', 'Shapes', 'max-addons' ),
				'x'                => 'X',
			],
			'inline'      => true,
			'clearable'   => true,
			'pasteStyles' => false,
			'default'     => 'circle',
			'required'    => [ 'headingStyle', '=', 'highlight' ],
		];

		$this->controls['animationType'] = [
			'tab'         => 'content',
			'group'       => 'heading',
			'label'       => esc_html__( 'Animation', 'max-addons' ),
			'type'        => 'select',
			'options'     => [
				'typing'     => 'Typing',
				'clip'       => 'Clip',
				'flip'       => 'Flip',
				'swirl'      => 'Swirl',
				'blinds'     => 'Blinds',
				'drop-in'    => 'Drop-in',
				'wave'       => 'Wave',
				'slide'      => 'Slide',
				'slide-down' => 'Slide Down',
			],
			'inline'      => true,
			'clearable'   => true,
			'pasteStyles' => false,
			'default'     => 'typing',
			'required'    => [ 'headingStyle', '=', 'rotate' ],
		];

		$this->controls['prefix'] = [
			'tab'            => 'content',
			'group'          => 'heading',
			'label'          => esc_html__( 'Prefix', 'max-addons' ),
			'type'           => 'text',
			'default'        => esc_html__( 'Bricks is', 'max-addons' ),
			'hasDynamicData' => 'text',
		];

		$this->controls['suffix'] = [
			'tab'            => 'content',
			'group'          => 'heading',
			'label'          => esc_html__( 'Suffix', 'max-addons' ),
			'type'           => 'text',
			'default'        => '',
			'hasDynamicData' => 'text',
		];

		$this->controls['highlightedText'] = [
			'tab'            => 'content',
			'group'          => 'heading',
			'label'          => esc_html__( 'Highlighted text', 'max-addons' ),
			'type'           => 'text',
			'default'        => esc_html__( 'Awesome', 'max-addons' ),
			'hasDynamicData' => 'text',
			'required'       => [ 'headingStyle', '=', 'highlight' ],
		];

		$this->controls['rotatingText'] = [
			'tab'           => 'content',
			'group'         => 'heading',
			'label'         => esc_html__( 'Rotating text', 'max-addons' ),
			'type'          => 'repeater',
			'titleProperty' => 'text',
			'default'       => [
				[ 'text' => 'Awesome' ],
				[ 'text' => 'Amazing' ],
				[ 'text' => 'Cool' ],
			],
			'placeholder'   => esc_html__( 'Text block', 'max-addons' ),
			'fields'        => [
				'text' => [
					'label' => esc_html__( 'Text', 'max-addons' ),
					'type'  => 'text',
				],
			],
			'reloadScripts' => true,
			'required'      => [ 'headingStyle', '=', 'rotate' ],
		];

		$this->controls['tag'] = [
			'tab'         => 'content',
			'group'       => 'heading',
			'label'       => esc_html__( 'HTML tag', 'max-addons' ),
			'type'        => 'select',
			'options'     => [
				'h1'     => 'h1',
				'h2'     => 'h2',
				'h3'     => 'h3',
				'h4'     => 'h4',
				'h5'     => 'h5',
				'h6'     => 'h6',
				'custom' => esc_html__( 'Custom', 'max-addons' ),
			],
			'inline'      => true,
			'placeholder' => ! empty( $this->theme_styles['tag'] ) ? $this->theme_styles['tag'] : 'h3',
		];

		$this->controls['customTag'] = [
			'tab'         => 'content',
			'group'       => 'heading',
			'label'       => esc_html__( 'Custom tag', 'max-addons' ),
			'type'        => 'text',
			'inline'      => true,
			'placeholder' => 'div',
			'required'    => [ 'tag', '=', 'custom' ],
		];

		// Link
		$this->controls['linkSeparator'] = [
			'tab'   => 'content',
			'group' => 'heading',
			'label' => esc_html__( 'Link', 'max-addons' ),
			'type'  => 'separator',
		];

		$this->controls['link'] = [
			'tab'   => 'content',
			'group' => 'heading',
			'label' => esc_html__( 'Link type', 'max-addons' ),
			'type'  => 'link',
		];
	}

	// Set heading style controls
	public function set_heading_style_controls() {
		$this->controls['textTypography'] = [
			'tab'    => 'content',
			'group'  => 'headingStyle',
			'label'  => esc_html__( 'Text typography', 'max-addons' ),
			'type'   => 'typography',
			'css'    => [
				[
					'property' => 'font',
					'selector' => '.mab-headline-plain-text',
				],
			],
			'inline' => true,
			'small'  => true,
		];

		$this->controls['animatedTextTypography'] = [
			'tab'    => 'content',
			'group'  => 'headingStyle',
			'label'  => esc_html__( 'Animated text typography', 'max-addons' ),
			'type'   => 'typography',
			'css'    => [
				[
					'property' => 'font',
					'selector' => '.mab-headline-dynamic-text',
				],
			],
			'inline' => true,
			'small'  => true,
		];
	}

	// Set marker controls
	public function set_marker_controls() {
		$this->controls['markerColor'] = [
			'tab'    => 'content',
			'group'  => 'marker',
			'label'  => esc_html__( 'Color', 'max-addons' ),
			'type'   => 'color',
			'css'    => [
				[
					'property' => 'stroke',
					'selector' => '.mab-headline-dynamic-wrapper path',
				],
				[
					'property' => '--shape-color',
					'selector' => '',
				],
			],
			'inline' => true,
			'small'  => true,
		];

		$this->controls['strokeWidth'] = [
			'tab'   => 'content',
			'group' => 'marker',
			'label' => esc_html__( 'Width', 'max-addons' ),
			'type'  => 'number',
			'units' => true,
			'css'   => [
				[
					'property' => 'stroke-width',
					'selector' => '.mab-headline-dynamic-wrapper path',
				],
			],
		];
	}

	// Set settings controls
	public function set_settings_controls() {
		$this->controls['loop'] = [
			'tab'   => 'content',
			'group' => 'settings',
			'label' => esc_html__( 'Infinite loop', 'max-addons' ),
			'type'  => 'checkbox',
			'reset' => true,
		];

		$this->controls['highlightAnimationDuration'] = [
			'tab'      => 'content',
			'group'    => 'settings',
			'label'    => esc_html__( 'Duration', 'max-addons' ) . ' (ms)',
			'type'     => 'number',
			'unit'     => 'ms',
			'css'      => [
				[
					'property' => '--animation-duration',
					'selector' => '',
				],
			],
			'default'  => 1200,
			'required' => [ 'headingStyle', '=', 'highlight' ],
		];

		$this->controls['highlightIterationDelay'] = [
			'tab'      => 'content',
			'group'    => 'settings',
			'label'    => esc_html__( 'Delay', 'max-addons' ) . ' (ms)',
			'type'     => 'number',
			'units'    => false,
			'default'  => 8000,
			'required' => [
				[ 'headingStyle', '=', 'highlight' ],
				[ 'loop', '!=', '' ]
			],
		];

		$this->controls['rotateIterationDelay'] = [
			'tab'      => 'content',
			'group'    => 'settings',
			'label'    => esc_html__( 'Duration', 'max-addons' ) . ' (ms)',
			'type'     => 'number',
			'units'    => false,
			'default'  => 2500,
			'required' => [ 'headingStyle', '=', 'rotate' ],
		];
	}

	public function render() {
		$settings = $this->settings;
		$style    = ( ! empty( $settings['headingStyle'] ) ) ? $settings['headingStyle'] : 'highlight';

		$this->set_attribute( '_root', 'class', 'mab-headline--style-' . $style );

		if ( ! empty( $settings['marker'] ) && 'rectangle' === $settings['marker'] ) {
			$this->set_attribute( '_root', 'class', 'mab-headline-shape-rectangle' );
		}

		if ( ! empty( $settings['headingStyle'] ) && 'rotate' === $settings['headingStyle'] ) {
			$this->set_attribute( '_root', 'class', 'mab-headline-animation-type-' . $settings['animationType'] );
			
			$is_letter_animation = in_array( $settings['animationType'], [ 'typing', 'swirl', 'blinds', 'wave' ] );
			
			if ( $is_letter_animation ) {
				$this->set_attribute( '_root', 'class', 'mab-headline-letters' );
			}
		}

		$heading_settings = [];

		if ( ! empty( $settings['loop'] ) ) {
			$heading_settings['loop'] = $settings['loop'];
		}

		if ( ! empty( $settings['headingStyle'] ) ) {
			$heading_settings['style'] = $settings['headingStyle'];

			if ( 'highlight' === $settings['headingStyle'] ) {
				if ( ! empty( $settings['marker'] ) ) {
					$heading_settings['marker'] = $settings['marker'];
				}

				if ( ! empty( $settings['highlightAnimationDuration'] ) ) {
					$heading_settings['highlight_animation_duration'] = $settings['highlightAnimationDuration'];
				}

				if ( ! empty( $settings['highlightIterationDelay'] ) && ! empty( $settings['loop'] ) ) {
					$heading_settings['highlight_iteration_delay'] = $settings['highlightIterationDelay'];
				}
			}

			if ( 'rotate' === $settings['headingStyle'] ) {
				if ( ! empty( $settings['animationType'] ) ) {
					$heading_settings['animation_type'] = $settings['animationType'];
				}

				if ( ! empty( $settings['rotateIterationDelay'] ) ) {
					$heading_settings['iteration_delay'] = $settings['rotateIterationDelay'];
				}
			}
		}

		$this->set_attribute( '_root', 'data-settings', wp_json_encode( $heading_settings ) );
		$this->set_attribute( 'dynamic-wrapper', 'class', ['mab-headline-dynamic-wrapper', 'mab-headline-text-wrapper'] );
		$this->set_attribute( 'prefix', 'class', ['mab-headline-plain-text', 'mab-headline-text-wrapper'] );
		$this->set_attribute( 'suffix', 'class', ['mab-headline-plain-text', 'mab-headline-text-wrapper'] );
		$this->set_attribute( 'highlighted-text', 'class', ['mab-headline-dynamic-text', 'mab-headline-text-active'] );

		$output = "<{$this->tag} {$this->render_attributes( '_root' )}>";

		if ( isset( $settings['link'] ) ) {
			$this->set_link_attributes( 'a', $settings['link'] );
			$output .= "<a {$this->render_attributes( 'a' )}>";
		}

		if ( isset( $settings['prefix'] ) ) {
			$output .= '<span ' . $this->render_attributes( 'prefix' ) . '>' . wp_kses_post( $settings['prefix'] ) . ' </span>';
		}

		$output .= '<span ' . $this->render_attributes( 'dynamic-wrapper' ) . '>';

		if ( ! empty( $settings['headingStyle'] ) ) {
			if ( 'highlight' === $settings['headingStyle'] && ! empty( $settings['highlightedText'] ) ) {
				$output .= '<span ' . $this->render_attributes( 'highlighted-text' ) . '>' . wp_kses_post( $settings['highlightedText'] ) . '</span>';
			} elseif ( 'rotate' === $settings['headingStyle'] && isset( $settings['rotatingText'] ) && is_array( $settings['rotatingText'] ) && count( $settings['rotatingText'] ) ) {
				foreach ( $settings['rotatingText'] as $key => $string ) {
					if ( isset( $string['text'] ) && $string['text'] != '' ) {
						$active_class = ( 1 > $key ) ? ' mab-headline-text-active' : '';
						$output .= '<span class="mab-headline-dynamic-text' . $active_class . '">';
						$output .= $string['text'];
						$output .= '</span>';
					}
				}
			}
		}

		$output .= '</span>';

		if ( ! empty( $settings['suffix'] ) ) {
			$output .= '<span ' . $this->render_attributes( 'suffix' ) . '> ' . wp_kses_post( $settings['suffix'] ) . '</span>';
		}

		if ( isset( $settings['link'] ) ) {
			$output .= '</a>';
		}

		$output .= "</{$this->tag}>";

		echo $output;
	}
}
