<?php
namespace MaxAddonsPro\Elements;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly
}

class Copy_To_Clipboard_Element extends \Bricks\Element {
	// Element properties
	public $category = 'max-addons-elements'; // Use predefined element category 'general'
	public $name     = 'max-copy-to-clipboard'; // Make sure to prefix your elements
	public $icon     = 'ti-files max-element'; // Themify icon font class
	public $tag      = 'button';

	// Return localized element label
	public function get_label() {
		return esc_html__( 'Copy to Clipboard', 'max-addons' );
	}

	// Enqueue element styles and scripts
	public function enqueue_scripts() {
		wp_enqueue_style( 'mab-copy-to-clipboard' );
		wp_enqueue_script( 'mab-copy-to-clipboard' );
	}

	// Set builder control groups
	public function set_control_groups() {
		$this->control_groups['general'] = [
			'title' => esc_html__( 'General', 'max-addons' ),
			'tab'   => 'content',
		];

		$this->control_groups['button'] = [
			'title' => esc_html__( 'Button', 'max-addons' ),
			'tab'   => 'content',
		];

		$this->control_groups['buttonStyle'] = [
			'title' => esc_html__( 'Button Style', 'max-addons' ),
			'tab'   => 'content',
		];
	}

	public function set_controls() {

		// Set button width CSS selector
		$this->controls['_width']['css'][0]['selector'] = $this->css_selector;

		$this->set_general_controls();

		$this->set_button_controls();

		$this->set_button_style_controls();
	}

	public function set_general_controls() {
		$this->controls['copyTextFrom'] = [
			'tab'         => 'content',
			'group'       => 'general',
			'label'       => esc_html__( 'Copy text from', 'max-addons' ),
			'type'        => 'select',
			'options'     => [
				'text'     => esc_html__( 'Text Field', 'max-addons' ),
				'selector' => esc_html__( 'CSS Selector', 'max-addons' ),
			],
			'inline'      => true,
			'placeholder' => esc_html__( 'Text Field', 'max-addons' ),
			'default'     => 'text',
		];

		$this->controls['textToCopy'] = [
			'tab'            => 'content',
			'group'          => 'general',
			'label'          => esc_html__( 'Copy text', 'max-addons' ),
			'type'           => 'textarea',
			'default'        => esc_html__( 'This is copied text', 'max-addons' ),
			'hasDynamicData' => 'text',
			'inline'         => false,
			'required'       => [ 'copyTextFrom', '!=', 'selector' ],
		];

		$this->controls['textSelector'] = [
			'tab'            => 'content',
			'group'          => 'general',
			'label'          => esc_html__( 'CSS selector', 'max-addons' ),
			'description'    => esc_html__( 'Enter the CSS ID or class of the element containing the text to be copied', 'max-addons' ),
			'type'           => 'text',
			'default'        => esc_html__( '', 'max-addons' ),
			'hasDynamicData' => 'text',
			'inline'         => true,
			'required'       => [ 'copyTextFrom', '=', 'selector' ],
		];

		$this->controls['prefix'] = [
			'tab'            => 'content',
			'group'          => 'general',
			'label'          => esc_html__( 'Add prefix', 'max-addons' ),
			'description'    => esc_html__( 'Add text before copied content', 'max-addons' ),
			'type'           => 'text',
			'hasDynamicData' => 'text',
			'inline'         => true,
		];

		$this->controls['suffix'] = [
			'tab'            => 'content',
			'group'          => 'general',
			'label'          => esc_html__( 'Add suffix', 'max-addons' ),
			'description'    => esc_html__( 'Add text after copied content', 'max-addons' ),
			'type'           => 'text',
			'hasDynamicData' => 'text',
			'inline'         => true,
		];

		$this->controls['onCopySep'] = [
			'tab'   => 'content',
			'group' => 'general',
			'type'  => 'separator',
			'label' => esc_html__( 'On Copy', 'max-addons' ),
		];

		$this->controls['shakeButton'] = [
			'tab'   => 'content',
			'group' => 'general',
			'label' => esc_html__( 'Shake button', 'max-addons' ),
			'type'  => 'checkbox',
			'reset' => true,
		];

		$this->controls['changeText'] = [
			'tab'   => 'content',
			'group' => 'general',
			'label' => esc_html__( 'Change text', 'max-addons' ),
			'type'  => 'checkbox',
			'reset' => true,
		];

		$this->controls['textCopied'] = [
			'tab'            => 'content',
			'group'          => 'general',
			'label'          => esc_html__( 'Copied text', 'max-addons' ),
			'type'           => 'text',
			'default'        => esc_html__( 'Copied', 'max-addons' ),
			'hasDynamicData' => 'text',
			'inline'         => true,
			'required'       => [ 'changeText', '!=', '' ],
		];

		$this->controls['changeIcon'] = [
			'tab'     => 'content',
			'group'   => 'general',
			'label'   => esc_html__( 'Change icon', 'max-addons' ),
			'type'    => 'checkbox',
			'default' => true,
			'reset'   => true,
		];

		$this->controls['iconCopied'] = [
			'tab'      => 'content',
			'group'    => 'general',
			'label'    => esc_html__( 'Copied icon', 'max-addons' ),
			'type'     => 'icon',
			'default'  => [
				'library' => 'themify',
				'icon'    => 'ti-check',
			],
			'css' 	=> [
				[
					'selector' => '.icon-svg',
				],
			],
			'required' => [ 'changeIcon', '!=', '' ],
		];
	}

	public function set_button_controls() {
		$this->controls['text'] = [
			'tab'            => 'content',
			'group'          => 'button',
			'label'          => esc_html__( 'Button text', 'max-addons' ),
			'type'           => 'text',
			'default'        => esc_html__( 'Copy', 'max-addons' ),
			'hasDynamicData' => 'text',
			'inline'         => true,
		];

		$this->controls['size'] = [
			'tab'         => 'content',
			'group'       => 'button',
			'label'       => esc_html__( 'Size', 'max-addons' ),
			'type'        => 'select',
			'options'     => $this->control_options['buttonSizes'],
			'inline'      => true,
			'reset'       => true,
			'placeholder' => esc_html__( 'Medium', 'max-addons' ),
		];

		$this->controls['style'] = [
			'tab'         => 'content',
			'group'       => 'button',
			'label'       => esc_html__( 'Style', 'max-addons' ),
			'type'        => 'select',
			'options'     => $this->control_options['styles'],
			'inline'      => true,
			'reset'       => true,
			'default'     => 'primary',
			'placeholder' => esc_html__( 'None', 'max-addons' ),
		];

		$this->controls['circle'] = [
			'tab'   => 'content',
			'group' => 'button',
			'label' => esc_html__( 'Circle', 'max-addons' ),
			'type'  => 'checkbox',
			'reset' => true,
		];

		$this->controls['outline'] = [
			'tab'   => 'content',
			'group' => 'button',
			'label' => esc_html__( 'Outline', 'max-addons' ),
			'type'  => 'checkbox',
			'reset' => true,
		];

		$this->controls['buttonIconSep'] = [
			'tab'   => 'content',
			'group' => 'button',
			'type'  => 'separator',
			'label' => esc_html__( 'Icon', 'max-addons' ),
		];

		$this->controls['iconPosition'] = [
			'tab'         => 'content',
			'group'       => 'button',
			'label'       => esc_html__( 'Position', 'max-addons' ),
			'type'        => 'select',
			'options'     => [
				'left'  => esc_html__( 'Left', 'max-addons' ),
				'right' => esc_html__( 'Right', 'max-addons' ),
			],
			'inline'      => true,
			'placeholder' => esc_html__( 'Right', 'max-addons' ),
			'default'     => 'right',
		];

		$this->controls['icon'] = [
			'tab'      => 'content',
			'group'    => 'button',
			'label'    => esc_html__( 'Icon', 'max-addons' ),
			'type'     => 'icon',
			'default'  => [
				'library' => 'themify',
				'icon'    => 'ti-files',
			],
			'css' 	=> [
				[
					'selector' => '.icon-svg',
				],
			],
		];

		$this->controls['iconTypography'] = [
			'tab'      => 'content',
			'group'    => 'button',
			'label'    => esc_html__( 'Typography', 'max-addons' ),
			'type'     => 'typography',
			'css'      => [
				[
					'property' => 'font',
					'selector' => 'i',
				],
			],
			'exclude'  => [
				'font-family',
				'font-weight',
				'font-style',
				'text-align',
				'text-decoration',
				'text-transform',
				'line-height',
				'letter-spacing',
			],
			'inline'   => true,
			'small'    => true,
		];

		$this->controls['iconSpacing'] = [
			'tab'      => 'content',
			'group'    => 'button',
			'label'    => esc_html__( 'Gap', 'max-addons' ),
			'type'     => 'number',
			'units'    => true,
			'css'      => [
				[
					'property' => 'margin-right',
					'selector' => '.icon-left',
				],
				[
					'property' => 'margin-left',
					'selector' => '.icon-right',
				],
			],
			'default'  => '10px',
		];
	}

	public function set_button_style_controls() {
		$this->controls['buttonTypography'] = [
			'tab'    => 'content',
			'group'  => 'buttonStyle',
			'label'  => esc_html__( 'Typography', 'max-addons' ),
			'type'   => 'typography',
			'css'    => [
				[
					'property' => 'font',
					'selector' => '',
				],
			],
			'inline' => true,
		];

		$this->controls['buttonBackgroundColor'] = [
			'tab'    => 'content',
			'group'  => 'buttonStyle',
			'label'  => esc_html__( 'Background', 'max-addons' ),
			'type'   => 'color',
			'css'    => [
				[
					'property' => 'background-color',
					'selector' => '',
				],
			],
			'inline' => true,
			'small'  => true,
		];

		$this->controls['buttonBorder'] = [
			'tab'    => 'content',
			'group'  => 'buttonStyle',
			'label'  => esc_html__( 'Border', 'max-addons' ),
			'type'   => 'border',
			'inline' => true,
			'small'  => true,
			'css'    => [
				[
					'property' => 'border',
					'selector' => '',
				],
			],
		];

		$this->controls['buttonBoxShadow'] = [
			'tab'    => 'content',
			'group'  => 'buttonStyle',
			'label'  => esc_html__( 'Box Shadow', 'max-addons' ),
			'type'   => 'box-shadow',
			'css'    => [
				[
					'property' => 'box-shadow',
					'selector' => '',
				],
			],
			'inline' => true,
			'small'  => true,
		];

		$this->controls['copiedButtonSep'] = [
			'tab'   => 'content',
			'group' => 'buttonStyle',
			'type'  => 'separator',
			'label' => esc_html__( 'Copied Button', 'max-addons' ),
		];

		$this->controls['copiedButtonTypography'] = [
			'tab'    => 'content',
			'group'  => 'buttonStyle',
			'label'  => esc_html__( 'Typography', 'max-addons' ),
			'type'   => 'typography',
			'css'    => [
				[
					'property' => 'font',
					'selector' => '&.copying',
				],
			],
			'inline' => true,
		];

		$this->controls['copiedButtonBackgroundColor'] = [
			'tab'    => 'content',
			'group'  => 'buttonStyle',
			'label'  => esc_html__( 'Background', 'max-addons' ),
			'type'   => 'color',
			'css'    => [
				[
					'property' => 'background-color',
					'selector' => '&.copying',
				],
			],
			'inline' => true,
			'small'  => true,
		];

		$this->controls['copiedButtonBorder'] = [
			'tab'    => 'content',
			'group'  => 'buttonStyle',
			'label'  => esc_html__( 'Border', 'max-addons' ),
			'type'   => 'border',
			'inline' => true,
			'small'  => true,
			'css'    => [
				[
					'property' => 'border',
					'selector' => '&.copying',
				],
			],
		];

		$this->controls['copiedButtonBoxShadow'] = [
			'tab'    => 'content',
			'group'  => 'buttonStyle',
			'label'  => esc_html__( 'Box Shadow', 'max-addons' ),
			'type'   => 'box-shadow',
			'css'    => [
				[
					'property' => 'box-shadow',
					'selector' => '&.copying',
				],
			],
			'inline' => true,
			'small'  => true,
		];
	}

	public function render_button_icon( $icon ) {
		$settings    = $this->settings;
		$icon_copied = ! empty( $settings['iconCopied'] ) ? self::render_icon( $settings['iconCopied'] ) : false;

		$output = '<span class="mab-button-icon">';

		if ( ! empty( $settings['changeIcon'] ) && $icon_copied ) {
			$output .= '<span class="mab-button-icon-copied">';
			$output .= $icon_copied;
			$output .= '</span>';
		}

		$output .= '<span class="mab-button-icon-copy">';
		$output .= $icon;
		$output .= '</span>';
		$output .= '</span>';

		return $output; // Escaping will be handled in the calling function
	}

	public function render() {
		$settings = $this->settings;
		$options  = [];

		$this->set_attribute( '_root', 'class', 'bricks-button' );

		$copy_text_from = ( ! empty( $settings['copyTextFrom'] ) ) ? $settings['copyTextFrom'] : 'text';

		if ( 'selector' === $copy_text_from && ! empty( $settings['textSelector'] ) ) {
			$options['text_selector'] = $settings['textSelector'];
		} elseif ( 'text' === $copy_text_from && ! empty( $settings['textToCopy'] ) ) {
			$options['copy_text'] = $settings['textToCopy'];
		}

		if ( ! empty( $settings['prefix'] ) ) {
			$options['prefix'] = $settings['prefix'];
		}

		if ( ! empty( $settings['suffix'] ) ) {
			$options['suffix'] = $settings['suffix'];
		}

		if ( ! empty( $settings['changeText'] ) && ! empty( $settings['textCopied'] ) ) {
			$options['text_copied'] = $settings['textCopied'];
		}

		if ( ! empty( $settings['shakeButton'] ) ) {
			$options['shake_effect'] = true;
		}
	
		if ( ! empty( $options ) ) {
			$this->set_attribute( '_root', 'data-script-args', wp_json_encode( $options ) );
		}

		if ( ! empty( $settings['size'] ) ) {
			$this->set_attribute( '_root', 'class', $settings['size'] );
		}

		// Outline
		if ( isset( $settings['outline'] ) ) {
			$this->set_attribute( '_root', 'class', 'outline' );
		}

		if ( ! empty( $settings['style'] ) ) {
			$style_class = sanitize_html_class( "bricks-" . ( isset( $settings['outline'] ) ? "color-{$settings['style']}" : "background-{$settings['style']}" ) );
			$this->set_attribute( '_root', 'class', $style_class );
		}

		// Button circle
		if ( isset( $settings['circle'] ) ) {
			$this->set_attribute( '_root', 'class', 'circle' );
		}

		if ( isset( $settings['block'] ) ) {
			$this->set_attribute( '_root', 'class', 'block' );
		}

		$output = "<{$this->tag} {$this->render_attributes( '_root' )}>";

		$icon          = ! empty( $settings['icon'] ) ? self::render_icon( $settings['icon'] ) : false;
		$icon_position = ! empty( $settings['iconPosition'] ) ? $settings['iconPosition'] : 'right';

		if ( $icon && $icon_position === 'left' ) {
			$output .= $this->render_button_icon( $icon );
		}

		if ( isset( $settings['text'] ) ) {
			$this->set_attribute( 'button-text', 'class', 'mab-button-text' );

			$output .= "<span {$this->render_attributes( 'button-text' )}>";
			$output .= esc_html( trim( $settings['text'] ) );
			$output .= '</span>';
		}

		if ( $icon && $icon_position === 'right' ) {
			$output .= $this->render_button_icon( $icon );
		}

		$output .= "</{$this->tag}>";

		echo $output;
	}

	public static function render_builder() { ?>
		<script type="text/x-template" id="tmpl-bricks-element-max-copy-to-clipboard">
			<component
				:is="tag"
				:class="[
					'bricks-button',
					settings.size ? settings.size : null,
					settings.outline ? `outline bricks-color-${settings.style}` : null,
					settings.style ? `bricks-background-${settings.style}` : null,
					settings.circle ? 'circle' : null,
					settings.block ? 'block' : null,
					settings.icon && settings.iconPosition ? `icon-${settings.iconPosition}` : null,
					settings.icon && !settings.iconPosition ? 'icon-right' : null
				]">
				<icon-svg v-if="settings.iconPosition === 'left' && (settings?.icon?.icon || settings?.icon?.svg)" :iconSettings="settings.icon"/>
				<contenteditable tag="span" :name="name" controlKey="text" toolbar="style" :settings="settings"/>
				<icon-svg v-if="settings.iconPosition !== 'left' && (settings?.icon?.icon || settings?.icon?.svg)" :iconSettings="settings.icon"/>
			</component>
		</script>
		<?php
	}
}
