<?php
namespace MaxAddonsPro\Elements;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly
}

class Dynamic_Charts_Element extends \Bricks\Element {
	// Element properties
	public $category     = 'max-addons-elements';
	public $name         = 'max-dynamic-charts';
	public $icon         = 'ti-bar-chart max-element';
	public $css_selector = '';
	public $loop_index   = 0;
	public $scripts      = [ 'mabDynamicCharts' ]; // Script(s) run when element is rendered on frontend or updated in builder

	public function get_label() {
		return esc_html__( 'Dynamic Charts', 'max-addons' );
	}

	// Enqueue element styles and scripts
	public function enqueue_scripts() {
		wp_enqueue_script( 'chartjs' );
		wp_enqueue_script( 'mab-frontend' );
		wp_enqueue_script( 'mab-dynamic-charts' );
	}

	// Set builder control groups
	public function set_control_groups() {
		$this->control_groups['chart'] = [
			'title' => esc_html__( 'Chart', 'max-addons' ),
			'tab'   => 'content',
		];

		$this->control_groups['dataset'] = [
			'title' => esc_html__( 'Chart Data', 'max-addons' ),
			'tab'   => 'content',
		];

		$this->control_groups['gridOptions'] = [
			'title'    => esc_html__( 'Grid', 'max-addons' ),
			'tab'      => 'content',
			'required' => [ 'chartType', '!=', [ 'radar', 'pie', 'doughnut', 'polarArea' ] ],
		];

		$this->control_groups['legend'] = [
			'title' => esc_html__( 'Legend', 'max-addons' ),
			'tab'   => 'content',
		];

		$this->control_groups['labels'] = [
			'title'    => esc_html__( 'Labels', 'max-addons' ),
			'tab'      => 'content',
			'required' => [ 'chartType', '!=', [ 'radar', 'pie', 'doughnut', 'polarArea' ] ],
		];

		$this->control_groups['tooltip'] = [
			'title' => esc_html__( 'Tooltip', 'max-addons' ),
			'tab'   => 'content',
		];

		$this->control_groups['chartTitle'] = [
			'title' => esc_html__( 'Chart Title', 'max-addons' ),
			'tab'   => 'content',
		];

		$this->control_groups['pointStyles'] = [
			'title'    => esc_html__( 'Point Styles', 'max-addons' ),
			'tab'      => 'content',
			'required' => [ 'chartType', '=', [ 'line', 'radar', 'bubble' ] ],
		];

		$this->control_groups['animationOptions'] = [
			'title' => esc_html__( 'Animation', 'max-addons' ),
			'tab'   => 'content',
		];
	}

	public function set_controls() {
		$this->controls['_background']['css'][0]['selector'] = '';
		$this->controls['_gradient']['css'][0]['selector'] = '';

		$this->set_chart_controls();
		$this->set_dataset_controls();
		$this->set_grid_controls();
		$this->set_labels_controls();
		$this->set_legend_controls();
		$this->set_tooltip_controls();
		$this->set_chart_title_controls();
		$this->set_point_styles_controls();
		$this->set_animation_controls();
	}

	// Set items controls
	public function set_chart_controls() {
		$this->controls['chartType'] = [
			'tab'         => 'content',
			'group'       => 'chart',
			'label'       => esc_html__( 'Chart Type', 'max-addons' ),
			'type'        => 'select',
			'options'     => [
				'line'      => esc_html__( 'Line', 'max-addons' ),
				'bar'       => esc_html__( 'Bar', 'max-addons' ),
				'radar'     => esc_html__( 'Radar', 'max-addons' ),
				'pie'       => esc_html__( 'Pie', 'max-addons' ),
				'doughnut'  => esc_html__( 'Doughnut', 'max-addons' ),
				'polarArea' => esc_html__( 'Polar Area', 'max-addons' ),
				'bubble'    => esc_html__( 'Bubble', 'max-addons' ),
			],
			'inline'      => true,
			'clearable'   => false,
			'pasteStyles' => false,
			'placeholder' => esc_html__( 'Line', 'max-addons' ),
			'default'     => 'line',
		];

		$this->controls['steppedLine'] = [
			'tab'     => 'content',
			'group'   => 'chart',
			'label'   => esc_html__( 'Stepped Line Chart', 'max-addons' ),
			'type'    => 'checkbox',
			'inline'  => true,
			'reset'   => true,
			'default' => false,
			'required' => [ 'chartType', '=', [ '', 'line'] ],
		];

		$this->controls['tension'] = [
			'tab'         => 'content',
			'group'       => 'chart',
			'label'       => esc_html__( 'Line Tension', 'max-addons' ),
			'type'        => 'number',
			'min'         => 0,
			'max'         => 10,
			'step'        => 0.1,
			'inline'      => true,
			'required'    => [ 'chartType', '=', 'line' ],
		];

		$this->controls['lineBorderWidth'] = [
			'tab'         => 'content',
			'group'       => 'chart',
			'label'       => esc_html__( 'Line Border Width', 'max-addons' ),
			'type'        => 'number',
			'min'         => 0,
			'max'         => 10,
			'step'        => 1,
			'placeholder' => esc_html__( '3', 'max-addons' ),
			'inline'      => true,
			'required'    => [ 'chartType', '=', 'line' ],
		];

		$this->controls['radarBorderWidth'] = [
			'tab'         => 'content',
			'group'       => 'chart',
			'label'       => esc_html__( 'Line Border Width', 'max-addons' ),
			'type'        => 'number',
			'min'         => 0,
			'max'         => 10,
			'step'        => 1,
			'placeholder' => esc_html__( '3', 'max-addons' ),
			'inline'      => true,
			'required'    => [ 'chartType', '=', 'radar' ],
		];

		$this->controls['barChartType'] = [
			'tab'         => 'content',
			'group'       => 'chart',
			'label'       => esc_html__( 'Orientation', 'max-addons' ),
			'type'        => 'select',
			'options'     => [
				'vertical_bar'   => esc_html__( 'Vertical Bar', 'max-addons' ),
				'horizontal_bar' => esc_html__( 'Horizontal Bar', 'max-addons' ),
			],
			'inline'      => true,
			'clearable'   => true,
			'placeholder' => esc_html__( 'Vertical Bar', 'max-addons' ),
			'required'    => [ 'chartType', '=', 'bar' ],
		];

		$this->controls['stacked'] = [
			'tab'      => 'content',
			'group'    => 'chart',
			'label'    => esc_html__( 'Stacked', 'max-addons' ),
			'type'     => 'checkbox',
			'inline'   => true,
			'reset'    => true,
			'default'  => false,
			'required' => [ 'chartType', '=', 'bar' ],
		];

		$this->controls['barBorderWidth'] = [
			'tab'         => 'content',
			'group'       => 'chart',
			'label'       => esc_html__( 'Bar Border Width', 'max-addons' ),
			'type'        => 'number',
			'min'         => 0,
			'max'         => 10,
			'step'        => 1,
			'placeholder' => esc_html__( '1', 'max-addons' ),
			'inline'      => true,
			'required'    => [ 'chartType', '=', 'bar' ],
		];

		$this->controls['barBorderRadius'] = [
			'tab'      => 'content',
			'group'    => 'chart',
			'label'    => esc_html__( 'Border Radius', 'max-addons' ),
			'type'     => 'number',
			'min'      => 0,
			'max'      => 100,
			'step'     => 1,
			'inline'   => true,
			'required' => [ 'chartType', '=', 'bar' ],
		];

		$this->controls['maxbarthickness'] = [
			'tab'      => 'content',
			'group'    => 'chart',
			'label'    => esc_html__( 'Bar Size', 'max-addons' ),
			'type'     => 'number',
			'min'      => 1,
			'max'      => 50,
			'step'     => 1,
			'inline'   => true,
			'required' => [ 'chartType', '=', 'bar' ],
		];

		$this->controls['barthickness'] = [
			'tab'      => 'content',
			'group'    => 'chart',
			'label'    => esc_html__( 'Bar Space', 'max-addons' ),
			'type'     => 'number',
			'min'      => 1,
			'max'      => 50,
			'step'     => 1,
			'inline'   => true,
			'required' => [ 'chartType', '=', 'bar' ],
		];

		$this->controls['pieBorderWidth'] = [
			'tab'         => 'content',
			'group'       => 'chart',
			'label'       => esc_html__( 'Arc Border Width', 'max-addons' ),
			'type'        => 'number',
			'min'         => 0,
			'max'         => 10,
			'step'        => 1,
			'placeholder' => esc_html__( '1', 'max-addons' ),
			'inline'      => true,
			'required'    => [ 'chartType', '=', [ 'pie', 'doughnut', 'polarArea' ] ],
		];

		$this->controls['bubbleBorderWidth'] = [
			'tab'         => 'content',
			'group'       => 'chart',
			'label'       => esc_html__( 'Bubble Border Width', 'max-addons' ),
			'type'        => 'number',
			'min'         => 0,
			'max'         => 10,
			'step'        => 1,
			'placeholder' => esc_html__( '1', 'max-addons' ),
			'inline'      => true,
			'required'    => [ 'chartType', '=', 'bubble' ],
		];

		$this->controls['maintainAspectRatio'] = [
			'tab'     => 'content',
			'group'   => 'chart',
			'label'   => esc_html__( 'Maintain Aspect Ratio', 'max-addons' ),
			'type'    => 'checkbox',
			'inline'  => true,
			'reset'   => true,
			'default' => true,
		];

		$this->controls['chartHeight'] = [
			'tab'      => 'content',
			'group'    => 'chart',
			'label'    => esc_html__( 'Chart Height', 'max-addons' ),
			'type'     => 'number',
			'min'      => 100,
			'step'     => 1,
			'inline'   => true,
			'units'    => true,
			'rerender' => true,
			'default'  => '',
			'css'      => [
				[
					'selector' => '.mab-dynamic-charts',  
					'property' => 'height',
				],
			],
			'required' => [ 'maintainAspectRatio', '=', '' ],
		];

		$this->controls['chartWidth'] = [
			'tab'     => 'content',
			'group'   => 'chart',
			'label'   => esc_html__( 'Chart Width', 'max-addons' ),
			'type'    => 'number',
			'min'     => 100,
			'step'    => 1,
			'inline'  => true,
			'units'   => true,
			'default' => '500px',
			'css'     => [
				[
					'selector' => '',  
					'property' => 'width',
				],
			],
		];
	}

	public function set_dataset_controls() {
		$this->controls['source'] = [
			'tab'       => 'content',
			'group'     => 'dataset',
			'label'     => esc_html__( 'Source', 'max-addons' ),
			'type'      => 'select',
			'options'   => [
				'custom'        => esc_html__( 'Custom', 'max-addons' ),
				'query'         => esc_html__( 'Query', 'max-addons' ),
				'csv'           => esc_html__( 'CSV', 'max-addons' ),
				'google-sheets' => esc_html__( 'Google Sheets', 'max-addons' ),
			],
			'inline'    => true,
			'clearable' => false,
			'default'   => 'custom',
		];

		$this->controls['labels'] = [
			'tab'            => 'content',
			'group'          => 'dataset',
			'label'          => esc_html__( 'Labels', 'max-addons' ),
			'type'           => 'textarea',
			'default'        => esc_html__( 'January, February, March, April', 'max-addons' ),
			'description'    => esc_html__( 'Add labels separated by comma', 'max-addons' ),
			'hasDynamicData' => 'text',
			'required'       => [ 'source', '=', 'custom' ],
		];

		$this->controls['dynamicLabels'] = [
			'tab'            => 'content',
			'group'          => 'dataset',
			'label'          => esc_html__( 'Dynamic Labels', 'max-addons' ),
			'type'           => 'text',
			'hasDynamicData' => 'text',
			'required'       => [ 'source', '=', 'query' ],
		];

		$this->controls['query'] = [
			'tab'      => 'content',
			'group'    => 'dataset',
			'label'    => esc_html__( 'Query', 'max-addons' ),
			'type'     => 'query',
			'popup'    => true,
			'inline'   => true,
			'required' => [ 'source', '=', 'query' ],
		];

		$this->controls['csvSource'] = [
			'tab'       => 'content',
			'group'     => 'dataset',
			'label'     => esc_html__( 'CSV File Source', 'max-addons' ),
			'type'      => 'select',
			'options'   => [
				'url'  => esc_html__( 'File URL', 'max-addons' ),
				'text' => esc_html__( 'Textarea', 'max-addons' ),
			],
			'inline'    => true,
			'clearable' => false,
			'default'   => 'url',
			'required'  => [ 'source', '=', 'csv' ],
		];

		$this->controls['csvFileUrl'] = [
			'tab'      => 'content',
			'group'    => 'dataset',
			'label'    => esc_html__( 'CSV File URL', 'max-addons' ),
			'type'     => 'text',
			'required' => [
				[ 'source', '=', 'csv' ],
				[ 'csvSource', '=', 'url' ],
			],
		];

		$this->controls['csvFileText'] = [
			'tab'            => 'content',
			'group'          => 'dataset',
			'label'          => esc_html__( 'CSV Text', 'max-addons' ),
			'type'           => 'textarea',
			'description'    => esc_html__( 'Copy and paste the contents of CSV file here', 'max-addons' ),
			'hasDynamicData' => 'text',
			'required'       => [
				[ 'source', '=', 'csv' ],
				[ 'csvSource', '=', 'text' ],
			],
		];

		$this->controls['googleSheetsUrl'] = [
			'tab'      => 'content',
			'group'    => 'dataset',
			'label'    => esc_html__( 'Google Sheets URL', 'max-addons' ),
			'type'     => 'text',
			'required' => [ 'source', '=', 'google-sheets' ],
		];

		$this->controls['cellRange'] = [
			'tab'         => 'content',
			'group'       => 'dataset',
			'label'       => esc_html__( 'Table cell range', 'max-addons' ),
			'type'        => 'text',
			'default'     => 'Sheet1',
			'required'    => [ 'source', '=', 'google-sheets' ],
		];

		$this->controls['cacheDuration'] = [
			'tab'         => 'content',
			'group'       => 'dataset',
			'label'       => esc_html__( 'Refresh data after', 'max-addons' ),
			'type'        => 'select',
			'options'     => [
				'minute'    => esc_html__( '1 min', 'max-addons' ),
				'half_hour' => esc_html__( '30 minutes', 'max-addons' ),
				'hour'      => esc_html__( '1 hour', 'max-addons' ),
				'day'       => esc_html__( '1 day', 'max-addons' ),
				'week'      => esc_html__( '1 week', 'max-addons' ),
				'month'     => esc_html__( '1 month', 'max-addons' ),
				'year'      => esc_html__( '1 year', 'max-addons' ),
			],
			'placeholder' => esc_html__( '1 day', 'max-addons' ),
			'inline'      => true,
			'clearable'   => true,
			'required'    => [ 'source', '=', 'google-sheets' ],
		];

		$this->controls['chartDataset'] = [
			'tab'           => 'content',
			'group'         => 'dataset',
			'label'         => esc_html__( 'Dataset', 'max-addons' ),
			'type'          => 'repeater',
			'checkLoop'     => true,
			'placeholder'   => esc_html__( 'Data', 'max-addons' ),
			'titleProperty' => 'datasetLabel',
			'required'      => [
				[ 'source', '=', 'custom' ],
				[ 'chartType', '!=', 'bubble' ]
			],
			'fields'        => [
				'datasetLabel' => [
					'label'          => esc_html__( 'Label', 'max-addons' ),
					'type'           => 'text',
					'hasDynamicData' => 'text',
				],
				'datasetData' => [
					'label'          => esc_html__( 'Data', 'max-addons' ),
					'default'        => '40, 60, 10, 80',
					'description'    => esc_html__( 'Add data values separated by comma', 'max-addons' ),
					'type'           => 'text',
					'hasDynamicData' => 'text',
				],
				'backgroundColor' => [
					'label'    => esc_html__( 'Background Color', 'max-addons' ),
					'type'     => 'color',
					'inline'   => true,
				],
				'borderColor' => [
					'label'    => esc_html__( 'Border Color', 'max-addons' ),
					'type'     => 'color',
					'inline'   => true,
				],
				'fill' => [
					'label'       => esc_html__( 'Fill', 'max-addons' ),
					'description' => esc_html__( 'Fill option is supported by Line and Radar charts only', 'max-addons' ),
					'type'        => 'checkbox',
					'inline'      => true,
					'reset'       => true,
					'default'     => true,
				],
				'borderDash' => [
					'label'       => esc_html__( 'Border Dash', 'max-addons' ),
					'description' => esc_html__( 'Border Dash option is supported by Line and Radar charts only', 'max-addons' ),
					'type'        => 'checkbox',
					'inline'      => true,
					'reset'       => true,
					'default'     => false,
				],
				'backgroundColors' => [
					'label'       => esc_html__( 'Background Colors', 'max-addons' ),
					'description' => esc_html__( 'Background colors option is supported by Pie, Doughnut and Polar Area charts only', 'max-addons' ),
					'type'        => 'repeater',
					'placeholder' => esc_html__( 'Background Color', 'max-addons' ),
					'fields'      => [
						'bgColor' => [
							'label'    => esc_html__( 'Background Color', 'max-addons' ),
							'type'     => 'color',
							'inline'   => true,
						],
					]
				],
				'borderColors' => [
					'label'       => esc_html__( 'Border Colors', 'max-addons' ),
					'description' => esc_html__( 'Border colors option is supported by Pie, Doughnut and Polar Area charts only', 'max-addons' ),
					'type'        => 'repeater',
					'placeholder' => esc_html__( 'Border Color', 'max-addons' ),
					'fields'      => [
						'borderColor' => [
							'label'    => esc_html__( 'Border Color', 'max-addons' ),
							'type'     => 'color',
							'inline'   => true,
						],
					]
				],
			],
			'default'       => [
				[
					'datasetLabel'    => esc_html__( 'Data 1', 'max-addons' ),
					'datasetData'     => '40, 60, 10, 80',
					'backgroundColor' => [
						'hex' => '#EC6E8599',
					],
					'borderColor'     => [
						'hex' => '#EC6E85',
					],
				],
				[
					'datasetLabel'    => esc_html__( 'Data 2', 'max-addons' ),
					'datasetData'     => '5, 35, 55, 90',
					'backgroundColor' => [
						'hex' => '#569FE599',
					],
					'borderColor'     => [
						'hex' => '#569FE5',
					],
				],
				[
					'datasetLabel'    => esc_html__( 'Data 3', 'max-addons' ),
					'datasetData'     => '30, 25, 40, 5',
					'backgroundColor' => [
						'hex' => '#F7CF6B99',
					],
					'borderColor'     => [
						'hex' => '#F7CF6B',
					],
				],
			],
		];

		$this->controls['bubbleChartDataset'] = [
			'tab'           => 'content',
			'group'         => 'dataset',
			'label'         => esc_html__( 'Chart Data', 'max-addons' ),
			'type'          => 'repeater',
			'checkLoop'     => true,
			'placeholder'   => esc_html__( 'Chart Data', 'max-addons' ),
			'titleProperty' => 'datasetLabel',
			'required'      => [ 'chartType', '=', 'bubble' ],
			'fields'        => [
				'datasetLabel' => [
					'label'          => esc_html__( 'Label', 'max-addons' ),
					'type'           => 'text',
					'hasDynamicData' => 'text',
				],
				'bubble_data' => [
					'label'          => esc_html__( 'Data', 'max-addons' ),
					'default'        => '[5|15|15][10|18|12][7|14|14]',
					'description'    => esc_html__( 'Add data values separated by comma', 'max-addons' ),
					'type'           => 'textarea',
					'hasDynamicData' => 'text',
				],
			],
			'default'       => [
				[
					'datasetLabel' => esc_html__( 'Data 1', 'max-addons' ),
					'bubble_data'   => "[ 1, 10, 12 ]\n[ 3, 50, 10 ]\n[ 6, 18, 13 ]\n[ 7, 15, 12 ]",
				],
				[
					'datasetLabel' => esc_html__( 'Data 2', 'max-addons' ),
					'bubble_data'   => "[ 3, 28, 7 ]\n[ 5, 13, 11 ]\n[ 8, 36, 11 ]\n[ 9, 15, 6 ]",
				],
				[
					'datasetLabel' => esc_html__( 'Data 3', 'max-addons' ),
					'bubble_data'   => "[ 3, 18, 14 ]\n[ 8, 5, 17 ]\n[ 10, 10, 10 ]\n[ 18, 15, 15 ]",
				],
			],
		];

		$this->controls['dynamicChartDataset'] = [
			'tab'           => 'content',
			'group'         => 'dataset',
			'label'         => esc_html__( 'Dynamic Dataset', 'max-addons' ),
			'type'          => 'repeater',
			'checkLoop'     => true,
			'placeholder'   => esc_html__( 'Chart Data', 'max-addons' ),
			'titleProperty' => 'datasetLabel',
			'required'      => [ 'source', '=', 'query' ],
			'fields'        => [
				'datasetLabel' => [
					'label'          => esc_html__( 'Label', 'max-addons' ),
					'type'           => 'text',
					'hasDynamicData' => 'text',
				],
				'datasetData' => [
					'label'          => esc_html__( 'Data', 'max-addons' ),
					'default'        => '',
					'type'           => 'text',
					'hasDynamicData' => 'text',
				],
			],
		];

		$this->controls['dynamicColors'] = [
			'tab'           => 'content',
			'group'         => 'dataset',
			'label'         => esc_html__( 'Dataset Colors', 'max-addons' ),
			'type'          => 'repeater',
			'checkLoop'     => true,
			'placeholder'   => esc_html__( 'Color', 'max-addons' ),
			'titleProperty' => 'datasetLabel',
			'required'      => [
				[ 'source', '!=', 'custom' ],
				[ 'chartType', '!=', 'bubble' ]
			],
			'fields'        => [
				'datasetLabel' => [
					'label'          => esc_html__( 'Label', 'max-addons' ),
					'type'           => 'text',
					'hasDynamicData' => 'text',
				],
				'backgroundColor' => [
					'label'    => esc_html__( 'Background Color', 'max-addons' ),
					'type'     => 'color',
					'inline'   => true,
				],
				'borderColor' => [
					'label'    => esc_html__( 'Border Color', 'max-addons' ),
					'type'     => 'color',
					'inline'   => true,
				],
				'backgroundColors' => [
					'label'       => esc_html__( 'Background Colors', 'max-addons' ),
					'description' => esc_html__( 'Background colors option is supported by Pie, Doughnut and Polar Area charts only', 'max-addons' ),
					'type'        => 'repeater',
					'placeholder' => esc_html__( 'Background Color', 'max-addons' ),
					'fields'      => [
						'bgColor' => [
							'label'    => esc_html__( 'Background Color', 'max-addons' ),
							'type'     => 'color',
							'inline'   => true,
						],
					]
				],
				'borderColors' => [
					'label'       => esc_html__( 'Border Colors', 'max-addons' ),
					'description' => esc_html__( 'Border colors option is supported by Pie, Doughnut and Polar Area charts only', 'max-addons' ),
					'type'        => 'repeater',
					'placeholder' => esc_html__( 'Border Color', 'max-addons' ),
					'fields'      => [
						'borderColor' => [
							'label'    => esc_html__( 'Border Color', 'max-addons' ),
							'type'     => 'color',
							'inline'   => true,
						],
					]
				],
			],
		];
	}

	public function set_grid_controls() {
		$this->controls['xAxisSeparator'] = [
			'tab'      => 'content',
			'group'    => 'gridOptions',
			'label'    => esc_html__( 'X Axis', 'max-addons' ),
			'type'     => 'separator',
			'required' => [ 'chartType', '!=', [ 'radar', 'pie', 'doughnut', 'polarArea' ] ],
		];

		$this->controls['xAxisShowGridLines'] = [
			'tab'      => 'content',
			'group'    => 'gridOptions',
			'label'    => esc_html__( 'Show Grid Lines', 'max-addons' ),
			'type'     => 'checkbox',
			'inline'   => true,
			'reset'    => true,
			'default'  => true,
			'required' => [ 'chartType', '!=', [ 'radar', 'pie', 'doughnut', 'polarArea' ] ],
		];

		$this->controls['xAxisShowTitle'] = [
			'tab'      => 'content',
			'group'    => 'gridOptions',
			'label'    => esc_html__( 'Show Title', 'max-addons' ),
			'type'     => 'checkbox',
			'inline'   => true,
			'reset'    => true,
			'default'  => false,
			'required' => [ 'chartType', '!=', [ 'radar', 'pie', 'doughnut', 'polarArea' ] ],
		];

		$this->controls['xAxisTitle'] = [
			'tab'            => 'content',
			'group'          => 'gridOptions',
			'label'          => esc_html__( 'Title', 'max-addons' ),
			'type'           => 'text',
			'spellcheck'     => true,
			'inlineEditing'  => false,
			'hasDynamicData' => 'text',
			'inline'         => true,
			'required'       => [
				[ 'chartType', '!=', [ 'radar', 'pie', 'doughnut', 'polarArea' ] ],
				[ 'xAxisShowTitle', '!=', '' ]
			],
		];

		$this->controls['xAxisGridColor'] = [
			'tab'      => 'content',
			'group'    => 'gridOptions',
			'label'    => esc_html__( 'Grid Line Color', 'max-addons' ),
			'type'     => 'color',
			'inline'   => true,
			'default'  => '',
			'required' => [
				[ 'chartType', '!=', [ 'radar', 'pie', 'doughnut', 'polarArea' ] ],
				[ 'xAxisShowGridLines', '!=', '' ]
			],
		];

		$this->controls['xAxisGridWidth'] = [
			'tab'      => 'content',
			'group'    => 'gridOptions',
			'label'    => esc_html__( 'Grid Line Width', 'max-addons' ),
			'type'     => 'number',
			'min'      => 0,
			'max'      => 10,
			'step'     => 1,
			'inline'   => true,
			'required' => [
				[ 'chartType', '!=', [ 'radar', 'pie', 'doughnut', 'polarArea' ] ],
				[ 'xAxisShowGridLines', '!=', '' ]
			],
		];

		$this->controls['xAxisTitleColor'] = [
			'tab'      => 'content',
			'group'    => 'gridOptions',
			'label'    => esc_html__( 'Title Color', 'max-addons' ),
			'type'     => 'color',
			'inline'   => true,
			'default'  => '',
			'required' => [
				[ 'chartType', '!=', [ 'radar', 'pie', 'doughnut', 'polarArea' ] ],
				[ 'xAxisShowTitle', '!=', '' ]
			],
		];

		$this->controls['xAxisTitleFontSize'] = [
			'tab'      => 'content',
			'group'    => 'gridOptions',
			'label'    => esc_html__( 'Font Size', 'max-addons' ),
			'type'     => 'number',
			'step'     => 1,
			'inline'   => true,
			'required' => [
				[ 'chartType', '!=', [ 'radar', 'pie', 'doughnut', 'polarArea' ] ],
				[ 'xAxisShowTitle', '!=', '' ]
			],
		];

		$this->controls['xAxisTitleFontWeight'] = [
			'tab'         => 'content',
			'group'       => 'gridOptions',
			'label'       => esc_html__( 'Font Weight', 'max-addons' ),
			'type'        => 'select',
			'options'     => [
				'normal'  => esc_html__( 'Normal', 'max-addons' ),
				'italic'  => esc_html__( 'Italic', 'max-addons' ),
				'oblique' => esc_html__( 'Oblique', 'max-addons' ),
			],
			'inline'      => true,
			'clearable'   => true,
			'pasteStyles' => false,
			'placeholder' => esc_html__( 'Normal', 'max-addons' ),
			'required'    => [
				[ 'chartType', '!=', [ 'radar', 'pie', 'doughnut', 'polarArea' ] ],
				[ 'xAxisShowTitle', '!=', '' ]
			],
		];

		$this->controls['xAxisTitleFontStyle'] = [
			'tab'         => 'content',
			'group'       => 'gridOptions',
			'label'       => esc_html__( 'Font Style', 'max-addons' ),
			'type'        => 'select',
			'options'     => [
				'normal'  => esc_html__( 'Normal', 'max-addons' ),
				'italic'  => esc_html__( 'Italic', 'max-addons' ),
				'oblique' => esc_html__( 'Oblique', 'max-addons' ),
			],
			'inline'      => true,
			'clearable'   => true,
			'pasteStyles' => false,
			'placeholder' => esc_html__( 'Normal', 'max-addons' ),
			'required'    => [
				[ 'chartType', '!=', [ 'radar', 'pie', 'doughnut', 'polarArea' ] ],
				[ 'xAxisShowTitle', '!=', '' ]
			],
		];

		$this->controls['xAxisTitleLineHeight'] = [
			'tab'      => 'content',
			'group'    => 'gridOptions',
			'label'    => esc_html__( 'Line Height', 'max-addons' ),
			'type'     => 'number',
			'step'     => 1,
			'inline'   => true,
			'required' => [
				[ 'chartType', '!=', [ 'radar', 'pie', 'doughnut', 'polarArea' ] ],
				[ 'xAxisShowTitle', '!=', '' ]
			],
		];

		$this->controls['yAxisSeparator'] = [
			'tab'      => 'content',
			'group'    => 'gridOptions',
			'label'    => esc_html__( 'Y Axis', 'max-addons' ),
			'type'     => 'separator',
			'required' => [ 'chartType', '!=', [ 'radar', 'pie', 'doughnut', 'polarArea' ] ],
		];

		$this->controls['beginAtZero'] = [
			'tab'     => 'content',
			'group'   => 'gridOptions',
			'label'   => esc_html__( 'Begin at Zero', 'max-addons' ),
			'type'    => 'checkbox',
			'inline'  => true,
			'reset'   => true,
			'default' => true,
			'required' => [ 'chartType', '!=', [ 'radar', 'pie', 'doughnut', 'polarArea' ] ],
		];

		$this->controls['yAxisShowGridLines'] = [
			'tab'      => 'content',
			'group'    => 'gridOptions',
			'label'    => esc_html__( 'Show Grid Lines', 'max-addons' ),
			'type'     => 'checkbox',
			'inline'   => true,
			'reset'    => true,
			'default'  => true,
			'required' => [ 'chartType', '!=', [ 'radar', 'pie', 'doughnut', 'polarArea' ] ],
		];

		$this->controls['yAxisShowTitle'] = [
			'tab'      => 'content',
			'group'    => 'gridOptions',
			'label'    => esc_html__( 'Show Title', 'max-addons' ),
			'type'     => 'checkbox',
			'inline'   => true,
			'reset'    => true,
			'default'  => false,
			'required' => [ 'chartType', '!=', [ 'radar', 'pie', 'doughnut', 'polarArea' ] ],
		];

		$this->controls['yAxisTitle'] = [
			'tab'            => 'content',
			'group'          => 'gridOptions',
			'label'          => esc_html__( 'Title', 'max-addons' ),
			'type'           => 'text',
			'spellcheck'     => true,
			'inlineEditing'  => false,
			'hasDynamicData' => 'text',
			'inline'         => true,
			'required'       => [
				[ 'chartType', '!=', [ 'radar', 'pie', 'doughnut', 'polarArea' ] ],
				[ 'yAxisShowTitle', '!=', '' ]
			],
		];

		$this->controls['yAxisGridColor'] = [
			'tab'      => 'content',
			'group'    => 'gridOptions',
			'label'    => esc_html__( 'Grid Line Color', 'max-addons' ),
			'type'     => 'color',
			'inline'   => true,
			'default'  => '',
			'required' => [
				[ 'chartType', '!=', [ 'radar', 'pie', 'doughnut', 'polarArea' ] ],
				[ 'yAxisShowGridLines', '!=', '' ]
			],
		];

		$this->controls['yAxisGridWidth'] = [
			'tab'      => 'content',
			'group'    => 'gridOptions',
			'label'    => esc_html__( 'Grid Line Width', 'max-addons' ),
			'type'     => 'number',
			'min'      => 0,
			'max'      => 10,
			'step'     => 1,
			'inline'   => true,
			'required' => [
				[ 'chartType', '!=', [ 'radar', 'pie', 'doughnut', 'polarArea' ] ],
				[ 'yAxisShowGridLines', '!=', '' ]
			],
		];

		$this->controls['yAxisTitleColor'] = [
			'tab'      => 'content',
			'group'    => 'gridOptions',
			'label'    => esc_html__( 'Title Color', 'max-addons' ),
			'type'     => 'color',
			'inline'   => true,
			'default'  => '',
			'required' => [
				[ 'chartType', '!=', [ 'radar', 'pie', 'doughnut', 'polarArea' ] ],
				[ 'yAxisShowTitle', '!=', '' ]
			],
		];

		$this->controls['yAxisTitleFontSize'] = [
			'tab'      => 'content',
			'group'    => 'gridOptions',
			'label'    => esc_html__( 'Font Size', 'max-addons' ),
			'type'     => 'number',
			'step'     => 1,
			'inline'   => true,
			'required' => [
				[ 'chartType', '!=', [ 'radar', 'pie', 'doughnut', 'polarArea' ] ],
				[ 'yAxisShowTitle', '!=', '' ]
			],
		];

		$this->controls['yAxisTitleFontWeight'] = [
			'tab'         => 'content',
			'group'       => 'gridOptions',
			'label'       => esc_html__( 'Font Weight', 'max-addons' ),
			'type'        => 'select',
			'options'     => [
				'normal'  => esc_html__( 'Normal', 'max-addons' ),
				'italic'  => esc_html__( 'Italic', 'max-addons' ),
				'oblique' => esc_html__( 'Oblique', 'max-addons' ),
			],
			'inline'      => true,
			'clearable'   => true,
			'pasteStyles' => false,
			'placeholder' => esc_html__( 'Normal', 'max-addons' ),
			'required'    => [
				[ 'chartType', '!=', [ 'radar', 'pie', 'doughnut', 'polarArea' ] ],
				[ 'yAxisShowTitle', '!=', '' ]
			],
		];

		$this->controls['yAxisTitleFontStyle'] = [
			'tab'         => 'content',
			'group'       => 'gridOptions',
			'label'       => esc_html__( 'Font Style', 'max-addons' ),
			'type'        => 'select',
			'options'     => [
				'normal'  => esc_html__( 'Normal', 'max-addons' ),
				'italic'  => esc_html__( 'Italic', 'max-addons' ),
				'oblique' => esc_html__( 'Oblique', 'max-addons' ),
			],
			'inline'      => true,
			'clearable'   => true,
			'pasteStyles' => false,
			'placeholder' => esc_html__( 'Normal', 'max-addons' ),
			'required'    => [
				[ 'chartType', '!=', [ 'radar', 'pie', 'doughnut', 'polarArea' ] ],
				[ 'yAxisShowTitle', '!=', '' ]
			],
		];

		$this->controls['yAxisTitleLineHeight'] = [
			'tab'      => 'content',
			'group'    => 'gridOptions',
			'label'    => esc_html__( 'Line Height', 'max-addons' ),
			'type'     => 'number',
			'step'     => 1,
			'inline'   => true,
			'required' => [
				[ 'chartType', '!=', [ 'radar', 'pie', 'doughnut', 'polarArea' ] ],
				[ 'yAxisShowTitle', '!=', '' ]
			],
		];

		$this->controls['yAxisMin'] = [
			'tab'         => 'content',
			'group'       => 'gridOptions',
			'label'       => esc_html__( 'Minimum Value', 'max-addons' ),
			'description' => esc_html__( 'Set minimum value for Y-axis. This value is ignored when data has a smaller value.', 'max-addons' ),
			'type'        => 'number',
			'step'        => 1,
			'inline'      => true,
			'required'    => [ 'chartType', '!=', [ 'radar', 'pie', 'doughnut', 'polarArea' ] ],
		];

		$this->controls['yAxisMax'] = [
			'tab'         => 'content',
			'group'       => 'gridOptions',
			'label'       => esc_html__( 'Maximum Value', 'max-addons' ),
			'description' => esc_html__( 'Set maximum value for Y-axis. This value is ignored when data has a larger value.', 'max-addons' ),
			'type'        => 'number',
			'step'        => 1,
			'inline'      => true,
			'required'    => [ 'chartType', '!=', [ 'radar', 'pie', 'doughnut', 'polarArea' ] ],
		];
	}

	public function set_labels_controls() {
		$this->controls['xAxisLabelsSeparator'] = [
			'tab'      => 'content',
			'group'    => 'labels',
			'label'    => esc_html__( 'X Axis', 'max-addons' ),
			'type'     => 'separator',
			'required' => [ 'chartType', '!=', [ 'radar', 'pie', 'doughnut', 'polarArea' ] ],
		];

		$this->controls['xAxisShowLabels'] = [
			'tab'      => 'content',
			'group'    => 'labels',
			'label'    => esc_html__( 'Show Labels', 'max-addons' ),
			'type'     => 'checkbox',
			'inline'   => true,
			'reset'    => true,
			'default'  => true,
			'required' => [ 'chartType', '!=', [ 'radar', 'pie', 'doughnut', 'polarArea' ] ],
		];

		$this->controls['xAxisLabelsColor'] = [
			'tab'      => 'content',
			'group'    => 'labels',
			'label'    => esc_html__( 'Labels Color', 'max-addons' ),
			'type'     => 'color',
			'inline'   => true,
			'default'  => '',
			'required' => [
				[ 'chartType', '!=', [ 'radar', 'pie', 'doughnut', 'polarArea' ] ],
				[ 'xAxisShowLabels', '!=', '' ]
			],
		];

		$this->controls['xAxisLabelsFontSize'] = [
			'tab'      => 'content',
			'group'    => 'labels',
			'label'    => esc_html__( 'Font Size', 'max-addons' ),
			'type'     => 'number',
			'step'     => 1,
			'inline'   => true,
			'required' => [
				[ 'chartType', '!=', [ 'radar', 'pie', 'doughnut', 'polarArea' ] ],
				[ 'xAxisShowLabels', '!=', '' ]
			],
		];

		$this->controls['xAxisLabelsFontWeight'] = [
			'tab'         => 'content',
			'group'       => 'labels',
			'label'       => esc_html__( 'Font Weight', 'max-addons' ),
			'type'        => 'select',
			'options'     => [
				'normal'  => esc_html__( 'Normal', 'max-addons' ),
				'italic'  => esc_html__( 'Italic', 'max-addons' ),
				'oblique' => esc_html__( 'Oblique', 'max-addons' ),
			],
			'inline'      => true,
			'clearable'   => true,
			'pasteStyles' => false,
			'placeholder' => esc_html__( 'Normal', 'max-addons' ),
			'required'    => [
				[ 'chartType', '!=', [ 'radar', 'pie', 'doughnut', 'polarArea' ] ],
				[ 'xAxisShowLabels', '!=', '' ]
			],
		];

		$this->controls['xAxisLabelsFontStyle'] = [
			'tab'         => 'content',
			'group'       => 'labels',
			'label'       => esc_html__( 'Font Style', 'max-addons' ),
			'type'        => 'select',
			'options'     => [
				'normal'  => esc_html__( 'Normal', 'max-addons' ),
				'italic'  => esc_html__( 'Italic', 'max-addons' ),
				'oblique' => esc_html__( 'Oblique', 'max-addons' ),
			],
			'inline'      => true,
			'clearable'   => true,
			'pasteStyles' => false,
			'placeholder' => esc_html__( 'Normal', 'max-addons' ),
			'required'    => [
				[ 'chartType', '!=', [ 'radar', 'pie', 'doughnut', 'polarArea' ] ],
				[ 'xAxisShowLabels', '!=', '' ]
			],
		];

		$this->controls['xAxisLabelsLineHeight'] = [
			'tab'      => 'content',
			'group'    => 'labels',
			'label'    => esc_html__( 'Line Height', 'max-addons' ),
			'type'     => 'number',
			'step'     => 1,
			'inline'   => true,
			'required' => [
				[ 'chartType', '!=', [ 'radar', 'pie', 'doughnut', 'polarArea' ] ],
				[ 'xAxisShowLabels', '!=', '' ]
			],
		];

		$this->controls['yAxisLabelsSeparator'] = [
			'tab'      => 'content',
			'group'    => 'labels',
			'label'    => esc_html__( 'Y Axis', 'max-addons' ),
			'type'     => 'separator',
			'required' => [ 'chartType', '!=', [ 'radar', 'pie', 'doughnut', 'polarArea' ] ],
		];

		$this->controls['yAxisShowLabels'] = [
			'tab'      => 'content',
			'group'    => 'labels',
			'label'    => esc_html__( 'Show Labels', 'max-addons' ),
			'type'     => 'checkbox',
			'inline'   => true,
			'reset'    => true,
			'default'  => true,
			'required' => [ 'chartType', '!=', [ 'radar', 'pie', 'doughnut', 'polarArea' ] ],
		];

		$this->controls['y_axis_labels_prefix'] = [
			'tab'            => 'content',
			'group'          => 'labels',
			'label'          => esc_html__( 'Labels Prefix', 'max-addons' ),
			'type'           => 'text',
			'spellcheck'     => true,
			'inlineEditing'  => false,
			'hasDynamicData' => 'text',
			'inline'         => true,
			'required'       => [
				[ 'chartType', '!=', [ 'radar', 'pie', 'doughnut', 'polarArea' ] ],
				[ 'yAxisShowLabels', '!=', '' ]
			],
		];

		$this->controls['y_axis_labels_suffix'] = [
			'tab'            => 'content',
			'group'          => 'labels',
			'label'          => esc_html__( 'Labels Suffix', 'max-addons' ),
			'type'           => 'text',
			'spellcheck'     => true,
			'inlineEditing'  => false,
			'hasDynamicData' => 'text',
			'inline'         => true,
			'required'       => [
				[ 'chartType', '!=', [ 'radar', 'pie', 'doughnut', 'polarArea' ] ],
				[ 'yAxisShowLabels', '!=', '' ]
			],
		];

		$this->controls['yAxisLabelsColor'] = [
			'tab'      => 'content',
			'group'    => 'labels',
			'label'    => esc_html__( 'Labels Color', 'max-addons' ),
			'type'     => 'color',
			'inline'   => true,
			'default'  => '',
			'required' => [
				[ 'chartType', '!=', [ 'radar', 'pie', 'doughnut', 'polarArea' ] ],
				[ 'yAxisShowLabels', '!=', '' ]
			],
		];

		$this->controls['yAxisLabelsFontSize'] = [
			'tab'      => 'content',
			'group'    => 'labels',
			'label'    => esc_html__( 'Font Size', 'max-addons' ),
			'type'     => 'number',
			'step'     => 1,
			'inline'   => true,
			'required' => [
				[ 'chartType', '!=', [ 'radar', 'pie', 'doughnut', 'polarArea' ] ],
				[ 'yAxisShowLabels', '!=', '' ]
			],
		];

		$this->controls['yAxisLabelsFontWeight'] = [
			'tab'         => 'content',
			'group'       => 'labels',
			'label'       => esc_html__( 'Font Weight', 'max-addons' ),
			'type'        => 'select',
			'options'     => [
				'normal'  => esc_html__( 'Normal', 'max-addons' ),
				'italic'  => esc_html__( 'Italic', 'max-addons' ),
				'oblique' => esc_html__( 'Oblique', 'max-addons' ),
			],
			'inline'      => true,
			'clearable'   => true,
			'pasteStyles' => false,
			'placeholder' => esc_html__( 'Normal', 'max-addons' ),
			'required'    => [
				[ 'chartType', '!=', [ 'radar', 'pie', 'doughnut', 'polarArea' ] ],
				[ 'yAxisShowLabels', '!=', '' ]
			],
		];

		$this->controls['yAxisLabelsFontStyle'] = [
			'tab'         => 'content',
			'group'       => 'labels',
			'label'       => esc_html__( 'Font Style', 'max-addons' ),
			'type'        => 'select',
			'options'     => [
				'normal'  => esc_html__( 'Normal', 'max-addons' ),
				'italic'  => esc_html__( 'Italic', 'max-addons' ),
				'oblique' => esc_html__( 'Oblique', 'max-addons' ),
			],
			'inline'      => true,
			'clearable'   => true,
			'pasteStyles' => false,
			'placeholder' => esc_html__( 'Normal', 'max-addons' ),
			'required'    => [
				[ 'chartType', '!=', [ 'radar', 'pie', 'doughnut', 'polarArea' ] ],
				[ 'yAxisShowLabels', '!=', '' ]
			],
		];

		$this->controls['yAxisLabelsLineHeight'] = [
			'tab'      => 'content',
			'group'    => 'labels',
			'label'    => esc_html__( 'Line Height', 'max-addons' ),
			'type'     => 'number',
			'step'     => 1,
			'inline'   => true,
			'required' => [
				[ 'chartType', '!=', [ 'radar', 'pie', 'doughnut', 'polarArea' ] ],
				[ 'yAxisShowLabels', '!=', '' ]
			],
		];
	}

	public function set_legend_controls() {
		$this->controls['showLegend'] = [
			'tab'     => 'content',
			'group'   => 'legend',
			'label'   => esc_html__( 'Show Legend', 'max-addons' ),
			'type'    => 'checkbox',
			'inline'  => true,
			'reset'   => true,
			'default' => true,
		];

		$this->controls['legendReverse'] = [
			'tab'      => 'content',
			'group'    => 'legend',
			'label'    => esc_html__( 'Reverse', 'max-addons' ),
			'type'     => 'checkbox',
			'inline'   => true,
			'reset'    => true,
			'required' => [ 'showLegend', '!=', '' ],
		];

		$this->controls['legendPosition'] = [
			'tab'         => 'content',
			'group'       => 'legend',
			'label'       => esc_html__( 'Position', 'max-addons' ),
			'type'        => 'select',
			'options'     => [
				'left'   => esc_html__( 'Left', 'max-addons' ),
				'top'    => esc_html__( 'Top', 'max-addons' ),
				'bottom' => esc_html__( 'Bottom', 'max-addons' ),
				'right'  => esc_html__( 'Right', 'max-addons' ),
			],
			'inline'      => true,
			'clearable'   => true,
			'placeholder' => esc_html__( 'Top', 'max-addons' ),
			'required'    => [ 'showLegend', '!=', '' ],
		];

		$this->controls['legendAlign'] = [
			'tab'         => 'content',
			'group'       => 'legend',
			'label'       => esc_html__( 'Alignment', 'max-addons' ),
			'type'        => 'select',
			'options'     => [
				'start'  => esc_html__( 'Start', 'max-addons' ),
				'center' => esc_html__( 'Center', 'max-addons' ),
				'end'    => esc_html__( 'End', 'max-addons' ),
			],
			'inline'      => true,
			'clearable'   => true,
			'placeholder' => esc_html__( 'Center', 'max-addons' ),
			'required'    => [ 'showLegend', '!=', '' ],
		];

		$this->controls['legendColor'] = [
			'tab'      => 'content',
			'group'    => 'legend',
			'label'    => esc_html__( 'Label Color', 'max-addons' ),
			'type'     => 'color',
			'inline'   => true,
			'default'  => '',
			'required' => [ 'showLegend', '!=', '' ],
		];

		$this->controls['legendFontSize'] = [
			'tab'      => 'content',
			'group'    => 'legend',
			'label'    => esc_html__( 'Font Size', 'max-addons' ),
			'type'     => 'number',
			'step'     => 1,
			'inline'   => true,
			'required' => [ 'showLegend', '!=', '' ],
		];

		$this->controls['legendFontWeight'] = [
			'tab'         => 'content',
			'group'       => 'legend',
			'label'       => esc_html__( 'Font Weight', 'max-addons' ),
			'type'        => 'select',
			'options'     => [
				'normal'  => esc_html__( 'Normal', 'max-addons' ),
				'italic'  => esc_html__( 'Italic', 'max-addons' ),
				'oblique' => esc_html__( 'Oblique', 'max-addons' ),
			],
			'inline'      => true,
			'clearable'   => true,
			'pasteStyles' => false,
			'placeholder' => esc_html__( 'Normal', 'max-addons' ),
			'required'    => [ 'showLegend', '!=', '' ],
		];

		$this->controls['legendFontStyle'] = [
			'tab'         => 'content',
			'group'       => 'legend',
			'label'       => esc_html__( 'Font Style', 'max-addons' ),
			'type'        => 'select',
			'options'     => [
				'normal'  => esc_html__( 'Normal', 'max-addons' ),
				'italic'  => esc_html__( 'Italic', 'max-addons' ),
				'oblique' => esc_html__( 'Oblique', 'max-addons' ),
			],
			'inline'      => true,
			'clearable'   => true,
			'pasteStyles' => false,
			'placeholder' => esc_html__( 'Normal', 'max-addons' ),
			'required'    => [ 'showLegend', '!=', '' ],
		];

		$this->controls['legendLineHeight'] = [
			'tab'      => 'content',
			'group'    => 'legend',
			'label'    => esc_html__( 'Line Height', 'max-addons' ),
			'type'     => 'number',
			'step'     => 1,
			'inline'   => true,
			'required' => [ 'showLegend', '!=', '' ],
		];
	}

	public function set_tooltip_controls() {
		$this->controls['showTooltip'] = [
			'tab'     => 'content',
			'group'   => 'tooltip',
			'label'   => esc_html__( 'Show Tooltip', 'max-addons' ),
			'type'    => 'checkbox',
			'inline'  => true,
			'reset'   => true,
			'default' => true,
		];

		$this->controls['tooltipEvent'] = [
			'tab'         => 'content',
			'group'       => 'tooltip',
			'label'       => esc_html__( 'Tooltip Event', 'max-addons' ),
			'type'        => 'select',
			'options'     => [
				'hover' => esc_html__( 'Hover', 'max-addons' ),
				'click' => esc_html__( 'Click', 'max-addons' ),
			],
			'inline'      => true,
			'clearable'   => true,
			'placeholder' => esc_html__( 'Hover', 'max-addons' ),
			'required'    => [ 'showTooltip', '!=', '' ],
		];

		$this->controls['interactionMode'] = [
			'tab'         => 'content',
			'group'       => 'tooltip',
			'label'       => esc_html__( 'Tooltip Mode', 'max-addons' ),
			'type'        => 'select',
			'options'     => [
				'index'   => esc_html__( 'Index', 'max-addons' ),
				'point'   => esc_html__( 'Point', 'max-addons' ),
				'dataset' => esc_html__( 'Dataset', 'max-addons' ),
			],
			'inline'      => true,
			'clearable'   => true,
			'placeholder' => esc_html__( 'Point', 'max-addons' ),
			'required'    => [ 'showTooltip', '!=', '' ],
		];

		$this->controls['tooltipBackgroundColor'] = [
			'tab'      => 'content',
			'group'    => 'tooltip',
			'label'    => esc_html__( 'Background Color', 'max-addons' ),
			'type'     => 'color',
			'inline'   => true,
			'required' => [ 'showTooltip', '!=', '' ],
		];

		$this->controls['tooltipBorderColor'] = [
			'tab'      => 'content',
			'group'    => 'tooltip',
			'label'    => esc_html__( 'Border Color', 'max-addons' ),
			'type'     => 'color',
			'inline'   => true,
			'required' => [ 'showTooltip', '!=', '' ],
		];

		$this->controls['tooltipBorderWidth'] = [
			'tab'      => 'content',
			'group'    => 'tooltip',
			'label'    => esc_html__( 'Border Width', 'max-addons' ),
			'type'     => 'number',
			'min'      => 0,
			'max'      => 10,
			'step'     => 1,
			'inline'   => true,
			'required' => [ 'showTooltip', '!=', '' ],
		];

		$this->controls['tooltipBorderRadius'] = [
			'tab'      => 'content',
			'group'    => 'tooltip',
			'label'    => esc_html__( 'Border Radius', 'max-addons' ),
			'type'     => 'number',
			'min'      => 0,
			'max'      => 100,
			'step'     => 1,
			'inline'   => true,
			'required' => [ 'showTooltip', '!=', '' ],
		];

		$this->controls['tooltipPadding'] = [
			'tab'         => 'content',
			'group'       => 'tooltip',
			'label'       => esc_html__( 'Padding', 'max-addons' ),
			'type'        => 'number',
			'min'         => 0,
			'max'         => 10,
			'step'        => 1,
			'inline'      => true,
			'placeholder' => esc_html__( '6', 'max-addons' ),
			'required'    => [ 'showTooltip', '!=', '' ],
		];

		$this->controls['tooltipTitleSeparator'] = [
			'tab'      => 'content',
			'group'    => 'tooltip',
			'label'    => esc_html__( 'Title', 'max-addons' ),
			'type'     => 'separator',
			'required' => [ 'showTooltip', '!=', '' ],
		];

		$this->controls['tooltipTitleColor'] = [
			'tab'      => 'content',
			'group'    => 'tooltip',
			'label'    => esc_html__( 'Color', 'max-addons' ),
			'type'     => 'color',
			'inline'   => true,
			'required' => [ 'showTooltip', '!=', '' ],
		];

		$this->controls['tooltipTitleFontSize'] = [
			'tab'         => 'content',
			'group'       => 'tooltip',
			'label'       => esc_html__( 'Font Size', 'max-addons' ),
			'type'        => 'number',
			'min'         => 1,
			'max'         => 50,
			'step'        => 1,
			'inline'      => true,
			'placeholder' => esc_html__( '12', 'max-addons' ),
			'required'    => [ 'showTooltip', '!=', '' ],
		];

		$this->controls['tooltipBodySeparator'] = [
			'tab'      => 'content',
			'group'    => 'tooltip',
			'label'    => esc_html__( 'Body', 'max-addons' ),
			'type'     => 'separator',
			'required' => [ 'showTooltip', '!=', '' ],
		];

		$this->controls['tooltipBodyColor'] = [
			'tab'      => 'content',
			'group'    => 'tooltip',
			'label'    => esc_html__( 'Color', 'max-addons' ),
			'type'     => 'color',
			'inline'   => true,
			'required' => [ 'showTooltip', '!=', '' ],
		];

		$this->controls['tooltipBodyFontSize'] = [
			'tab'         => 'content',
			'group'       => 'tooltip',
			'label'       => esc_html__( 'Font Size', 'max-addons' ),
			'type'        => 'number',
			'min'         => 1,
			'max'         => 50,
			'step'        => 1,
			'inline'      => true,
			'placeholder' => esc_html__( '12', 'max-addons' ),
			'required'    => [ 'showTooltip', '!=', '' ],
		];
	}

	public function set_chart_title_controls() {
		$this->controls['showChartTitle'] = [
			'tab'     => 'content',
			'group'   => 'chartTitle',
			'label'   => esc_html__( 'Show Chart Title', 'max-addons' ),
			'type'    => 'checkbox',
			'inline'  => true,
			'reset'   => true,
		];

		$this->controls['chartTitle'] = [
			'tab'            => 'content',
			'group'          => 'chartTitle',
			'label'          => esc_html__( 'Chart Title', 'max-addons' ),
			'type'           => 'text',
			'spellcheck'     => true,
			'inlineEditing'  => false,
			'hasDynamicData' => 'text',
			'inline'         => true,
			'required'       => [ 'showChartTitle', '!=', '' ],
		];

		$this->controls['chartTitlePosition'] = [
			'tab'         => 'content',
			'group'       => 'chartTitle',
			'label'       => esc_html__( 'Position', 'max-addons' ),
			'type'        => 'select',
			'options'     => [
				'left'   => esc_html__( 'Left', 'max-addons' ),
				'top'    => esc_html__( 'Top', 'max-addons' ),
				'bottom' => esc_html__( 'Bottom', 'max-addons' ),
				'right'  => esc_html__( 'Right', 'max-addons' ),
			],
			'inline'      => true,
			'clearable'   => true,
			'placeholder' => esc_html__( 'Top', 'max-addons' ),
			'required'    => [ 'showChartTitle', '!=', '' ],
		];

		$this->controls['chartTitleAlign'] = [
			'tab'         => 'content',
			'group'       => 'chartTitle',
			'label'       => esc_html__( 'Alignment', 'max-addons' ),
			'type'        => 'select',
			'options'     => [
				'start'  => esc_html__( 'Start', 'max-addons' ),
				'center' => esc_html__( 'Center', 'max-addons' ),
				'end'    => esc_html__( 'End', 'max-addons' ),
			],
			'inline'      => true,
			'clearable'   => true,
			'placeholder' => esc_html__( 'Center', 'max-addons' ),
			'required'    => [ 'showChartTitle', '!=', '' ],
		];

		$this->controls['chartTitleColor'] = [
			'tab'      => 'content',
			'group'    => 'chartTitle',
			'label'    => esc_html__( 'Color', 'max-addons' ),
			'type'     => 'color',
			'inline'   => true,
			'required' => [ 'showChartTitle', '!=', '' ],
		];

		$this->controls['chartTitleFontSize'] = [
			'tab'      => 'content',
			'group'    => 'chartTitle',
			'label'    => esc_html__( 'Font Size', 'max-addons' ),
			'type'     => 'number',
			'step'     => 1,
			'inline'   => true,
			'required' => [ 'showChartTitle', '!=', '' ],
		];

		$this->controls['chartTitleFontWeight'] = [
			'tab'         => 'content',
			'group'       => 'chartTitle',
			'label'       => esc_html__( 'Font Weight', 'max-addons' ),
			'type'        => 'select',
			'options'     => [
				'normal'  => esc_html__( 'Normal', 'max-addons' ),
				'italic'  => esc_html__( 'Italic', 'max-addons' ),
				'oblique' => esc_html__( 'Oblique', 'max-addons' ),
			],
			'inline'      => true,
			'clearable'   => true,
			'pasteStyles' => false,
			'placeholder' => esc_html__( 'Normal', 'max-addons' ),
			'required'    => [ 'showChartTitle', '!=', '' ],
		];

		$this->controls['chartTitleFontStyle'] = [
			'tab'         => 'content',
			'group'       => 'chartTitle',
			'label'       => esc_html__( 'Font Style', 'max-addons' ),
			'type'        => 'select',
			'options'     => [
				'normal'  => esc_html__( 'Normal', 'max-addons' ),
				'italic'  => esc_html__( 'Italic', 'max-addons' ),
				'oblique' => esc_html__( 'Oblique', 'max-addons' ),
			],
			'inline'      => true,
			'clearable'   => true,
			'pasteStyles' => false,
			'placeholder' => esc_html__( 'Normal', 'max-addons' ),
			'required'    => [ 'showChartTitle', '!=', '' ],
		];

		$this->controls['chartTitleLineHeight'] = [
			'tab'      => 'content',
			'group'    => 'chartTitle',
			'label'    => esc_html__( 'Line Height', 'max-addons' ),
			'type'     => 'number',
			'step'     => 1,
			'inline'   => true,
			'required' => [ 'showChartTitle', '!=', '' ],
		];
	}

	public function set_point_styles_controls() {
		$this->controls['customPointStyles'] = [
			'tab'     => 'content',
			'group'   => 'pointStyles',
			'label'   => esc_html__( 'Custom Point Styles', 'max-addons' ),
			'type'    => 'checkbox',
			'inline'  => true,
			'reset'   => true,
			'default' => false,
			'required' => [ 'chartType', '=', [ 'line', 'radar', 'bubble' ] ],
		];

		$this->controls['pointStyle'] = [
			'tab'         => 'content',
			'group'       => 'pointStyles',
			'label'       => esc_html__( 'Point Style', 'max-addons' ),
			'type'        => 'select',
			'options'     => [
				'circle'      => esc_html__( 'Circle', 'max-addons' ),
				'cross'       => esc_html__( 'Cross', 'max-addons' ),
				'crossRot'    => esc_html__( 'CrossRot', 'max-addons' ),
				'dash'        => esc_html__( 'Dash', 'max-addons' ),
				'line'        => esc_html__( 'Line', 'max-addons' ),
				'rect'        => esc_html__( 'Rect', 'max-addons' ),
				'rectRounded' => esc_html__( 'RectRounded', 'max-addons' ),
				'rectRot'     => esc_html__( 'RectRot', 'max-addons' ),
				'star'        => esc_html__( 'Star', 'max-addons' ),
				'triangle'    => esc_html__( 'Triangle', 'max-addons' ),
			],
			'inline'      => true,
			'clearable'   => true,
			'pasteStyles' => false,
			'placeholder' => esc_html__( 'Circle', 'max-addons' ),
			'required'    => [
				[ 'chartType', '=', [ 'line', 'radar', 'bubble' ] ],
				[ 'customPointStyles', '!=', '' ]
			],
		];

		$this->controls['pointBackground'] = [
			'tab'      => 'content',
			'group'    => 'pointStyles',
			'label'    => esc_html__( 'Background Color', 'max-addons' ),
			'type'     => 'color',
			'inline'   => true,
			'required' => [
				[ 'chartType', '=', [ 'line', 'radar', 'bubble' ] ],
				[ 'customPointStyles', '!=', '' ]
			],
		];

		$this->controls['pointBorderColor'] = [
			'tab'      => 'content',
			'group'    => 'pointStyles',
			'label'    => esc_html__( 'Border Color', 'max-addons' ),
			'type'     => 'color',
			'inline'   => true,
			'required' => [
				[ 'chartType', '=', [ 'line', 'radar', 'bubble' ] ],
				[ 'customPointStyles', '!=', '' ]
			],
		];

		$this->controls['pointBorderWidth'] = [
			'tab'      => 'content',
			'group'    => 'pointStyles',
			'label'    => esc_html__( 'Border Width', 'max-addons' ),
			'type'     => 'number',
			'min'      => 0,
			'max'      => 10,
			'step'     => 1,
			'inline'   => true,
			'required' => [
				[ 'chartType', '=', [ 'line', 'radar', 'bubble' ] ],
				[ 'customPointStyles', '!=', '' ]
			],
		];

		$this->controls['pointSize'] = [
			'tab'         => 'content',
			'group'       => 'pointStyles',
			'label'       => esc_html__( 'Point Size', 'max-addons' ),
			'type'        => 'number',
			'min'         => 0,
			'max'         => 50,
			'step'        => 1,
			'placeholder' => esc_html__( '5', 'max-addons' ),
			'inline'      => true,
			'required'    => [
				[ 'chartType', '=', [ 'line', 'radar', 'bubble' ] ],
				[ 'customPointStyles', '!=', '' ]
			],
		];

		$this->controls['pointHoverSize'] = [
			'tab'         => 'content',
			'group'       => 'pointStyles',
			'label'       => esc_html__( 'Point Hover Size', 'max-addons' ),
			'type'        => 'number',
			'min'         => 0,
			'max'         => 50,
			'step'        => 1,
			'placeholder' => esc_html__( '10', 'max-addons' ),
			'inline'      => true,
			'required'    => [
				[ 'chartType', '=', [ 'line', 'radar', 'bubble' ] ],
				[ 'customPointStyles', '!=', '' ]
			],
		];
	}

	public function set_animation_controls() {
		$this->controls['chartAnimation'] = [
			'tab'         => 'content',
			'group'       => 'animationOptions',
			'label'       => esc_html__( 'Animation', 'max-addons' ),
			'type'        => 'select',
			'options'     => [
				'linear'           => esc_html__( 'linear', 'max-addons' ),
				'easeInQuad'       => esc_html__( 'easeInQuad', 'max-addons' ),
				'easeOutQuad'      => esc_html__( 'easeOutQuad', 'max-addons' ),
				'easeInOutQuad'    => esc_html__( 'easeInOutQuad', 'max-addons' ),
				'easeInCubic'      => esc_html__( 'easeInCubic', 'max-addons' ),
				'easeOutCubic'     => esc_html__( 'easeOutCubic', 'max-addons' ),
				'easeInOutCubic'   => esc_html__( 'easeInOutCubic', 'max-addons' ),
				'easeInQuart'      => esc_html__( 'easeInQuart', 'max-addons' ),
				'easeOutQuart'     => esc_html__( 'easeOutQuart', 'max-addons' ),
				'easeInOutQuart'   => esc_html__( 'easeInOutQuart', 'max-addons' ),
				'easeInQuint'      => esc_html__( 'easeInQuint', 'max-addons' ),
				'easeOutQuint'     => esc_html__( 'easeOutQuint', 'max-addons' ),
				'easeInOutQuint'   => esc_html__( 'easeInOutQuint', 'max-addons' ),
				'easeInSine'       => esc_html__( 'easeInSine', 'max-addons' ),
				'easeOutSine'      => esc_html__( 'easeOutSine', 'max-addons' ),
				'easeInOutSine'    => esc_html__( 'easeInOutSine', 'max-addons' ),
				'easeInExpo'       => esc_html__( 'easeInExpo', 'max-addons' ),
				'easeOutExpo'      => esc_html__( 'easeOutExpo', 'max-addons' ),
				'easeInOutExpo'    => esc_html__( 'easeInOutExpo', 'max-addons' ),
				'easeInCirc'       => esc_html__( 'easeInCirc', 'max-addons' ),
				'easeOutCirc'      => esc_html__( 'easeOutCirc', 'max-addons' ),
				'easeInOutCirc'    => esc_html__( 'easeInOutCirc', 'max-addons' ),
				'easeInElastic'    => esc_html__( 'easeInElastic', 'max-addons' ),
				'easeOutElastic'   => esc_html__( 'easeOutElastic', 'max-addons' ),
				'easeInOutElastic' => esc_html__( 'easeInOutElastic', 'max-addons' ),
				'easeInBack'       => esc_html__( 'easeInBack', 'max-addons' ),
				'easeOutBack'      => esc_html__( 'easeOutBack', 'max-addons' ),
				'easeInOutBack'    => esc_html__( 'easeInOutBack', 'max-addons' ),
				'easeInBounce'     => esc_html__( 'easeInBounce', 'max-addons' ),
				'easeOutBounce'    => esc_html__( 'easeOutBounce', 'max-addons' ),
				'easeInOutBounce'  => esc_html__( 'easeInOutBounce', 'max-addons' ),
			],
			'inline'      => true,
			'clearable'   => true,
			'pasteStyles' => false,
			'default'     => 'easeOutQuart',
		];

		$this->controls['chartAnimationDuration'] = [
			'tab'         => 'content',
			'group'       => 'animationOptions',
			'label'       => esc_html__( 'Duration', 'max-addons' ),
			'type'        => 'number',
			'min'         => 1,
			'max'         => 10000,
			'step'        => 100,
			'inline'      => true,
			'default'     => 1000,
		];

		$this->controls['chartAnimationLoop'] = [
			'tab'     => 'content',
			'group'   => 'animationOptions',
			'label'   => esc_html__( 'Loop Animation', 'max-addons' ),
			'type'    => 'checkbox',
			'inline'  => true,
			'reset'   => true,
			'default' => false,
		];
	}

	/**
	 * Retrieves the color code from settings based on color control output.
	 *
	 * @param array  $color The output array containing color data.
	 * @return string The color code in RGB or HEX format, or an empty string if not set.
	 */
	function get_color_code( $color ) {
		if ( ! empty( $color['rgb'] ) ) {
			return $color['rgb'];
		}

		if ( ! empty( $color['hex'] ) ) {
			return $color['hex'];
		}

		return '';
	}

	public function get_default_color(&$colorIndex) {
		static $defaultColors = ['#FF6384', '#36A2EB', '#FFCD56', '#F09640'];

		$color = $defaultColors[$colorIndex % count($defaultColors)];

		$colorIndex++;

		return $color;
	}

	/**
	 * Fetch CSV data from from csv file.
	 *
	 * @access protected
	 */
	protected function get_csv_data() {
		$settings = $this->settings;

		if ( isset( $settings['source'] ) && 'csv' !== $settings['source'] ) {
			return array(
				'labels' => '',
				'data' => '',
			);
		}

		$csv_source = isset( $settings['csvSource'] ) ? $settings['csvSource'] : 'url';
		$csv_url = ! empty( $settings['csvFileUrl'] ) ? $settings['csvFileUrl'] : '';

		if ( 'url' === $csv_source && '' !== $csv_url ) {
			$response = wp_remote_get(
				$csv_url,
				array(
					'sslverify' => false,
				)
			);

			if ( is_wp_error( $response ) || 200 !== $response['response']['code'] || '.csv' !== substr( $csv_url, -4 ) ) {
				return array(
					'data' => esc_html__( '<p>Please provide a valid CSV file.</p>', 'max-addons' ),
					'labels' => '',
				);
			}

			$body = wp_remote_retrieve_body($response);
		} else {
			$body = isset( $settings['csvFileText'] ) ? $settings['csvFileText'] : '';
		}

    	$csv_data_array = explode("\n", $body);

		$csv_data = [];

		if ( is_array( $csv_data_array ) ) {
			foreach ( $csv_data_array as $items ) {
				$csv_data[] = explode( ',', $items );
			}

			foreach ( $csv_data as $item ) {
				$csv_data_labels[] = $item[0];
			}
		}

		array_shift( $csv_data_labels );

		return array(
			'labels' => $csv_data_labels,
			'data' => $csv_data,
		);
	}

	/**
	 * Gets expire time of transient.
	 *
	 * @since 1.14.0
	 * @param array $settings The settings array.
	 * @return the transient expire time.
	 * @access public
	 */
	public function get_transient_expire( $settings ) {
		$expire_value = isset( $settings['cacheDuration'] ) ? $settings['cacheDuration'] : 'day';
		$expire_time  = 24 * HOUR_IN_SECONDS;

		if ( 'minute' === $expire_value ) {
			$expire_time = 1 * MINUTE_IN_SECONDS;
		} elseif ( 'half_hour' === $expire_value ) {
			$expire_time = 30 * MINUTE_IN_SECONDS;
		} elseif ( 'hour' === $expire_value ) {
			$expire_time = 60 * MINUTE_IN_SECONDS;
		} elseif ( 'week' === $expire_value ) {
			$expire_time = 7 * DAY_IN_SECONDS;
		} elseif ( 'month' === $expire_value ) {
			$expire_time = 30 * DAY_IN_SECONDS;
		} elseif ( 'year' === $expire_value ) {
			$expire_time = 365 * DAY_IN_SECONDS;
		}

		return $expire_time;
	}

	/**
	 * Retrieve Google Sheets API from Max Addons settings.
	 *
	 * @since 1.14.0
	 * @return string
	 */
	private function get_google_sheets_api() {
		return \MaxAddonsPro\Classes\Admin_Settings::get_option( 'mab_google_sheets_api_key' );
	}

	/**
	 * Fetch data from Google Sheets API.
	 *
	 * @return array|\WP_Error API response or error object.
	 */
	private function get_google_sheets_data() {
		$settings = $this->settings;
		$api_key  = $this->get_google_sheets_api();

		if ( empty( $api_key ) ) {
			return array(
				'data'  => array(),
				'error' => new \WP_Error(
					'missing_api_key',
					esc_html__( 'To fetch data from Google Sheets, you need to set up an API key.', 'max-addons' )
				),
			);
		}

		$sheet_id       = ! empty( $settings['googleSheetsUrl'] ) ? $this->extract_sheet_id( $settings['googleSheetsUrl'] ) : '';
		$cell_range     = ! empty( $settings['cellRange'] ) ? $settings['cellRange'] : '';
		$cache_duration = $this->get_transient_expire( $settings );
		$url            = sprintf( 'https://sheets.googleapis.com/v4/spreadsheets/%s/values/%s?key=%s', sanitize_text_field( $sheet_id ), $cell_range, sanitize_text_field( $api_key ) );

		if ( empty( $sheet_id ) ) {
			return array(
				'data'  => array(),
				'error' => new \WP_Error(
					'invalid_google_sheets_url',
					esc_html__( 'Please provide a valid Google Sheets URL.', 'max-addons' )
				),
			);
		}

		if ( empty( $cell_range ) ) {
			return array(
				'data'  => array(),
				'error' => new \WP_Error(
					'empty_google_sheets_cell_range',
					esc_html__( 'Please enter table cell range for provided Google Sheets URL.', 'max-addons' )
				),
			);
		}

		// Generate a unique transient key based on the sheet ID
		$transient_key_data = 'mab_google_sheets_data_' . md5( $api_key . $sheet_id . $cache_duration );
		$transient_key_url  = 'mab_google_sheets_url_' . md5( $api_key . $sheet_id . $cache_duration );

		// Check if cached data exists
		$cached_data = get_transient( $transient_key_data );
		$cached_url  = get_transient( $transient_key_url );

		if ( false !== $cached_data && $url === $cached_url ) {
			return array(
				'data'  => $cached_data,
				'error' => false,
			);
		}

		$response = wp_remote_get( esc_url_raw( $url ) );

		if ( is_wp_error( $response ) ) {
			return array(
				'data'  => array(),
				'error' => $response,
			);
		}

		if ( 200 !== wp_remote_retrieve_response_code( $response ) ) {
			return array(
				'data'  => array(),
				'error' => new \WP_Error(
					'google_sheets_fetch_error',
					esc_html__( 'Failed to fetch data from Google Sheets.', 'max-addons' )
				),
			);
		}

		$body = wp_remote_retrieve_body( $response );

		if ( empty( $body ) ) {
			return array(
				'data'  => array(),
				'error' => new \WP_Error(
					'google_sheets_empty',
					esc_html__( 'The provided Google Sheet is empty.', 'max-addons' )
				),
			);
		}

		$data = json_decode( $body, true );

		$response_data = array(
			'data'  => isset( $data['values'] ) ? $data['values'] : array(),
			'error' => false,
		);

		// Cache the data in a transient
		set_transient( $transient_key_data, $response_data['data'], intval( $cache_duration ) );
		set_transient( $transient_key_url, $url, intval( $cache_duration ) );

		return $response_data;
	}

	/**
	 * Extract Google Sheets ID from URL.
	 *
	 * @param string $url Google Sheets URL.
	 * @return string Google Sheets ID.
	 */
	private function extract_sheet_id( $url ) {
		preg_match( '/\/spreadsheets\/d\/([a-zA-Z0-9-_]+)/', $url, $matches );
		return isset( $matches[1] ) ? $matches[1] : '';
	}

	protected function get_additional_options( $settings, $options ) {
		if ( isset( $settings['showChartTitle'] ) && ! empty( $settings['chartTitle'] ) ) {
			$chart_title_color = isset( $settings['chartTitleColor'] ) ? $this->get_color_code( $settings['chartTitleColor'] ) : '';

			$options['plugins']['title'] = [
				'display' => true,
				'text'    => $settings['chartTitle'] ?? '',
				'font'    => [
					'size'       => isset( $settings['chartTitleFontSize'] ) ? $settings['chartTitleFontSize'] : 12,
					'style'      => isset( $settings['chartTitleFontStyle'] ) ? $settings['chartTitleFontStyle'] : '',
					'weight'     => isset( $settings['chartTitleFontWeight'] ) ? $settings['chartTitleFontWeight'] : '',
					'lineHeight' => isset( $settings['chartTitleLineHeight'] ) ? $settings['chartTitleLineHeight'] : '',
				],
				'padding' => [
					'bottom' => 20,
				],
			];

			if ( '' !== $chart_title_color ) {
				$options['plugins']['title']['color'] = $chart_title_color;
			}

			$chart_title_position = isset( $settings['chartTitlePosition'] ) ? $settings['chartTitlePosition'] : 'top';
			$chart_title_align    = isset( $settings['chartTitleAlign'] ) ? $settings['chartTitleAlign'] : 'center';

			if ( 'top' !== $chart_title_position ) {
				$options['plugins']['title']['position'] = $chart_title_position;
			}

			if ( 'center' !== $chart_title_align ) {
				$options['plugins']['title']['align'] = $chart_title_align;
			}
		}

		if ( ! empty( $settings['showLegend'] ) ) {
			$legend_color    = isset( $settings['legendColor'] ) ? $this->get_color_code( $settings['legendColor'] ) : '#666666';
			$legend_position = isset( $settings['legendPosition'] ) ? $settings['legendPosition'] : 'top';
			$legend_align    = isset( $settings['legendAlign'] ) ? $settings['legendAlign'] : 'center';

			$options['plugins']['legend'] = [
				'reverse'  => isset( $settings['legendReverse'] ),
				'labels'   => [
					'color' => $legend_color,
					'font'  => [
						'size'       => isset( $settings['legendFontSize'] ) ? $settings['legendFontSize'] : 12,
						'style'      => isset( $settings['legendFontStyle'] ) ? $settings['legendFontStyle'] : '',
						'weight'     => isset( $settings['legendFontWeight'] ) ? $settings['legendFontWeight'] : '',
						'lineHeight' => isset( $settings['legendLineHeight'] ) ? $settings['legendLineHeight'] : '',
					],
				],
			];

			if ( 'top' !== $legend_position ) {
				$options['plugins']['legend']['position'] = $legend_position;
			}

			if ( 'center' !== $legend_align ) {
				$options['plugins']['legend']['align'] = $legend_align;
			}
		} else {
			$options['plugins']['legend'] = [ 'display' => false ];
		}

		if ( ! empty( $settings['chartAnimation'] ) ) {
			$options['animation'] = [
				'duration' => isset( $settings['chartAnimationDuration'] ) ? $settings['chartAnimationDuration'] : '',
				'easing'   => $settings['chartAnimation'],
				'loop'     => ! empty( $settings['chartAnimationLoop'] ),
			];
		}

		if ( isset( $settings['showTooltip'] ) ) {
			$interaction_mode    = isset( $settings['interactionMode'] ) ? $settings['interactionMode'] : 'point';
			$tooltip_event       = isset( $settings['tooltipEvent'] ) ? $settings['tooltipEvent'] : 'hover';
			$tooltip_title_color = isset( $settings['tooltipTitleColor'] ) ? $this->get_color_code( $settings['tooltipTitleColor'] ) : '';
			$tooltip_body_color  = isset( $settings['tooltipBodyColor'] ) ? $this->get_color_code( $settings['tooltipBodyColor'] ) : '';

			$options['interaction']['mode'] = $interaction_mode;

			$options['plugins']['tooltip'] = [
				'backgroundColor' => isset( $settings['tooltipBackgroundColor'] ) ? $settings['tooltipBackgroundColor'] : '',
				'borderColor'     => $settings['tooltipBorderColor'] ?? '',
				'borderWidth'     => isset( $settings['tooltipBorderWidth'] ) ? $settings['tooltipBorderWidth'] : 0,
				'cornerRadius'    => isset( $settings['tooltipBorderRadius'] ) ? $settings['tooltipBorderRadius'] : 6,
				'padding'         => isset( $settings['tooltipPadding'] ) ? $settings['tooltipPadding'] : 6,
				'titleColor'      => ( '' !== $tooltip_title_color ) ? $tooltip_title_color : '#FFFFFF',
				'bodyColor'       => ( '' !== $tooltip_body_color ) ? $tooltip_body_color : '#FFFFFF',
				'titleFont'       => [ 'size' => isset( $settings['tooltipTitleFontSize'] ) ? $settings['tooltipTitleFontSize'] : 12 ],
				'bodyFont'        => [ 'size' => isset( $settings['tooltipBodyFontSize'] ) ? $settings['tooltipBodyFontSize'] : 12 ],
			];

			if ( 'click' === $tooltip_event ) {
				$options['plugins']['tooltip']['events'] = ['click'];
				$options['events'] = ['click'];
			}
		} else {
			$options['plugins']['tooltip'] = [ 'enabled' => false ];
		}

		$chart_height = isset( $settings['chartHeight'] ) ? $settings['chartHeight'] : '';

		if ( ! isset( $settings['maintainAspectRatio'] ) && '' !== $chart_height ) {
			$options['maintainAspectRatio'] = false;
		}

		return $options;
	}

	protected function bubble_chart_data_array( $bubble_data ) {
		$bubble_data = trim( $bubble_data );
		$matches = [];
		preg_match_all( '#\[([^\]]+)\]#U', $bubble_data, $matches );

		if ( empty( $matches[1] ) ) {
			return [];
		}

		$bubble = [];
		foreach ( $matches[1] as $item ) {
			$values = explode( ',', trim( $item, '[] ' ) );
			if ( 3 !== count( $values ) ) {
				continue;
			}

			$bubble[] = (object) [
				'x' => floatval( $values[0] ),
				'y' => floatval( $values[1] ),
				'r' => floatval( $values[2] ),
			];
		}

		return $bubble;
	}

	private function get_ticks_settings( $settings, $axis ) {
		$show_labels_key  = "{$axis}AxisShowLabels";
		$labels_color_key = "{$axis}AxisLabelsColor";
		$size_key         = "{$axis}AxisLabelsFontSize";
		$font_style_key   = "{$axis}AxisLabelsFontStyle";
		$font_weight_key  = "{$axis}AxisLabelsFontWeight";
		$line_height_key  = "{$axis}AxisLabelsLineHeight";
		$step_size_key    = 'step_size';

		$labels_color = isset( $settings[$labels_color_key] ) ? $this->get_color_code( $settings[$labels_color_key] ) : '';

		$ticks_data = [
			'display'  => ! empty( $settings[$show_labels_key] ),
			'font'     => [
				'size'       => ! empty( $settings[$size_key] ) ? $settings[$size_key] : 12,
				'style'      => $settings[$font_style_key] ?? '',
				'weight'     => $settings[$font_weight_key] ?? '',
				'lineHeight' => ! empty( $settings[$line_height_key] ) ? $settings[$line_height_key] . $settings[$line_height_key] : '',
			],
			'stepSize' => ! empty( $settings[$step_size_key] ) ? $settings[$step_size_key] : 0,
		];

		if ( $labels_color ) {
			$ticks_data['color'] = $labels_color;
		}

		return $ticks_data;
	}

	private function get_axis_settings( $settings, $axis ) {
		$begin_at_zero_key = 'beginAtZero';
		$chart_type_key    = 'chartType';
		$stacked_key       = 'stacked';

		return [
			'beginAtZero' => isset( $settings[$begin_at_zero_key] ) ? true : false,
			'stacked'     => ( isset( $settings[$chart_type_key] ) && 'bar' === $settings[$chart_type_key] && isset( $settings[$stacked_key] ) ) ? true : false,
			'ticks'       => $this->get_ticks_settings( $settings, $axis ),
		];
	}

	private function get_grid_settings( $settings, $axis ) {
		$show_grid_lines_key = "{$axis}AxisShowGridLines";
		$grid_color_key      = "{$axis}AxisGridColor";
		$grid_width_key      = "{$axis}AxisGridWidth";

		$grid_line_color = isset( $settings[$grid_color_key] ) ? $this->get_color_code( $settings[$grid_color_key] ) : '#e0e0e0';

		return [
			'display'   => ! empty( $settings[$show_grid_lines_key] ),
			'color'     => $grid_line_color,
			'lineWidth' => ! empty( $settings[$grid_width_key] ) ? $settings[$grid_width_key] : 1,
		];
	}

	protected function get_grid_options( $settings ) {
		$x_axis_title = $this->get_axis_title( $settings, 'x' );
		$y_axis_title = $this->get_axis_title( $settings, 'y' );

		return [
			'x' => array_merge( $this->get_axis_settings( $settings, 'x' ), [
				'title' => $x_axis_title,
				'grid'  => $this->get_grid_settings( $settings, 'x' ),
			] ),
			'y' => array_merge( $this->get_axis_settings( $settings, 'y' ), [
				'title'        => $y_axis_title,
				'suggestedMin' => isset( $settings['yAxisMin'] ) ? $settings['yAxisMin'] : '',
				'suggestedMax' => isset( $settings['yAxisMax'] ) ? $settings['yAxisMax'] : '',
				'grid'         => $this->get_grid_settings( $settings, 'y' ),
			] ),
		];
	}

	private function get_axis_title( $settings, $axis ) {
		$show_title_key  = "{$axis}AxisShowTitle";
		$title_key       = "{$axis}AxisTitle";
		$color_key       = "{$axis}AxisTitleColor";
		$size_key        = "{$axis}AxisTitleFontSize";
		$font_style_key  = "{$axis}AxisTitleFontStyle";
		$font_weight_key = "{$axis}AxisTitleFontWeight";
		$line_height_key = "{$axis}AxisTitleLineHeight";

		$title_color = isset( $settings[$color_key] ) ? $this->get_color_code( $settings[$color_key] ) : '';

		if ( isset( $settings[$show_title_key] ) ) {
			return [
				'display' => true,
				'text'    => $settings[$title_key] ?? '',
				'color'   => $title_color,
				'font'    => [
					'size'       => ! empty( $settings[$size_key]['size'] ) ? $settings[$size_key]['size'] : 12,
					'style'      => $settings[$font_style_key] ?? '',
					'weight'     => $settings[$font_weight_key] ?? '',
					'lineHeight' => ! empty( $settings[$line_height_key]['size'] ) ? $settings[$line_height_key]['size'] . $settings[$line_height_key]['unit'] : '',
				],
			];
		}

		return [ 'display' => false ];
	}

	protected function get_point_styles( $settings, $chart_type ) {
		$styles = [];

		$point_border_color = isset( $settings['pointBorderColor'] ) ? $this->get_color_code( $settings['pointBorderColor'] ) : '';
		$point_bg_color     = isset( $settings['pointBackground'] ) ? $this->get_color_code( $settings['pointBackground'] ) : '';

		if ( isset( $settings['customPointStyles'] ) ) {
			$styles['pointStyle'] = $settings['pointStyle'] ?? '';

			if ( 'bubble' !== $chart_type ) {
				if ( '' !== $point_bg_color ) {
					$styles['pointBackgroundColor'] = $point_bg_color;
				}

				$styles['pointRadius'] = $settings['pointSize'] ?? 5;
				$styles['pointHoverRadius'] = $settings['pointHoverSize'] ?? 10;
			}

			if ( isset( $settings['pointBorderWidth'] ) ) {
				$styles['pointBorderWidth'] = $settings['pointBorderWidth'];
			}

			if ( '' !== $point_border_color ) {
				$styles['pointBorderColor'] = $point_border_color;
			}
		}

		return $styles;
	}

	protected function get_chart_options( $settings, $chart_type ) {
		$options = [];
		$bar_chart_type = isset( $settings['barChartType'] ) ? $settings['barChartType'] : 'vertical_bar';

		if ( 'bar' === $chart_type && 'horizontal_bar' === $bar_chart_type ) {
			$options['indexAxis'] = 'y';
		}

		if ( 'pie' === $chart_type ) {
			$options['cutout'] = 0;
		} elseif ( 'doughnut' === $chart_type ) {
			$options['cutout'] = 50;
		} elseif ( 'line' === $chart_type || 'bar' === $chart_type || 'bubble' === $chart_type ) {
			$options['scales'] = $this->get_grid_options( $settings );
		}

		$options = $this->get_additional_options( $settings, $options );

		return $options;
	}

	protected function get_default_border_width() {
		$chart_type = $this->get_chart_type();

		switch ( $chart_type ) {
			case 'bar':
				$border_width = 0;
				break;

			case 'line':
			case 'radar':
			case 'bubble':
				$border_width = 3;
				break;

			case 'pie':
			case 'doughnut':
			case 'polarArea':
				$border_width = 2;
				break;
			
			default:
				$border_width = 1;
				break;
		}

		return $border_width;
	}

	protected function get_border_width() {
		$settings   = $this->settings;
		$chart_type = $this->get_chart_type();

		switch ( $chart_type ) {
			case 'bar':
				$data_border_width = isset( $settings['barBorderWidth'] ) ? $settings['barBorderWidth'] : 1;
				break;

			case 'line':
				$data_border_width = isset( $settings['lineBorderWidth'] ) ? $settings['lineBorderWidth'] : 3;
				break;

			case 'radar':
				$data_border_width = isset( $settings['radarBorderWidth'] ) ? $settings['radarBorderWidth'] : 3;
				break;

			case 'bubble':
				$data_border_width = isset( $settings['bubbleBorderWidth'] ) ? $settings['bubbleBorderWidth'] : 1;
				break;

			case 'pie':
			case 'doughnut':
			case 'polarArea':
				$data_border_width = isset( $settings['pieBorderWidth'] ) ? $settings['pieBorderWidth'] : 1;
				break;
			
			default:
				$data_border_width = 0;
				break;
		}

		$border_width = ( '' !== $data_border_width ) ? $data_border_width : $this->get_default_border_width();

		return $border_width;
	}

	function get_dynamic_dataset( $items ) {
		static $data = [];
		static $labels = [];
		$dataset_labels = [];

		$settings = $this->settings;
		$dataset_labels_input = ! empty( $settings['dynamicLabels'] ) ? $this->render_dynamic_data_tag( $settings['dynamicLabels'] ) : '';

		foreach ( $items as $item ) {
			$country = $dataset_labels_input;
			$value = $this->render_dynamic_data_tag( $item['datasetData'] );
			$label = $item['datasetLabel'];

			if ( ! isset( $data[ $country ] ) ) {
				$data[ $country ] = [ $country ];
			}

			$data[ $country ][] = $value;

			if ( ! in_array( $country, $labels, true ) ) {
				$labels[] = $country;
			}

			$dataset_labels[] = $label;
		}

		$data_values = [
			'labels' => $labels,
			'data' => array_values( $data ),
			'dataset_labels' => array_unique( $dataset_labels )
		];

		return $data_values;
	}

	protected function get_datasets( $settings, $chart_type ) {
		$source = isset( $settings['source'] ) ? $settings['source'] : 'custom';
		$datasets = [];
		$colorIndex = 0;

		switch ( $source ) {
			case 'query':
				$columns = $settings['dynamicChartDataset'] ?? [];
				$query = new \Bricks\Query( [
					'id'       => $this->id,
					'settings' => $settings,
				] );
	
				$item = $columns;
				$dynamic_data = $query->render( [ $this, 'get_dynamic_dataset' ], compact( 'item' ), true );
				array_shift( $dynamic_data );
				$dynamic_data = end( $dynamic_data );
	
				$dynamic_labels = $dynamic_data['labels'];
				$dynamic_dataset = $dynamic_data['data'];
				array_unshift( $dynamic_dataset, $dynamic_data['dataset_labels'] );
	
				$sheet_data = [
					'data'   => $dynamic_dataset,
					'labels' => $dynamic_labels,
				];
	
				$query->destroy();
				unset( $query );
				break;
	
			case 'csv':
				$sheet_data = $this->get_csv_data();
				break;
	
			case 'google-sheets':
				$sheet_data = $this->get_google_sheets_data();
				break;
		}

		if ( 'custom' !== $source ) {
			if ( ! empty( $sheet_data['data'] ) ) {
				$header_row = $sheet_data['data'][0]; // This assumes the first row contains headers
				$dynamic_settings = $settings['dynamicColors'] ?? [];

				if ( 'query' === $source ) {
					array_unshift( $header_row, '' );
				}

				// Iterate over each column starting from the second column
				for ( $col = 1; $col < count($header_row); $col++ ) {
					$chart_data = [];

					foreach ( $sheet_data['data'] as $index => $row ) {
						if (0 === $index) { // Skip header row
							continue;
						}

						$chart_data[] = $row[$col];
					}

					$dataset = [
						'datasetLabel' => $header_row[$col],
						'datasetData'  => implode(',', $chart_data)
					];

					// Get dynamic colors if available
					if ( array_key_exists( $col - 1, $dynamic_settings ) ) {
						$default_color = $this->get_default_color($colorIndex);

						$bg_color = ! empty( $dynamic_settings[$col - 1]['backgroundColor'] ) ? $this->get_color_code( $dynamic_settings[$col - 1]['backgroundColor'] ) : $default_color;
						$border_color = ! empty( $dynamic_settings[$col - 1]['borderColor'] ) ? $this->get_color_code( $dynamic_settings[$col - 1]['borderColor'] ) : $default_color;

						$dataset['backgroundColor'] = $bg_color;
						$dataset['borderColor'] = $border_color;

						if ( in_array( $chart_type, ['pie', 'doughnut', 'polarArea'], true ) ) {
							$bg_colors = isset( $dynamic_settings[$col - 1]['backgroundColors'] ) ? $dynamic_settings[$col - 1]['backgroundColors'] : [];
							$border_colors = isset( $dynamic_settings[$col - 1]['borderColors'] ) ? $dynamic_settings[$col - 1]['borderColors'] : [];

							$dataset['backgroundColors'] = $bg_colors;
							$dataset['borderColors'] = $border_colors;
						}
					} else {
						$default_color = $this->get_default_color($colorIndex);

						$dataset['backgroundColor'] = $default_color;
						$dataset['borderColor'] = $default_color;

						if ( in_array( $chart_type, ['pie', 'doughnut', 'polarArea'], true ) ) {
							$dataset['backgroundColors'] = [];
							$dataset['borderColors'] = [];
						}
					}

					$chart_datasets[] = $dataset;
				}

				foreach ( $chart_datasets as $item ) {
					$datasets[] = $this->get_chart_dataset( $settings, $chart_type, $item );
				}
			}
		} else {
			$chart_datasets = ( 'bubble' === $chart_type )
				? ( isset( $settings['bubbleChartDataset'] ) ? $settings['bubbleChartDataset'] : [] )
				: ( isset( $settings['chartDataset'] ) ? $settings['chartDataset'] : [] );

			foreach ( $chart_datasets as $item ) {
				$datasets[] = $this->get_chart_dataset( $settings, $chart_type, $item );
			}
		}

		return $datasets;
	}

	protected function get_chart_dataset( $settings, $chart_type, $item ) {
		$dataset = [
			'label'           => $item['datasetLabel'],
			'data'            => ( 'bubble' === $chart_type )
				? $this->bubble_chart_data_array( $item['bubble_data'] )
				: array_map( 'floatval', explode( ',', $item['datasetData'] ) ),
			'fill'            => isset( $item['fill'] ),
			'borderWidth'     => $this->get_border_width(),
		];

		if ( in_array( $chart_type, ['pie', 'doughnut', 'polarArea'], true ) ) {
			$bg_colors = [];
			$border_colors = [];

			if ( ! empty( $item['backgroundColors'] ) ) {
				foreach ( $item['backgroundColors'] as $bg_color ) {
					if ( ! empty( $bg_color['bgColor'] ) ) {
						$bg_colors[] = $this->get_color_code( $bg_color['bgColor'] );
					}
				}
			} else {
				$bg_colors = ['#FF6384', '#36A2EB', '#FFCD56', '#F09640'];
			}

			if ( ! empty( $item['borderColors'] ) ) {
				foreach ( $item['borderColors'] as $bg_color ) {
					if ( ! empty( $bg_color['borderColor'] ) ) {
						$border_colors[] = $this->get_color_code( $bg_color['borderColor'] );
					}
				}
			} else {
				$border_colors = ['#FFFFFF', '#FFFFFF', '#FFFFFF', '#FFFFFF'];
			}

			$dataset['backgroundColor'] = $bg_colors;
			$dataset['borderColor'] = $border_colors;
		} else {
			$dataset['backgroundColor'] = isset( $item['backgroundColor'] ) ? ( is_array( $item['backgroundColor'] ) ? $this->get_color_code( $item['backgroundColor'] ) : $item['backgroundColor'] ) : '';
			$dataset['borderColor'] = isset( $item['borderColor'] ) ? ( is_array( $item['borderColor'] ) ? $this->get_color_code( $item['borderColor'] ) : $item['borderColor'] ) : '';
		}

		if ( in_array( $chart_type, ['line', 'radar'], true ) ) {
			$dataset['borderDash'] = ( ! empty( $item['borderDash'] ) ) ? [5, 5] : [];
		}

		if ( in_array( $chart_type, ['line', 'radar', 'bubble'], true ) ) {
			$dataset = array_merge( $dataset, $this->get_point_styles( $settings, $chart_type ) );
		}

		if ( 'line' === $chart_type && isset( $settings['steppedLine'] ) ) {
			$dataset['stepped'] = true;
		}

		if ( 'line' === $chart_type && ! empty( $settings['tension'] ) ) {
			$dataset['tension'] = $settings['tension'];
		}

		if ( 'bar' === $chart_type ) {
			if ( ! empty( $settings['barBorderRadius'] ) ) {
				$dataset['borderRadius']  = $settings['barBorderRadius'];
				$dataset['borderSkipped'] = false;
			}

			if ( ! empty( $settings['barthickness'] ) ) {
				$dataset['barThickness'] = $settings['barthickness'];
			}

			if ( ! empty( $settings['maxbarthickness'] ) ) {
				$dataset['maxBarThickness'] = $settings['maxbarthickness'];
			}
		}

		return $dataset;
	}

	public function get_labels() {
		$settings = $this->settings;
		$source = isset( $settings['source'] ) ? $settings['source'] : 'custom';
		$labels   = [];

		if ( 'query' === $source ) {
			$columns = ! empty( $settings['dynamicChartDataset'] ) ? $settings['dynamicChartDataset'] : [];

			$query = new \Bricks\Query(
				[
					'id'       => $this->id,
					'settings' => $settings,
				]
			);

			$item = $columns;

			$dynamic_data = $query->render( [ $this, 'get_dynamic_dataset' ], compact( 'item' ), true );

			array_shift( $dynamic_data );
			$sheet_data = end($dynamic_data);

			$labels = $sheet_data['labels'];

			// Destroy the Query to explicitly remove it from the global store
			$query->destroy();
			unset( $query );

		} elseif ( 'google-sheets' === $source ) {
			$sheet_data = $this->get_google_sheets_data();

			if ( ! empty( $sheet_data['data'] ) ) {
				foreach ( $sheet_data['data'] as $index => $item ) {
					if ( 0 === $index ) {
						continue;
					}

					$labels[] = $item[0];
				}
			}

		} elseif ( 'csv' === $source ) {
			$csv_data = $this->get_csv_data();
			$labels   = $csv_data['labels'];
		} else {
			$labels = isset( $settings['labels'] ) ? explode( ',', $settings['labels'] ) : '';
		}

		return $labels;
	}

	protected function get_chart_type() {
		$settings = $this->settings;
		$bar_chart_type = isset( $settings['barChartType'] ) ? $settings['barChartType'] : 'vertical_bar';

		if ( isset( $settings['chartType'] ) && 'bar' === $settings['chartType'] && 'horizontal_bar' === $bar_chart_type ) {
			return 'bar';
		}

		return $settings['chartType'] ?? 'line';
	}

	public function render() {
		$settings   = $this->settings;
		$chart_type = $this->get_chart_type();
		$datasets   = $this->get_datasets( $settings, $chart_type );
		$options    = $this->get_chart_options( $settings, $chart_type );
		$labels     = $this->get_labels();

		$datasets = apply_filters( 'mab_chart_datasets', $datasets );
		$options  = apply_filters( 'mab_chart_options', $options );

		$this->set_attribute( 'wrap', 'class', 'mab-dynamic-charts' );
		$this->set_attribute( 'wrap', 'data-settings', [
			wp_json_encode(
				array_filter( [
					'type'    => $chart_type,
					'data'    => [
						'labels'   => $labels,
						'datasets' => $datasets,
					],
					'options' => $options,
				] )
			),
		] );
		?>
		<div <?php echo wp_kses_post( $this->render_attributes( '_root' ) ); ?>>
			<div <?php echo wp_kses_post( $this->render_attributes( 'wrap' ) ); ?>></div>
		</div>
		<?php
	}
}
