<?php
namespace MaxAddonsPro\Elements;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly
}

class Dynamic_Table_Element extends \Bricks\Element {
	// Element properties
	public $category     = 'max-addons-elements'; // Use predefined element category 'general'
	public $name         = 'max-dynamic-table'; // Make sure to prefix your elements
	public $icon         = 'ti-widgetized'; // Themify icon font class
	public $css_selector = ''; // Default CSS selector
	public $header_loop_index = 0;
	public $loop_index   = 0;
	public $scripts      = [ 'mabDynamicTable' ]; // Script(s) run when element is rendered on frontend or updated in builder

	// Return localized element label
	public function get_label() {
		return esc_html__( 'Dynamic Table', 'max-addons' );
	}

	// Enqueue element styles and scripts
	public function enqueue_scripts() {
		wp_enqueue_style( 'mab-dynamic-table' );
		wp_enqueue_script( 'gridjs' );
		wp_enqueue_script( 'mab-dynamic-table' );
		wp_enqueue_script( 'mab-frontend' );
	}

	// Set builder control groups
	public function set_control_groups() {
		$this->control_groups['general'] = [ // Unique group identifier (lowercase, no spaces)
			'title' => esc_html__( 'General', 'max-addons' ), // Localized control group title
			'tab'   => 'content', // Set to either "content" or "style"
		];

		$this->control_groups['header'] = [
			'title'    => esc_html__( 'Header', 'max-addons' ),
			'tab'      => 'content',
			'required' => [ 'source', '!=', [ 'query', 'google-sheets' ] ],
		];

		$this->control_groups['body'] = [
			'title'    => esc_html__( 'Body', 'max-addons' ),
			'tab'      => 'content',
			'required' => [ 'source', '!=', [ 'query', 'google-sheets' ] ],
		];

		$this->control_groups['dynamicContent'] = [
			'title' => esc_html__( 'Dynamic Content', 'max-addons' ),
			'tab'   => 'content',
			'required' => [ 'source', '=', 'query' ],
		];

		$this->control_groups['search'] = [
			'title' => esc_html__( 'Search', 'max-addons' ),
			'tab'   => 'content',
		];

		$this->control_groups['pagination'] = [
			'title' => esc_html__( 'Pagination', 'max-addons' ),
			'tab'   => 'content',
		];

		$this->control_groups['headerStyle'] = [
			'title' => esc_html__( 'Header Style', 'max-addons' ),
			'tab'   => 'content',
		];

		$this->control_groups['rows'] = [
			'title' => esc_html__( 'Rows Style', 'max-addons' ),
			'tab'   => 'content',
		];

		$this->control_groups['cells'] = [
			'title' => esc_html__( 'Cells Style', 'max-addons' ),
			'tab'   => 'content',
		];

		$this->control_groups['iconStyle'] = [
			'title'    => esc_html__( 'Icon Style', 'max-addons' ),
			'tab'      => 'content',
			'required' => [ 'source', '!=', 'google-sheets' ],
		];

		$this->control_groups['buttonStyle'] = [
			'title'    => esc_html__( 'Button Style', 'max-addons' ),
			'tab'      => 'content',
			'required' => [ 'source', '=', 'custom' ],
		];
	}

	// Set builder controls
	public function set_controls() {

		$this->set_general_controls();

		$this->set_header_controls();

		$this->set_body_controls();

		$this->set_dynamic_content_controls();

		$this->set_search_controls();

		$this->set_pagination_controls();

		$this->set_header_style_controls();

		$this->set_rows_controls();

		$this->set_cells_controls();

		$this->set_icon_controls();

		$this->set_button_controls();
	}

	// Set general controls
	public function set_general_controls() {

		$this->controls['source'] = [
			'tab'       => 'content',
			'group'     => 'general',
			'label'     => esc_html__( 'Source', 'max-addons' ),
			'type'      => 'select',
			'options'   => [
				'custom'        => esc_html__( 'Custom', 'max-addons' ),
				'query'         => esc_html__( 'Query', 'max-addons' ),
				'google-sheets' => esc_html__( 'Google Sheets', 'max-addons' ),
			],
			'inline'    => true,
			'clearable' => false,
			'default'   => 'custom',
		];

		$this->controls['query'] = [
			'tab'      => 'content',
			'group'    => 'general',
			'label'    => esc_html__( 'Query', 'max-addons' ),
			'type'     => 'query',
			'popup'    => true,
			'inline'   => true,
			'required' => [ 'source', '=', 'query' ],
		];

		$this->controls['googleSheetsUrl'] = [
			'tab'      => 'content',
			'group'    => 'general',
			'label'    => esc_html__( 'Google Sheets URL', 'max-addons' ),
			'type'     => 'text',
			'required' => [ 'source', '=', 'google-sheets' ],
		];

		$this->controls['cellRange'] = [
			'tab'         => 'content',
			'group'       => 'general',
			'label'       => esc_html__( 'Table cell range', 'max-addons' ),
			'type'        => 'text',
			'required'    => [ 'source', '=', 'google-sheets' ],
		];

		$this->controls['cacheDuration'] = [
			'tab'         => 'content',
			'group'       => 'general',
			'label'       => esc_html__( 'Refresh data after', 'max-addons' ),
			'type'        => 'select',
			'options'     => [
				'hour'  => __( '1 hour', 'max-addons' ),
				'day'   => __( '1 day', 'max-addons' ),
				'week'  => __( '1 week', 'max-addons' ),
				'month' => __( '1 month', 'max-addons' ),
				'year'  => __( '1 year', 'max-addons' ),
			],
			'placeholder' => esc_html__( '1 day', 'max-addons' ),
			'inline'      => true,
			'clearable'   => true,
			'required'    => [ 'source', '=', 'google-sheets' ],
		];

		$this->controls['sortable'] = [
			'tab'         => 'content',
			'group'       => 'general',
			'label'       => esc_html__( 'Sortable table', 'max-addons' ),
			'description' => esc_html__( 'Enable sorting the table data by clicking on the table headers', 'max-addons' ),
			'type'        => 'checkbox',
		];

		$breakpoints        = \Bricks\Breakpoints::$breakpoints;
		$breakpoint_options = [];

		foreach ( $breakpoints as $index => $breakpoint ) {
			$breakpoint_options[ $breakpoint['key'] ] = $breakpoint['label'] . ' (<= ' . $breakpoint['width'] . 'px)';
		}

		$breakpoint_options['never']  = esc_html__( 'Never', 'max-addons' );

		$this->controls['stackTable'] = [
			'tab'         => 'content',
			'group'       => 'general',
			'label'       => esc_html__( 'Stack at breakpoint', 'max-addons' ),
			'type'        => 'select',
			'options'     => $breakpoint_options,
			'rerender'    => true,
			'placeholder' => esc_html__( 'Never', 'max-addons' ),
		];

		$this->controls['stackedDisplay'] = [
			'tab'         => 'content',
			'group'       => 'general',
			'label'       => esc_html__( 'Stacked content display', 'max-addons' ),
			'type'        => 'select',
			'options'     => [
				'row'    => esc_html__( 'Row', 'max-addons' ),
				'column' => esc_html__( 'Column', 'max-addons' ),
			],
			'inline'      => true,
			'clearable'   => true,
			'default'     => 'column',
			'placeholder' => esc_html__( 'Column', 'max-addons' ),
			'required'    => [ 'stackTable', '!=', [ '', 'never' ] ],
		];
	}

	// Set header controls
	public function set_header_controls() {
		$this->controls['columns'] = [
			'tab'           => 'content',
			'group'         => 'header',
			'placeholder'   => esc_html__( 'Column', 'max-addons' ),
			'type'          => 'repeater',
			'titleProperty' => 'cellText',
			'fields'        => [
				'cellText'         => [
					'label'          => esc_html__( 'Text', 'max-addons' ),
					'type'           => 'text',
					'hasDynamicData' => 'text',
				],
				'cellIconType'     => [
					'label'     => esc_html__( 'Icon Type', 'max-addons' ),
					'type'      => 'select',
					'options'   => [
						'icon'  => esc_html__( 'Icon', 'max-addons' ),
						'image' => esc_html__( 'Image', 'max-addons' ),
					],
					'inline'    => true,
					'clearable' => true,
					'default'   => '',
				],
				'cellIcon'         => [
					'label'    => esc_html__( 'Icon', 'max-addons' ),
					'type'     => 'icon',
					'default'  => [
						'library' => 'themify',
						'icon'    => 'ti-wordpress',
					],
					'required' => [ 'cellIconType', '=', [ 'icon' ] ],
				],
				'cellImage'        => [
					'label'    => esc_html__( 'Image', 'max-addons' ),
					'type'     => 'image',
					'required' => [ 'cellIconType', '=', [ 'image' ] ],
				],
				'cellIconPosition' => [
					'label'     => esc_html__( 'Icon Position', 'max-addons' ),
					'type'      => 'select',
					'options'   => [
						'before' => esc_html__( 'Before', 'max-addons' ),
						'after'  => esc_html__( 'After', 'max-addons' ),
					],
					'inline'    => true,
					'clearable' => true,
					'default'   => 'before',
					'required'  => [ 'cellIconType', '=', [ 'image', 'icon' ] ],
				],
				'columnWidth' => [
					'label' => esc_html__( 'Width', 'max-addons' ),
					'type'  => 'number',
					'units' => true,
				],
				'disableSorting' => [
					'label' => esc_html__( 'Disable sorting', 'max-addons' ),
					'info'  => esc_html__( 'Disable sorting on this column', 'max-addons' ),
					'type'  => 'checkbox',
				],
				'columnAttributes' => [
					'label'         => esc_html__( 'Attributes', 'max-addons' ),
					'type'          => 'repeater',
					'titleProperty' => 'name',
					'placeholder'   => esc_html__( 'Attribute', 'max-addons' ),
					'fields'        => [
						'name'  => [
							'label'    => esc_html__( 'Name', 'max-addons' ),
							'type'     => 'text',
							'rerender' => false,
						],
						'value' => [
							'label'    => esc_html__( 'Value', 'max-addons' ),
							'type'     => 'text',
							'rerender' => false,
						],
					],
				],
			],
			'default'       => [
				[
					'cellText'         => esc_html__( 'Header Column #1', 'max-addons' ),
					'cellIconType'     => '',
					'cellIcon'         => [
						'library' => 'themify',
						'icon'    => 'ti-wordpress',
					],
					'cellIconPosition' => 'before',
				],
				[
					'cellText'         => esc_html__( 'Header Column #2', 'max-addons' ),
					'cellIconType'     => '',
					'cellIcon'         => [
						'library' => 'themify',
						'icon'    => 'ti-wordpress',
					],
					'cellIconPosition' => 'before',
				],
				[
					'cellText'         => esc_html__( 'Header Column #3', 'max-addons' ),
					'cellIconType'     => '',
					'cellIcon'         => [
						'library' => 'themify',
						'icon'    => 'ti-wordpress',
					],
					'cellIconPosition' => 'before',
				],
			],
			'required'      => [ 'source', '!=', [ 'query', 'google-sheets' ] ],
		];
	}

	// Set body controls
	public function set_body_controls() {
		$this->controls['bodyRows'] = [
			'tab'         => 'content',
			'group'       => 'body',
			'placeholder' => esc_html__( 'Row', 'max-addons' ),
			'type'        => 'repeater',
			'fields'      => [
				'cell' => [
					'label'         => esc_html__( 'Cells', 'max-addons' ),
					'type'          => 'repeater',
					'placeholder'   => esc_html__( 'Cell', 'max-addons' ),
					'titleProperty' => 'cellText',
					'fields'        => [
						'cellTextType' => [
							'label'     => esc_html__( 'Text type', 'max-addons' ),
							'type'      => 'select',
							'options'   => [
								'basic' => esc_html__( 'Basic text', 'max-addons' ),
								'rich'  => esc_html__( 'Rich text', 'max-addons' ),
							],
							'inline'    => true,
							'clearable' => true,
							'default'   => 'basic',
						],
						'cellText' => [
							'label'          => esc_html__( 'Text', 'max-addons' ),
							'type'           => 'text',
							'hasDynamicData' => 'text',
							'required'       => [ 'cellTextType', '!=', 'rich' ],
						],
						'cellTextRich' => [
							'label'          => esc_html__( 'Text', 'max-addons' ),
							'type'           => 'editor',
							'hasDynamicData' => 'text',
							'required'       => [ 'cellTextType', '=', 'rich' ],
						],
						'link' => [
							'label' => esc_html__( 'Link', 'max-addons' ),
							'type'  => 'link',
						],
						'showButton' => [
							'label' => esc_html__( 'Show button', 'max-addons' ),
							'type'  => 'checkbox',
						],
						'buttonText' => [
							'label'          => esc_html__( 'Button text', 'max-addons' ),
							'type'           => 'text',
							'hasDynamicData' => 'text',
							'required'       => [ 'showButton', '!=', '' ],
						],
						'applyLinkTo' => [
							'label'       => esc_html__( 'Apply link to', 'max-addons' ),
							'type'        => 'select',
							'options'     => [
								'button' => esc_html__( 'Button', 'max-addons' ),
								'cell'   => esc_html__( 'Cell', 'max-addons' ),
							],
							'placeholder' => esc_html__( 'Button', 'max-addons' ),
							'inline'      => true,
							'clearable'   => true,
							'default'     => 'button',
							'required'    => [
								[ 'showButton', '!=', '' ],
								[ 'link', '!=', '' ],
							],
						],
						'cellIconType' => [
							'label'     => esc_html__( 'Icon type', 'max-addons' ),
							'type'      => 'select',
							'options'   => [
								'icon'  => esc_html__( 'Icon', 'max-addons' ),
								'image' => esc_html__( 'Image', 'max-addons' ),
							],
							'inline'    => true,
							'clearable' => true,
							'default'   => '',
						],
						'cellIcon' => [
							'label'    => esc_html__( 'Icon', 'max-addons' ),
							'type'     => 'icon',
							'default'  => [
								'library' => 'themify',
								'icon'    => 'ti-wordpress',
							],
							'required' => [ 'cellIconType', '=', [ 'icon' ] ],
						],
						'cellImage' => [
							'label'    => esc_html__( 'Image', 'max-addons' ),
							'type'     => 'image',
							'required' => [ 'cellIconType', '=', [ 'image' ] ],
						],
						'cellIconPosition' => [
							'label'     => esc_html__( 'Icon Position', 'max-addons' ),
							'type'      => 'select',
							'options'   => [
								'before' => esc_html__( 'Before', 'max-addons' ),
								'after'  => esc_html__( 'After', 'max-addons' ),
							],
							'inline'    => true,
							'clearable' => true,
							'default'   => 'before',
							'required'  => [ 'cellIconType', '=', [ 'image', 'icon' ] ],
						],
					],
					'default'       => [
						[
							'cellText' => esc_html__( 'Column 1', 'max-addons' ),
						],
						[
							'cellText' => esc_html__( 'Column 2', 'max-addons' ),
						],
						[
							'cellText' => esc_html__( 'Column 3', 'max-addons' ),
						],
					],
				],
			],
			'default'       => [
				[
					'cell' => '',
				],
			],
			'required'      => [ 'source', '!=', [ 'query', 'google-sheets' ] ],
		];
	}

	// Set dynamic content controls
	public function set_dynamic_content_controls() {
		$this->controls['dynamicColumns'] = [
			'tab'           => 'content',
			'group'         => 'dynamicContent',
			'placeholder'   => esc_html__( 'Item', 'max-addons' ),
			'type'          => 'repeater',
			'titleProperty' => 'cell',
			'fields'        => [
				'columnHeading'         => [
					'label'          => esc_html__( 'Column Heading', 'max-addons' ),
					'type'           => 'text',
					'hasDynamicData' => 'text',
				],
				'columnData'         => [
					'label'          => esc_html__( 'Column Data', 'max-addons' ),
					'type'           => 'text',
					'hasDynamicData' => 'text',
				],
				'link'             => [
					'label'    => esc_html__( 'Link', 'max-addons' ),
					'type'     => 'link',
				],
				'cellIconType'     => [
					'label'     => esc_html__( 'Icon Type', 'max-addons' ),
					'type'      => 'select',
					'options'   => [
						'icon'  => esc_html__( 'Icon', 'max-addons' ),
						'image' => esc_html__( 'Image', 'max-addons' ),
					],
					'inline'    => true,
					'clearable' => true,
					'default'   => '',
				],
				'cellIcon'         => [
					'label'    => esc_html__( 'Icon', 'max-addons' ),
					'type'     => 'icon',
					'default'  => [
						'library' => 'themify',
						'icon'    => 'ti-wordpress',
					],
					'required' => [ 'cellIconType', '=', [ 'icon' ] ],
				],
				'cellImage'        => [
					'label'    => esc_html__( 'Image', 'max-addons' ),
					'type'     => 'image',
					'required' => [ 'cellIconType', '=', [ 'image' ] ],
				],
				'cellIconPosition' => [
					'label'     => esc_html__( 'Icon Position', 'max-addons' ),
					'type'      => 'select',
					'options'   => [
						'before' => esc_html__( 'Before', 'max-addons' ),
						'after'  => esc_html__( 'After', 'max-addons' ),
					],
					'inline'    => true,
					'clearable' => true,
					'default'   => 'before',
					'required' => [ 'cellIconType', '!=', '' ],
				],
			],
			'default'       => [
				[
					'columnHeading'    => esc_html__( 'Post Title', 'max-addons' ),
					'columnData'       => '{post_title}',
				],
				[
					'columnHeading'    => esc_html__( 'Post Author', 'max-addons' ),
					'columnData'       => '{author_name}',
				],
				[
					'columnHeading'    => esc_html__( 'Post Date', 'max-addons' ),
					'columnData'       => '{post_date}',
				],
			],
			'required' => [ 'source', '=', 'query' ],
		];
	}

	// Set search controls
	public function set_search_controls() {
		$this->controls['search'] = [
			'tab'   => 'content',
			'group' => 'search',
			'label' => esc_html__( 'Search form', 'max-addons' ),
			'type'  => 'checkbox',
		];

		$this->controls['searchInputWidth'] = [
			'tab'      => 'content',
			'group'    => 'search',
			'label'    => esc_html__( 'Width', 'max-addons' ),
			'type'     => 'number',
			'units'    => true,
			'css'      => [
				[
					'property' => 'width',
					'selector' => '.gridjs-search',
				],
			],
			'required' => [ 'search', '!=', '' ],
		];

		$this->controls['searchFormAlign'] = [
			'tab'         => 'content',
			'group'       => 'search',
			'label'       => esc_html__( 'Align', 'max-addons' ),
			'type'        => 'text-align',
			'css'         => [
				[
					'property' => 'float',
					'selector' => '.gridjs-search',
				],
			],
			'placeholder' => esc_html__( 'Left', 'max-addons' ),
			'exclude'     => [
				'center',
				'justify'
			],
			'inline'      => true,
			'required'    => [ 'search', '!=', '' ],
		];

		$this->controls['searchPlaceholder'] = [
			'tab'         => 'content',
			'group'       => 'search',
			'type'        => 'text',
			'label'       => esc_html__( 'Placefolder', 'max-addons' ),
			'placeholder' => esc_html__( 'Search...', 'max-addons' ),
			'inline'      => true,
			'required'    => [ 'search', '!=', '' ],
		];

		$this->controls['searchPlaceholderColor'] = [
			'tab'      => 'content',
			'group'    => 'search',
			'label'    => esc_html__( 'Placeholder color', 'max-addons' ),
			'type'     => 'color',
			'css'      => [
				[
					'property' => 'color',
					'selector' => '.gridjs-search-input::placeholder',
				],
			],
			'required' => [ 'search', '!=', '' ],
		];

		$this->controls['searchTypography'] = [
			'tab'      => 'content',
			'group'    => 'search',
			'type'     => 'typography',
			'label'    => esc_html__( 'Typography', 'max-addons' ),
			'css'      => [
				[
					'property' => 'font',
					'selector' => '.gridjs-search-input',
				],
			],
			'exclude'  => [
				'text-align',
			],
			'inline'   => true,
			'small'    => true,
			'required' => [ 'search', '!=', '' ],
		];

		$this->controls['searchInputBackgroundColor'] = [
			'tab'      => 'content',
			'group'    => 'search',
			'label'    => esc_html__( 'Background color', 'max-addons' ),
			'type'     => 'color',
			'css'      => [
				[
					'property' => 'background-color',
					'selector' => '.gridjs-search-input',
				],
			],
			'required' => [ 'search', '!=', '' ],
		];

		$this->controls['searchInputBorder'] = [
			'tab'      => 'content',
			'group'    => 'search',
			'label'    => esc_html__( 'Border', 'max-addons' ),
			'type'     => 'border',
			'css'      => [
				[
					'property' => 'border',
					'selector' => '.gridjs-search-input',
				],
			],
			'required' => [ 'search', '!=', '' ],
		];

		$this->controls['searchInputBoxShadow'] = [
			'tab'      => 'content',
			'group'    => 'search',
			'label'    => esc_html__( 'Box shadow', 'max-addons' ),
			'type'     => 'box-shadow',
			'css'      => [
				[
					'property' => 'box-shadow',
					'selector' => '.gridjs-search-input',
				],
			],
			'required' => [ 'search', '!=', '' ],
		];

		$this->controls['searchInputMargin'] = [
			'tab'   => 'content',
			'group' => 'search',
			'type'  => 'spacing',
			'label' => esc_html__( 'Margin', 'max-addons' ),
			'css'   => [
				[
					'property' => 'margin',
					'selector' => '.gridjs-search-input',
				],
			],
			'required' => [ 'search', '!=', '' ],
		];

		$this->controls['searchInputPadding'] = [
			'tab'   => 'content',
			'group' => 'search',
			'type'  => 'spacing',
			'label' => esc_html__( 'Padding', 'max-addons' ),
			'css'   => [
				[
					'property' => 'padding',
					'selector' => '.gridjs-search-input',
				],
			],
			'required' => [ 'search', '!=', '' ],
		];
	}

	// Set pagination controls
	public function set_pagination_controls() {
		$this->controls['pagination'] = [
			'tab'   => 'content',
			'group' => 'pagination',
			'label' => esc_html__( 'Pagination', 'max-addons' ),
			'type'  => 'checkbox',
		];

		$this->controls['limit'] = [
			'tab'         => 'content',
			'group'       => 'pagination',
			'label'       => esc_html__( 'Rows per page', 'max-addons' ),
			'type'        => 'number',
			'min'         => 1,
			'step'        => 1,
			'inline'      => true,
			'placeholder' => 10,
			'required'    => [ 'pagination', '!=', '' ],
		];

		$this->controls['buttonsCount'] = [
			'tab'         => 'content',
			'group'       => 'pagination',
			'label'       => esc_html__( 'Buttons count', 'max-addons' ),
			'type'        => 'number',
			'min'         => 1,
			'step'        => 1,
			'inline'      => true,
			'placeholder' => 3,
			'required'    => [ 'pagination', '!=', '' ],
		];

		// Pagination Buttons
		$this->controls['paginationButtonsSep'] = [
			'tab'      => 'content',
			'group'    => 'pagination',
			'type'     => 'separator',
			'label'    => esc_html__( 'Buttons', 'max-addons' ),
			'required' => [ 'pagination', '!=', '' ],
		];

		$this->controls['nextButton'] = [
			'tab'     => 'content',
			'group'   => 'pagination',
			'label'   => esc_html__( 'Next button', 'max-addons' ),
			'type'    => 'checkbox',
			'default' => true,
			'required' => [ 'pagination', '!=', '' ],
		];

		$this->controls['nextButtonText'] = [
			'tab'      => 'content',
			'group'    => 'pagination',
			'type'     => 'text',
			'label'    => esc_html__( 'Next button text', 'max-addons' ),
			'placeholder' => esc_html__( 'Next', 'max-addons' ),
			'inline'   => true,
			'required' => [
				[ 'pagination', '!=', '' ],
				[ 'nextButton', '!=', '' ]
			],
		];

		$this->controls['prevButton'] = [
			'tab'     => 'content',
			'group'   => 'pagination',
			'label'   => esc_html__( 'Previous button', 'max-addons' ),
			'type'    => 'checkbox',
			'default' => true,
			'required' => [ 'pagination', '!=', '' ],
		];

		$this->controls['prevButtonText'] = [
			'tab'      => 'content',
			'group'    => 'pagination',
			'type'     => 'text',
			'label'    => esc_html__( 'Previous button text', 'max-addons' ),
			'placeholder' => esc_html__( 'Previous', 'max-addons' ),
			'inline'   => true,
			'required' => [
				[ 'pagination', '!=', '' ],
				[ 'prevButton', '!=', '' ]
			],
		];

		// Summary
		$this->controls['summarySep'] = [
			'tab'      => 'content',
			'group'    => 'pagination',
			'type'     => 'separator',
			'label'    => esc_html__( 'Summary', 'max-addons' ),
			'required' => [ 'pagination', '!=', '' ],
		];

		$this->controls['summary'] = [
			'tab'     => 'content',
			'group'   => 'pagination',
			'label'   => esc_html__( 'Summary', 'max-addons' ),
			'type'    => 'checkbox',
			'default' => true,
			'required' => [ 'pagination', '!=', '' ],
		];

		$this->controls['showingText'] = [
			'tab'      => 'content',
			'group'    => 'pagination',
			'type'     => 'text',
			'label'    => esc_html__( 'Showing text', 'max-addons' ),
			'placeholder' => esc_html__( 'Showing', 'max-addons' ),
			'inline'   => true,
			'required' => [
				[ 'pagination', '!=', '' ],
				[ 'summary', '!=', '' ]
			],
		];

		$this->controls['resultsText'] = [
			'tab'      => 'content',
			'group'    => 'pagination',
			'type'     => 'text',
			'label'    => esc_html__( 'Results text', 'max-addons' ),
			'placeholder' => esc_html__( 'results', 'max-addons' ),
			'inline'   => true,
			'required' => [
				[ 'pagination', '!=', '' ],
				[ 'summary', '!=', '' ]
			],
		];
	}

	// Set header style controls
	public function set_header_style_controls() {

		$this->controls['headerPadding'] = [
			'tab'   => 'content',
			'group' => 'headerStyle',
			'type'  => 'spacing',
			'label' => esc_html__( 'Padding', 'max-addons' ),
			'css'   => [
				[
					'property' => 'padding',
					'selector' => 'th.gridjs-th',
				]
			],
		];

		$this->controls['headerTypography'] = [
			'tab'    => 'content',
			'group'  => 'headerStyle',
			'label'  => esc_html__( 'Typography', 'max-addons' ),
			'type'   => 'typography',
			'css'    => [
				[
					'property' => 'font',
					'selector' => 'th.gridjs-th',
				],
				[
					'property' => 'font',
					'selector' => '.mab-stacked-table td:before',
				],
			],
			'inline' => true,
			'small'  => true,
		];

		$this->controls['headerBgColor'] = [
			'tab'     => 'content',
			'group'   => 'headerStyle',
			'type'    => 'color',
			'label'   => esc_html__( 'Background Color', 'max-addons' ),
			'css'     => [
				[
					'property' => 'background-color',
					'selector' => 'th.gridjs-th',
				],
				[
					'property' => 'background-color',
					'selector' => '.mab-stacked-table td:before',
				],
			],
			'inline'  => true,
			'small'   => true,
		];

		$this->controls['headerBorder'] = [
			'tab'     => 'content',
			'group'   => 'headerStyle',
			'label'   => esc_html__( 'Border', 'max-addons' ),
			'type'    => 'border',
			'css'     => [
				[
					'property' => 'border',
					'selector' => 'th.gridjs-th',
				],
				[
					'property' => 'border',
					'selector' => '.mab-stacked-table td:before',
				],
			],
			'inline'  => true,
			'small'   => true,
		];
	}

	// Set rows controls
	public function set_rows_controls() {
		$this->controls['stripedRows'] = [
			'tab'   => 'content',
			'group' => 'rows',
			'label' => esc_html__( 'Striped Rows', 'max-addons' ),
			'type'  => 'checkbox',
		];

		$this->controls['oddRowsColSeparator'] = [
			'tab'      => 'content',
			'group'    => 'rows',
			'type'     => 'separator',
			'label'    => esc_html__( 'Odd Rows', 'max-addons' ),
			'required' => [ 'stripedRows', '!=', '' ],
		];

		$this->controls['rowsBgColor'] = [
			'tab'      => 'content',
			'group'    => 'rows',
			'type'     => 'color',
			'label'    => esc_html__( 'Background Color', 'max-addons' ),
			'css'      => [
				[
					'property' => 'background-color',
					'selector' => 'tr',
				],
			],
			'inline'   => true,
			'small'    => true,
		];

		$this->controls['rowsTextColor'] = [
			'tab'      => 'content',
			'group'    => 'rows',
			'type'     => 'color',
			'label'    => esc_html__( 'Text Color', 'max-addons' ),
			'css'      => [
				[
					'property' => 'color',
					'selector' => 'tr .mab-table-cell-content',
				],
			],
			'inline'   => true,
			'small'    => true,
		];

		$this->controls['rowsBorder'] = [
			'tab'      => 'content',
			'group'    => 'rows',
			'label'    => esc_html__( 'Border', 'max-addons' ),
			'type'     => 'border',
			'css'      => [
				[
					'property' => 'border',
					'selector' => 'tr',
				],
			],
			'inline'   => true,
			'small'    => true,
		];

		// Even Rows Style
		$this->controls['evenRowsColSeparator'] = [
			'tab'      => 'content',
			'group'    => 'rows',
			'type'     => 'separator',
			'label'    => esc_html__( 'Even Rows', 'max-addons' ),
			'required' => [ 'stripedRows', '!=', '' ],
		];

		$this->controls['evenRowsBgColor'] = [
			'tab'      => 'content',
			'group'    => 'rows',
			'type'     => 'color',
			'label'    => esc_html__( 'Background Color', 'max-addons' ),
			'css'      => [
				[
					'property' => 'background-color',
					'selector' => 'tr:nth-child(even)',
				],
			],
			'inline'   => true,
			'small'    => true,
			'required' => [ 'stripedRows', '!=', '' ],
		];

		$this->controls['evenRowsTextColor'] = [
			'tab'      => 'content',
			'group'    => 'rows',
			'type'     => 'color',
			'label'    => esc_html__( 'Text Color', 'max-addons' ),
			'css'      => [
				[
					'property' => 'color',
					'selector' => 'tr:nth-child(even) .mab-table-cell-content',
				],
			],
			'inline'   => true,
			'small'    => true,
			'required' => [ 'stripedRows', '!=', '' ],
		];

		$this->controls['evenRowsBorder'] = [
			'tab'      => 'content',
			'group'    => 'rows',
			'label'    => esc_html__( 'Border', 'max-addons' ),
			'type'     => 'border',
			'css'      => [
				[
					'property' => 'border',
					'selector' => 'tr:nth-child(even)',
				],
			],
			'inline'   => true,
			'small'    => true,
			'required' => [ 'stripedRows', '!=', '' ],
		];
	}

	// Set cells style controls
	public function set_cells_controls() {
		$this->controls['cellPadding'] = [
			'tab'   => 'content',
			'group' => 'cells',
			'type'  => 'spacing',
			'label' => esc_html__( 'Padding', 'max-addons' ),
			'css'   => [
				[
					'property' => 'padding',
					'selector' => 'td.gridjs-td > *',
				],
				[
					'property' => 'padding',
					'selector' => '.mab-stacked-table td.gridjs-td:before',
				],
			],
		];

		$this->controls['cellTypography'] = [
			'tab'    => 'content',
			'group'  => 'cells',
			'label'  => esc_html__( 'Typography', 'max-addons' ),
			'type'   => 'typography',
			'css'    => [
				[
					'property' => 'font',
					'selector' => 'td.gridjs-td',
				],
			],
			'inline' => true,
			'small'  => true,
		];

		$this->controls['cellBgColor'] = [
			'tab'    => 'content',
			'group'  => 'cells',
			'type'   => 'color',
			'label'  => esc_html__( 'Background Color', 'max-addons' ),
			'css'    => [
				[
					'property' => 'background-color',
					'selector' => 'td.gridjs-td',
				],
			],
			'inline' => true,
			'small'  => true,
		];

		$this->controls['cellBorder'] = [
			'tab'     => 'content',
			'group'   => 'cells',
			'label'   => esc_html__( 'Border', 'max-addons' ),
			'type'    => 'border',
			'css'     => [
				[
					'property' => 'border',
					'selector' => 'td.gridjs-td',
				],
			],
			'default' => [
				'width' => [
					'top'    => 0,
					'right'  => 1,
					'bottom' => 1,
					'left'   => 0,
				],
				'style' => 'solid',
				'color' => [
					'rgb' => 'rgba(0, 0, 0, 0.1)',
				],
			],
			'inline'  => true,
			'small'   => true,
		];
	}

	// Set icon controls
	public function set_icon_controls() {
		$this->controls['iconSpacing'] = [
			'tab'   => 'content',
			'group' => 'iconStyle',
			'label' => esc_html__( 'Icon Spacing', 'max-addons' ),
			'type'  => 'number',
			'units' => true,
			'css'   => [
				[
					'property' => 'margin-right',
					'selector' => '.mab-table-cell-icon-before',
				],
				[
					'property' => 'margin-left',
					'selector' => '.mab-table-cell-icon-after',
				],
			],
		];

		$this->controls['iconTypography'] = [
			'tab'     => 'content',
			'group'   => 'iconStyle',
			'label'   => esc_html__( 'Icon Typography', 'max-addons' ),
			'type'    => 'typography',
			'css'     => [
				[
					'property' => 'font',
					'selector' => '.mab-table-cell .mab-table-cell-icon',
				],
			],
			'exclude' => [
				'font-family',
				'font-weight',
				'font-style',
				'text-align',
				'letter-spacing',
				'line-height',
				'text-transform',
				'text-decoration',
			],
			'inline'  => true,
			'small'   => true,
		];

		$this->controls['imageSize'] = [
			'tab'     => 'content',
			'group'   => 'iconStyle',
			'label'   => esc_html__( 'Image Size', 'max-addons' ),
			'type'    => 'number',
			'units'   => true,
			'css'     => [
				[
					'property' => 'width',
					'selector' => '.mab-table-cell-icon img',
				],
			],
			'default' => '100px',
		];
	}

	// Set button controls
	public function set_button_controls() {
		$this->controls['buttonTypography'] = [
			'tab'      => 'content',
			'group'    => 'buttonStyle',
			'label'    => esc_html__( 'Typography', 'max-addons' ),
			'type'     => 'typography',
			'css'      => [
				[
					'property' => 'font',
					'selector' => 'td.gridjs-td .bricks-button',
				],
			],
			'inline'   => true,
			'required' => [ 'source', '=', 'custom' ],
		];

		$this->controls['buttonBackgroundColor'] = [
			'tab'      => 'content',
			'group'    => 'buttonStyle',
			'label'    => esc_html__( 'Background', 'max-addons' ),
			'type'     => 'color',
			'css'      => [
				[
					'property' => 'background-color',
					'selector' => 'td.gridjs-td .bricks-button',
				],
			],
			'inline'   => true,
			'small'    => true,
			'required' => [ 'source', '=', 'custom' ],
		];

		$this->controls['buttonBorder'] = [
			'tab'      => 'content',
			'group'    => 'buttonStyle',
			'label'    => esc_html__( 'Border', 'max-addons' ),
			'type'     => 'border',
			'inline'   => true,
			'small'    => true,
			'css'      => [
				[
					'property' => 'border',
					'selector' => 'td.gridjs-td .bricks-button',
				],
			],
			'required' => [ 'source', '=', 'custom' ],
		];

		$this->controls['buttonBoxShadow'] = [
			'tab'      => 'content',
			'group'    => 'buttonStyle',
			'label'    => esc_html__( 'Box Shadow', 'max-addons' ),
			'type'     => 'box-shadow',
			'css'      => [
				[
					'property' => 'box-shadow',
					'selector' => 'td.gridjs-td .bricks-button',
				],
			],
			'inline'   => true,
			'small'    => true,
			'required' => [ 'source', '=', 'custom' ],
		];
	}

	public function get_normalized_image_settings( $settings ) {
		if ( ! isset( $settings['cellImage'] ) ) {
			$settings['cellImage'] = [
				'id'  => 0,
				'url' => '',
			];
			return $settings;
		}

		$image = $settings['cellImage'];

		if ( isset( $image['useDynamicData']['name'] ) ) {
			$images = $this->render_dynamic_data_tag( $image['useDynamicData']['name'], $image );
			$image['id'] = empty( $images ) ? 0 : $images[0];
		} else {
			$image['id'] = isset( $image['id'] ) ? $image['id'] : 0;
		}

		// Image Size
		$image['size'] = isset( $image['size'] ) ? $settings['cellImage']['size'] : BRICKS_DEFAULT_IMAGE_SIZE;

		// Image URL
		if ( ! isset( $image['url'] ) ) {
			$image['url'] = wp_get_attachment_image_url( $image['id'], $image['size'] );
		}

		$settings['cellImage'] = $image;

		return $settings;
	}

	/**
	 * Gets expire time of transient.
	 *
	 * @since 1.12.0
	 * @param array $settings The settings array.
	 * @return the transient expire time.
	 * @access public
	 */
	public function get_transient_expire( $settings ) {
		$expire_value = isset( $settings['cacheDuration'] ) ? $settings['cacheDuration'] : 'day';
		$expire_time  = 24 * HOUR_IN_SECONDS;

		if ( 'hour' === $expire_value ) {
			$expire_time = 60 * MINUTE_IN_SECONDS;
		} elseif ( 'week' === $expire_value ) {
			$expire_time = 7 * DAY_IN_SECONDS;
		} elseif ( 'month' === $expire_value ) {
			$expire_time = 30 * DAY_IN_SECONDS;
		} elseif ( 'year' === $expire_value ) {
			$expire_time = 365 * DAY_IN_SECONDS;
		}

		return $expire_time;
	}

	/**
	 * Retrieve Google Sheets API from Max Addons settings.
	 *
	 * @since 1.12.0
	 * @return string
	 */
	private function get_google_sheets_api() {
		return \MaxAddonsPro\Classes\Admin_Settings::get_option( 'mab_google_sheets_api_key' );
	}

	/**
	 * Fetch data from Google Sheets API.
	 *
	 * @return array|\WP_Error API response or error object.
	 */
	private function get_google_sheets_data() {
		$settings = $this->settings;
		$api_key  = $this->get_google_sheets_api();

		if ( empty( $api_key ) ) {
			return array(
				'data'  => array(),
				'error' => new \WP_Error(
					'missing_api_key',
					esc_html__( 'To fetch data from Google Sheets, you need to set up an API key.', 'max-addons' )
				),
			);
		}

		$sheet_id       = ! empty( $settings['googleSheetsUrl'] ) ? $this->extract_sheet_id( $settings['googleSheetsUrl'] ) : '';
		$cell_range     = ! empty( $settings['cellRange'] ) ? $settings['cellRange'] : '';
		$cache_duration = $this->get_transient_expire( $settings );
		$url            = sprintf( 'https://sheets.googleapis.com/v4/spreadsheets/%s/values/%s?key=%s', sanitize_text_field( $sheet_id ), $cell_range, sanitize_text_field( $api_key ) );

		if ( empty( $sheet_id ) ) {
			return array(
				'data'  => array(),
				'error' => new \WP_Error(
					'invalid_google_sheets_url',
					esc_html__( 'Please provide a valid Google Sheets URL.', 'max-addons' )
				),
			);
		}

		if ( empty( $cell_range ) ) {
			return array(
				'data'  => array(),
				'error' => new \WP_Error(
					'empty_google_sheets_cell_range',
					esc_html__( 'Please enter table cell range for provided Google Sheets URL.', 'max-addons' )
				),
			);
		}

		// Generate a unique transient key based on the sheet ID
		$transient_key_data = 'mab_google_sheets_data_' . md5( $api_key . $sheet_id . $cache_duration );
		$transient_key_url  = 'mab_google_sheets_url_' . md5( $api_key . $sheet_id . $cache_duration );

		// Check if cached data exists
		$cached_data = get_transient( $transient_key_data );
		$cached_url  = get_transient( $transient_key_url );

		if ( false !== $cached_data && $url === $cached_url ) {
			return array(
				'data'  => $cached_data,
				'error' => false,
			);
		}

		$response = wp_remote_get( esc_url_raw( $url ) );

		if ( is_wp_error( $response ) ) {
			return array(
				'data'  => array(),
				'error' => $response,
			);
		}

		if ( 200 !== wp_remote_retrieve_response_code( $response ) ) {
			return array(
				'data'  => array(),
				'error' => new \WP_Error(
					'google_sheets_fetch_error',
					esc_html__( 'Failed to fetch data from Google Sheets.', 'max-addons' )
				),
			);
		}

		$body = wp_remote_retrieve_body( $response );

		if ( empty( $body ) ) {
			return array(
				'data'  => array(),
				'error' => new \WP_Error(
					'google_sheets_empty',
					esc_html__( 'The provided Google Sheet is empty.', 'max-addons' )
				),
			);
		}

		$data = json_decode( $body, true );

		$response_data = array(
			'data'  => isset( $data['values'] ) ? $data['values'] : array(),
			'error' => false,
		);

		// Cache the data in a transient
		set_transient( $transient_key_data, $response_data['data'], intval( $cache_duration ) );
		set_transient( $transient_key_url, $url, intval( $cache_duration ) );

		return $response_data;
	}

	/**
	 * Extract Google Sheets ID from URL.
	 *
	 * @param string $url Google Sheets URL.
	 * @return string Google Sheets ID.
	 */
	private function extract_sheet_id( $url ) {
		preg_match( '/\/spreadsheets\/d\/([a-zA-Z0-9-_]+)/', $url, $matches );
		return isset( $matches[1] ) ? $matches[1] : '';
	}

	public function render_table_image( $settings ) {
		if ( isset( $settings['cellImage']['useDynamicData']['name'] ) ) {

			if ( empty( $settings['cellImage']['id'] ) ) {

				if ( 'ACF' === $settings['cellImage']['useDynamicData']['group'] && ! class_exists( 'ACF' ) ) {
					$message = esc_html__( 'Can\'t render element, as the selected ACF field is not available. Please activate ACF or edit the element to select different data.', 'max-addons' );
				} elseif ( '{featured_image}' == $settings['cellImage']['useDynamicData']['name'] ) {
					$message = esc_html__( 'No featured image set.', 'max-addons' );
				} else {
					$message = esc_html__( 'Dynamic Data %1$s (%2$s) is empty.', 'max-addons' );
				}

				return $this->render_element_placeholder( [
					'icon-class' => 'ti-image',
					'text'       => sprintf(
						$message,
						$settings['cellImage']['useDynamicData']['label'],
						$settings['cellImage']['useDynamicData']['group']
					),
				] );
			}
		}

		// Image id is empty or doesn't exist
		else {

			// No image
			if ( empty( $settings['cellImage']['id'] ) ) {
				return $this->render_element_placeholder( [
					'icon-class' => 'ti-image',
					'text'       => esc_html__( 'No image selected.', 'max-addons' ),
				] );
			}

			// Return if image ID does not exist
			if ( ! wp_get_attachment_image_src( $settings['cellImage']['id'] ) ) {
				return $this->render_element_placeholder( [
					'icon-class' => 'ti-image',
					'text'       => sprintf( esc_html__( 'Image ID (%s) no longer exist. Please select another image.', 'max-addons' ), $settings['cellImage']['id'] ),
				] );
			}
		}

		// Render
		$img_html = '';
		$image_atts = [];
		$image_atts['id'] = 'image-' . $settings['cellImage']['id'];

		$image_wrapper_classes = [ 'mab-table-icon-image' ];
		$img_classes = [ 'post-thumbnail', 'css-filter' ];

		$img_classes[] = 'size-' . $settings['cellImage']['size'];
		$image_atts['class'] = join( ' ', $img_classes );

		$this->set_attribute( 'image-wrapper', 'class', $image_wrapper_classes );

		$img_html .= '<div ' . $this->render_attributes( 'image-wrapper' ) . '>';

		// Lazy load atts set via 'wp_get_attachment_image_attributes' filter
		if ( isset( $settings['cellImage']['id'] ) ) {
			$img_html .= wp_get_attachment_image( $settings['cellImage']['id'], $settings['cellImage']['size'], false, $image_atts );
		} elseif ( ! empty( $settings['cellImage']['url'] ) ) {
			$img_html .= '<img src="' . $settings['cellImage']['url'] . '">';
		}

		$img_html .= '</div>';

		return $img_html;
	}

	// Table Header
	public function render_header_content( $item, $loop_index, $is_google_sheets = false ) {
		$settings           = $this->settings;
		$cell_key           = "header_cell_key_$loop_index";
		$cell_text_key      = "header_cell_text_key_$loop_index";
		$cell_icon_wrap_key = "header_icon_wrap_key_$loop_index";
		$cell_icon_key      = "header_icon_key_$loop_index";

		$this->set_attribute( $cell_text_key, 'class', 'mab-table-cell-text' );
		$this->set_attribute( $cell_key, 'class', 'mab-table-cell' );

		if ( ! $is_google_sheets ) {
			$attributes = isset( $item['columnAttributes'] ) ? $item['columnAttributes'] : [];
			$column_attributes = [];

			$column_width = ! empty( $item['columnWidth'] ) ? $item['columnWidth'] : '';
			$disable_sorting = ! empty( $item['disableSorting'] ) ? $item['disableSorting'] : false;

			if ( '' !== $column_width ) {
				$this->set_attribute( $cell_key, 'data-column-width', esc_attr( $column_width ) );
			}

			if ( $disable_sorting ) {
				$this->set_attribute( $cell_key, 'data-column-sort', esc_attr( $disable_sorting ) );
			}

			if ( ! empty( $attributes ) ) {
				foreach ( $attributes as $attribute ) {
					$name  = isset( $attribute['name'] ) ? $attribute['name'] : '';
					$value = isset( $attribute['value'] ) ? $attribute['value'] : '';

					if ( $name ) {
						$column_attributes[ $name ] = $value;
					}
				}

				if ( ! empty( $column_attributes ) ) {
					$this->set_attribute( $cell_key, 'data-column-attributes', wp_json_encode( $column_attributes ) );
				}
			}

			if ( ! empty( $item['cellIconType'] ) ) {
				$this->set_attribute( $cell_icon_wrap_key, 'class', 'mab-table-cell-icon' );

				$icon_position = ! empty( $item['cellIconPosition'] ) ? esc_attr( $item['cellIconPosition'] ) : 'before';
				$this->set_attribute( $cell_icon_wrap_key, 'class', 'mab-table-cell-icon-' . $icon_position );

				if ( ! empty( $item['cellIcon']['icon'] ) ) {
					$this->set_attribute( "icon-$loop_index", 'class', [
						'mab-icon',
						$item['cellIcon']['icon'], // Dont' sanitize_html_class (Font Awesome uses two class names, so we need spaces)
					] );
				}
			}
		}

		echo '<th ' . wp_kses_post( $this->render_attributes( $cell_key ) ) . '>';

		if ( ! empty( $item['cellIconType'] ) ) {
			echo '<span ' . wp_kses_post( $this->render_attributes( $cell_icon_wrap_key ) ) . '>';
			if ( 'image' === $item['cellIconType'] ) {
				$image_settings = $this->get_normalized_image_settings( $item );
				echo $this->render_table_image( $image_settings );
			} elseif ( 'icon' === $item['cellIconType'] && ! empty( $item['cellIcon']['icon'] ) ) {
				echo '<i ' . wp_kses_post( $this->render_attributes( "icon-$loop_index" ) ) . '></i>';
			}
			echo '</span>';
		}

		echo '<span ' . wp_kses_post( $this->render_attributes( $cell_text_key ) ) . '>';

		if ( isset( $settings['source'] ) && 'query' === $settings['source'] ) {
			if ( ! empty( $item['columnHeading'] ) ) {
				echo esc_html( $item['columnHeading'] );
			}
		} else {
			if ( ! empty( $item['cellText'] ) ) {
				echo $this->render_dynamic_data( $item['cellText'] );
			}
		}

		echo '</span>';
		echo '</th>';
	}

	// Table Header Custom
	public function render_header_custom( $item, $is_google_sheets = false ) {
		$settings   = $this->settings;
		$loop_index = $this->header_loop_index;
		ob_start();

		self::render_header_content( $item, $loop_index, $is_google_sheets );

		$html = ob_get_clean();
		$this->header_loop_index++;

		return $html;
	}

	// Table Header Dynamic
	public function render_header_dynamic( $item ) {
		$settings   = $this->settings;
		$loop_index = $this->header_loop_index;
		ob_start();

		$columns = ! empty( $settings['dynamicColumns'] ) ? $settings['dynamicColumns'] : [];

		foreach ( $columns as $index => $column_item ) {
			if ( 0 === $loop_index ) {
				if ( ! empty( $column_item['columnHeading'] ) || ! empty( $column_item['cellIconType'] ) ) {
					self::render_header_content( $column_item, $index );
				}
			}
		}

		$html = ob_get_clean();
		$this->header_loop_index++;

		return $html;
	}

	// Table Header
	protected function render_header() {
		$settings = $this->settings;
		$this->set_attribute( 'row', 'class', 'mab-table-row' );

		$columns = ! empty( $settings['dynamicColumns'] ) ? $settings['dynamicColumns'] : [];
		?>
		<thead>
			<tr <?php echo wp_kses_post( $this->render_attributes( 'row' ) ); ?>>
				<?php
				$output = '';

				// Query Loop
				if ( isset( $settings['source'] ) && 'query' === $settings['source'] ) {

					$query = new \Bricks\Query(
						[
							'id'       => $this->id,
							'settings' => $settings,
						]
					);

					$item = $columns[0];

					$output .= $query->render( [ $this, 'render_header_dynamic' ], compact( 'item' ) );

					// Destroy the Query to explicitly remove it from the global store
					$query->destroy();
					unset( $query );

				} elseif ( isset( $settings['source'] ) && 'google-sheets' === $settings['source'] ) {
					$sheet_data = $this->get_google_sheets_data();
					$cell_data  = [];

					if ( ! empty( $sheet_data['data'] ) ) {
						foreach ( $sheet_data['data'][0] as $index => $item ) {
							$cell_data['cellTextType'] = 'basic';
							$cell_data['cellText']     = $item;

							$output .= self::render_header_custom( $cell_data, true );
						}
					}

				} else {
					if ( ! empty( $settings['columns'] ) ) {
						foreach ( $settings['columns'] as $index => $item ) {
							$output .= self::render_header_custom( $item );
						}
					}
				}

				echo $output;
				?>
			</tr>
		</thead>
		<?php
	}

	// Table Body
	protected function render_body() {
		$settings        = $this->settings;
		$columns_dynamic = ! empty( $settings['dynamicColumns'] ) ? $settings['dynamicColumns'] : false;
		$columns_static  = ! empty( $settings['columns'] ) ? $settings['columns'] : [];
		?>
		<tbody>
			<?php
			$output = '';

			// Query Loop
			if ( isset( $settings['source'] ) && 'query' === $settings['source'] ) {	
				$query = new \Bricks\Query(
					[
						'id'       => $this->id,
						'settings' => $settings,
					]
				);

				$item = $columns_dynamic[0];

				$output .= $query->render( [ $this, 'render_body_content_dynamic' ], compact( 'item' ) );

				// Destroy the Query to explicitly remove it from the global store.
				$query->destroy();
				unset( $query );

			} elseif ( isset( $settings['source'] ) && 'google-sheets' === $settings['source'] ) {
				$google_sheet_data = $this->get_google_sheets_data();
				$item              = [];
				$cell_data         = [];

				if ( ! empty( $google_sheet_data ) ) {
					$sheet_data     = $google_sheet_data['data'];
					$header_columns = $sheet_data[0];

					foreach ( $sheet_data as $index => $table_body ) {
						if ( 0 === $index ) {
							continue;
						}

						$item['cell'] = $table_body;

						$output .= '<tr class="mab-table-row">';
							if ( ! empty( $item['cell'] ) ) {
								foreach ( array_slice( $item['cell'], 0, count( $header_columns ) ) as $key => $cell ) {
									$cell_data['cellTextType'] = 'basic';
									$cell_data['cellText']     = $cell;

									$output .= self::render_body_content_custom( $cell_data );
								}

								if ( count( $item['cell'] ) < count( $header_columns ) ) {
									$output .= str_repeat( '<td></td>', count( $header_columns ) - count( $item['cell'] ) );
								}
							} else {
								$output .= str_repeat( '<td></td>', count( $header_columns ) );
							}
						$output .= '</tr>';
					}
				}
			} else {
				if ( ! empty( $settings['bodyRows'] ) ) {
					foreach ( $settings['bodyRows'] as $index => $item ) {
						$output .= '<tr class="mab-table-row">';
						if ( ! empty( $item['cell'] ) ) {
							foreach ( array_slice( $item['cell'], 0, count( $columns_static ) ) as $key => $cell ) {
								$output .= self::render_body_content_custom( $cell );
							}

							if ( count( $item['cell'] ) < count( $columns_static ) ) {
								$output .= str_repeat( '<td></td>', count( $columns_static ) - count( $item['cell'] ) );
							}
						} else {
							$output .= str_repeat( '<td></td>', count( $columns_static ) );
						}
						$output .= '</tr>';
					}
				}
			}

			echo $output;
			?>
		</tbody>
		<?php
	}

	// Table Body
	public function render_body_content_custom( $item ) {
		$settings   = $this->settings;
		$loop_index = $this->loop_index;

		ob_start();

		$row_key            = "body_row_key_$loop_index";
		$cell_key           = "body_cell_key_$loop_index";
		$cell_text_key      = "body_cell_text_key_$loop_index";
		$cell_icon_wrap_key = "body_icon_wrap_key_$loop_index";
		$cell_icon_key      = "body_icon_key_$loop_index";
		$button_key         = "body_button_key_$loop_index";
		$button_tag         = 'span';

		$this->set_attribute( $cell_text_key, 'class', 'mab-table-cell-text' );
		$this->set_attribute( $row_key, 'class', 'mab-table-row' );
		$this->set_attribute( $cell_key, 'class', 'mab-table-cell' );
		$this->set_attribute( $button_key, 'class', 'bricks-button' );
		$this->set_attribute( $button_key, 'class', 'bricks-background-primary' );

		if ( ! empty( $item['cellIconType'] ) ) {
			$this->set_attribute( $cell_icon_wrap_key, 'class', 'mab-table-cell-icon' );

			$icon_position = ! empty( $item['cellIconPosition'] ) ? esc_attr( $item['cellIconPosition'] ) : 'before';
			$this->set_attribute( $cell_icon_wrap_key, 'class', 'mab-table-cell-icon-' . $icon_position );

			if ( ! empty( $item['cellIcon']['icon'] ) ) {
				$this->set_attribute( "icon-$loop_index", 'class', [
					'mab-icon',
					$item['cellIcon']['icon'],
				] );
			}
		}

		$apply_link_to = ( isset( $item['applyLinkTo'] ) ) ? $item['applyLinkTo'] : 'button';

		if ( ! empty( $item['link'] ) ) {
			$link_element = 'col-link-' . $loop_index;
	
			if ( 'button' === $apply_link_to ) {
				$button_tag   = 'a';
				$link_element = $button_key;
			}

			$this->set_link_attributes( $link_element, $item['link'] );
		}

		// if ( isset( $item['cellText'] ) || ! empty( $item['cellIconType'] ) ) {
			echo '<td ' . $this->render_attributes( $cell_key ) . '>';

			if ( ! empty( $item['link'] ) && 'cell' === $apply_link_to ) {
				echo '<a ' . $this->render_attributes( 'col-link-' . $loop_index ) . '>';
			}

			if ( ! empty( $item['cellIconType'] ) ) {
				echo '<span ' . $this->render_attributes( $cell_icon_wrap_key ) . '>';
				if ( 'icon' === $item['cellIconType'] && ! empty( $item['cellIcon']['icon'] ) ) {
					echo '<i ' . wp_kses_post( $this->render_attributes( "icon-$loop_index" ) ) . '></i>';
				} elseif ( 'image' === $item['cellIconType'] ) {
					$image_settings = $this->get_normalized_image_settings( $item );
					echo $this->render_table_image( $image_settings );
				}
				echo '</span>';
			}

			$cell_text      = '';
			$cell_text_type = ! empty( $item['cellTextType'] ) ? $item['cellTextType'] : 'basic';

			if ( 'rich' === $cell_text_type && ! empty( $item['cellTextRich'] ) ) {
				$cell_text = $item['cellTextRich'];
			} elseif ( ! empty( $item['cellText'] ) ) {
				$cell_text = $item['cellText'];
			}

			if ( $cell_text ) {
				echo '<span ' . $this->render_attributes( $cell_text_key ) . '>';
				echo wp_kses_post( $this->render_dynamic_data( $cell_text ) );
				echo '</span>';
			}

			if ( ! empty( $item['showButton'] ) && ! empty( $item['buttonText'] ) ) {
				echo '<' . esc_html( $button_tag ) . ' ' . $this->render_attributes( $button_key ) . '>';
				echo esc_html( $item['buttonText'] );
				echo '</' . esc_html( $button_tag ) . '>';
			}

			if ( ! empty( $item['link'] ) && 'cell' === $apply_link_to ) {
				echo '</a>';
			}

			echo '</td>';
		// }

		$html = ob_get_clean();

		$this->loop_index++;

		return $html;
	}

	// Table Body
	public function render_body_content_dynamic( $item ) {
		$settings   = $this->settings;
		$loop_index = $this->loop_index;
		ob_start();

		$row_key            = "body_row_key_$loop_index";
		$cell_key           = "body_cell_key_$loop_index";
		$cell_text_key      = "body_cell_text_key_$loop_index";
		$cell_icon_wrap_key = "body_icon_wrap_key_$loop_index";
		$cell_icon_key      = "body_icon_key_$loop_index";
		$link_key           = "body_link_key_$loop_index";

		$this->set_attribute( $cell_text_key, 'class', 'mab-table-cell-text' );
		$this->set_attribute( $row_key, 'class', 'mab-table-row' );
		$this->set_attribute( $cell_key, 'class', 'mab-table-cell' );

		$columns = ! empty( $settings['dynamicColumns'] ) ? $settings['dynamicColumns'] : [];

		if ( 0 === $this->loop_index ) {
			echo '<tr ' . $this->render_attributes( $row_key ) . '>';
		} else {
			echo '</tr><tr ' . $this->render_attributes( $row_key ) . '>';
		}

		foreach ( $columns as $index => $column_item ) {

			if ( ! empty( $column_item['cellIconType'] ) ) {
				$this->set_attribute( $cell_icon_wrap_key, 'class', 'mab-table-cell-icon' );

				$icon_position = ! empty( $column_item['cellIconPosition'] ) ? esc_attr( $column_item['cellIconPosition'] ) : 'before';
				$this->set_attribute( $cell_icon_wrap_key, 'class', 'mab-table-cell-icon-' . $icon_position );

				if ( ! empty( $column_item['cellIcon']['icon'] ) ) {
					$this->set_attribute( $cell_icon_key, 'class', [
						'mab-icon',
						$column_item['cellIcon']['icon'],
					] );
				}
			}

			if ( ! empty( $column_item['link'] ) ) {
				$this->set_link_attributes( $link_key, $column_item['link'] );
			}

			if ( ! empty( $column_item['columnData'] ) || ! empty( $column_item['cellIconType'] ) ) {
				echo '<td ' . $this->render_attributes( $cell_key ) . '>';

				if ( ! empty( $column_item['link'] ) ) {
					echo '<a ' . $this->render_attributes( $link_key ) . '>';
				}

				if ( ! empty( $column_item['cellIconType'] ) ) {
					echo '<span ' . $this->render_attributes( $cell_icon_wrap_key ) . '>';
					if ( 'icon' === $column_item['cellIconType'] && ! empty( $column_item['cellIcon']['icon'] ) ) {
						echo '<i ' . $this->render_attributes( $cell_icon_key ) . '></i>';
					} elseif ( 'image' === $column_item['cellIconType'] ) {
						$image_settings = $this->get_normalized_image_settings( $column_item );
						echo $this->render_table_image( $image_settings );
					}
					echo '</span>';
				}

				echo '<span ' . $this->render_attributes( $cell_text_key ) . '>';
				echo $this->render_dynamic_data( $column_item['columnData'] );
				echo '</span>';

				if ( ! empty( $column_item['link'] ) ) {
					echo '</a>';
				}

				echo '</td>';
			}
		}

		echo '</tr>';

		$html = ob_get_clean();
		$this->loop_index++;

		return $html;
	}

	// Render element HTML
	public function render() {
		$settings = $this->settings;

		if ( isset( $settings['source'] ) && 'google-sheets' === $settings['source'] ) {
			$sheet_data = $this->get_google_sheets_data();

			if ( is_wp_error( $sheet_data['error'] ) ) {
				$error_message = $sheet_data['error']->get_error_message();

				return $this->render_element_placeholder(
					[
						'icon-class' => 'ion-md-warning',
						'title'      => esc_html( $error_message ),
					]
				);
			}

			if ( empty( $sheet_data['data'] ) ) {
				return $this->render_element_placeholder(
					[
						'icon-class' => 'ion-md-warning',
						'title'      => esc_html__( 'Failed to fetch data from Google Sheets.', 'max-addons' ),
					]
				);
			}
		}

		$this->set_attribute( 'table', 'class', [ 'mab-table' ] );

		$table_settings = [];

		if ( ! empty( $settings['sortable'] ) ) {
			$table_settings['sortable'] = true;
		}

		if ( ! empty( $settings['search'] ) ) {
			$table_settings['search'] = true;

			$table_settings['search_placeholder'] = ! empty( $settings['searchPlaceholder'] ) ? esc_html( $settings['searchPlaceholder'] ) : esc_html__( 'Search...', 'max-addons' );
		}

		if ( ! empty( $settings['pagination'] ) ) {
			$table_settings['pagination'] = true;
		}

		if ( ! empty( $settings['limit'] ) ) {
			$table_settings['limit'] = intval( $settings['limit'] );
		}

		if ( ! empty( $settings['buttonsCount'] ) ) {
			$table_settings['buttons_count'] = intval( $settings['buttonsCount'] );
		}

		if ( ! empty( $settings['summary'] ) ) {
			$table_settings['summary'] = esc_html( $settings['summary'] );

			if ( ! empty( $settings['showingText'] ) ) {
				$table_settings['showing_text'] = esc_html( $settings['showingText'] );
			}

			if ( ! empty( $settings['resultsText'] ) ) {
				$table_settings['results_text'] = esc_html( $settings['resultsText'] );
			}
		}

		if ( ! empty( $settings['nextButton'] ) ) {
			$table_settings['next_button'] = esc_html( $settings['nextButton'] );

			if ( ! empty( $settings['nextButtonText'] ) ) {
				$table_settings['next_button_text'] = esc_html( $settings['nextButtonText'] );
			}
		}

		if ( ! empty( $settings['prevButton'] ) ) {
			$table_settings['prev_button'] = esc_html( $settings['prevButton'] );

			if ( ! empty( $settings['prevButtonText'] ) ) {
				$table_settings['prev_button_text'] = esc_html( $settings['prevButtonText'] );
			}
		}

		if ( ! empty( $settings['stackTable'] ) && 'never' !== $settings['stackTable'] ) {
			$breakpoint = \Bricks\Breakpoints::get_breakpoint_by( 'key', $settings['stackTable'] );

			if ( ! empty( $breakpoint['width'] ) ) {
				$table_settings['stack_on'] = intval( $breakpoint['width'] );
			}

			if ( isset( $settings['stackedDisplay'] ) && 'row' === $settings['stackedDisplay'] ) {
				$this->set_attribute( '_root', 'class', 'mab-stacked-table-row' );
			}
		}

		$this->set_attribute( '_root', 'data-settings', wp_json_encode( $table_settings ) );
		?>
		<div <?php echo wp_kses_post( $this->render_attributes( '_root' ) ); ?>>
			<div class="mab-dynamic-table-container"></div>
			<table <?php echo wp_kses_post( $this->render_attributes( 'table' ) ); ?>>
				<?php
					$this->render_header();
					$this->render_body();
				?>
			</table>
		</div>
		<?php
	}
}
