<?php
namespace MaxAddonsPro\Elements;

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

class Woo_Checkout_Billing_Fields_Element extends \Bricks\Element {
	public $category        = 'max-addons-woo-checkout-elements';
	public $name            = 'max-woo-checkout-billing-fields';
	public $icon            = 'ti-layout-cta-left max-element';

	public function get_label() {
		return esc_html__( 'Billing Fields', 'max-addons' );
	}

	public function get_keywords() {
		return [ 'checkout' ];
	}

	public function set_control_groups() {
		$this->control_groups['fields'] = [
			'title' => esc_html__( 'Fields', 'max-addons' ),
		];

		$this->control_groups['labels'] = [
			'title' => esc_html__( 'Labels', 'max-addons' ),
			'tab'   => 'content',
		];
	}

	// Enqueue element styles and scripts
	public function enqueue_scripts() {
		wp_enqueue_style( 'mab-woo-checkout-billing-fields' );
		// wp_enqueue_script( 'mab-woo-checkout-billing-fields' );
	}

	public function set_controls() {
		$this->set_fields_controls();
		$this->set_labels_controls();
	}

	// Set fields controls
	public function set_fields_controls() {
		$fields_opts = [];

		if ( WC()->checkout() ) {
			$fields = WC()->checkout()->get_checkout_fields( 'billing' );

			foreach ( $fields as $key => $field ) {
				$fields_opts[$key] = $field['label'];
			}

			$fields_opts['custom'] = esc_html__( 'Custom', 'max-addons' );
		}

		$this->controls['fields'] = [
			'tab'           => 'content',
			'group'         => 'fields',
			'placeholder'   => esc_html__( 'Field', 'max-addons' ),
			'type'          => 'repeater',
			'selector'      => '.form-row',
			'titleProperty' => 'field',
			'fields'        => [
				'field' => [
					'label'     => esc_html__( 'Field', 'max-addons' ),
					'type'      => 'select',
					'options'   => $fields_opts,
					'clearable' => false,
				],

				'custom_field_type' => [
					'label'     => esc_html__( 'Type', 'max-addons' ),
					'type'      => 'select',
					'options'   => [
						'email'      => esc_html__( 'Email', 'max-addons' ),
						'text'       => esc_html__( 'Text', 'max-addons' ),
						'textarea'   => esc_html__( 'Textarea', 'max-addons' ),
						'tel'        => esc_html__( 'Tel', 'max-addons' ),
						'number'     => esc_html__( 'Number', 'max-addons' ),
						'url'        => esc_html__( 'URL', 'max-addons' ),
						'checkbox'   => esc_html__( 'Checkbox', 'max-addons' ),
						'select'     => esc_html__( 'Select', 'max-addons' ),
						'radio'      => esc_html__( 'Radio', 'max-addons' ),
						'datepicker' => esc_html__( 'Datepicker', 'max-addons' ),
						'password'   => esc_html__( 'Password', 'max-addons' ),
						'hidden'     => esc_html__( 'Hidden', 'max-addons' ),
					],
					'clearable' => false,
					'required'  => [ 'field', '=', [ 'custom' ] ],
				],

				'custom_field_key' => [
					'label'       => esc_html__( 'Key', 'max-addons' ),
					'description' => esc_html__( 'Enter custom field key', 'max-addons' ),
					'placeholder' => 'billing_custom_field',
					'default'     => 'billing_',
					'type'        => 'text',
					'required'    => [ 'field', '=', [ 'custom' ] ],
				],

				'label' => [
					'label'       => esc_html__( 'Label', 'max-addons' ),
					'description' => esc_html__( 'Keep blank to use the default label', 'max-addons' ),
					'type'        => 'text',
				],

				'placeholder' => [
					'label'    => esc_html__( 'Placeholder', 'max-addons' ),
					'type'     => 'text',
					'required' => [ 'custom_field_type', '!=', [ 'hidden' ] ],
				],

				'description' => [
					'label'    => esc_html__( 'Description', 'max-addons' ),
					'type'     => 'textarea',
					'required' => [ 'custom_field_type', '!=', [ 'hidden' ] ],
				],

				'value' => [
					'label'    => esc_html__( 'Value', 'max-addons' ),
					'type'     => 'text',
					'info'     => esc_html__( 'Set the default field value/content.', 'max-addons' ),
				],

				'checkboxInfo' => [
					'content'  => esc_html__( 'Separate values by comma.', 'max-addons' ),
					'type'     => 'info',
					'required' => [
						[ 'custom_field_type', '=', 'checkbox' ],
						[ 'value', '!=', '' ],
					],
				],

				'options' => [
					'label'    => esc_html__( 'Options (one per line)', 'max-addons' ),
					'type'     => 'textarea',
					'required' => [
						[ 'field', '=', [ 'custom' ] ],
						[ 'custom_field_type', '=', [ 'checkbox', 'select', 'radio' ] ]
					],
				],

				'width' => [
					'label'       => esc_html__( 'Width', 'max-addons' ) . ' (%)',
					'type'        => 'number',
					'unit'        => '%',
					'min'         => 0,
					'max'         => 100,
					'placeholder' => 100,
					'css'         => [
						[
							'property' => 'width',
						],
					],
					'required'    => [ 'custom_field_type', '!=', 'hidden' ],
				],

				'height' => [
					'label'    => esc_html__( 'Height', 'max-addons' ),
					'type'     => 'number',
					'units'    => true,
					'css'      => [
						[
							'property' => 'height',
						],
					],
					'required' => [
						[ 'field', '=', [ 'custom' ] ],
						[ 'custom_field_type', '=', [ 'textarea' ] ]
					],
				],

				'required' => [
					'label'    => esc_html__( 'Required', 'max-addons' ),
					'type'     => 'checkbox',
					'inline'   => true,
					'default'  => 1,
					'required' => [ 'custom_field_type', '!=', [ 'hidden' ] ],
				],
			],

			'default'       => [
				[
					'field'       => 'billing_first_name',
					'label'       => esc_html__( 'First name', 'max-addons' ),
					'required'    => true,
				],
				[
					'field'       => 'billing_last_name',
					'label'       => esc_html__( 'Last name', 'max-addons' ),
					'required'    => true,
				],
				[
					'field'       => 'billing_company',
					'label'       => esc_html__( 'Company name', 'max-addons' ),
				],
				[
					'field'       => 'billing_country',
					'label'       => esc_html__( 'Country / Region', 'max-addons' ),
					'required'    => true,
				],
				[
					'field'       => 'billing_address_1',
					'label'       => esc_html__( 'Street address', 'max-addons' ),
					'required'    => true,
				],
				[
					'field'       => 'billing_address_2',
				],
				[
					'field'       => 'billing_city',
					'required'    => true,
				],
				[
					'field'       => 'billing_state',
					'required'    => true,
				],
				[
					'field'       => 'billing_postcode',
					'required'    => true,
				],
				[
					'field'       => 'billing_phone',
					'required'    => true,
				],
				[
					'field'       => 'billing_email',
					'required'    => true,
				],
			],
		];

		$this->controls['placeholderTypography'] = [
			'tab'    => 'content',
			'group'  => 'fields',
			'type'   => 'typography',
			'label'  => esc_html__( 'Placeholder typography', 'max-addons' ),
			'css'    => [
				[
					'property' => 'font',
					'selector' => '::placeholder',
				],
			],
		];

		$this->controls['descriptionTypography'] = [
			'tab'    => 'content',
			'group'  => 'fields',
			'type'   => 'typography',
			'label'  => esc_html__( 'Description typography', 'max-addons' ),
			'css'    => [
				[
					'property' => 'font',
					'selector' => '.form-row .description',
				],
			],
			'inline' => true,
			'small'  => true,
		];

		$this->controls['fieldSeparator'] = [
			'tab'   => 'content',
			'group' => 'fields',
			'label' => esc_html__( 'Field', 'max-addons' ),
			'type'  => 'separator',
		];

		$this->controls['inputTypography'] = [
			'tab'    => 'content',
			'group'  => 'fields',
			'type'   => 'typography',
			'label'  => esc_html__( 'Typography', 'max-addons' ),
			'css'    => [
				[
					'property' => 'font',
					'selector' => '.form-row input:not([type=submit]), .form-row textarea, .form-row select',
				],
			],
			'inline' => true,
			'small'  => true,
		];

		$this->controls['inputBackgroundColor'] = [
			'tab'    => 'content',
			'group'  => 'fields',
			'type'   => 'color',
			'label'  => esc_html__( 'Background color', 'max-addons' ),
			'css'    => [
				[
					'property' => 'background-color',
					'selector' => '.form-row input:not([type=submit]), .form-row textarea, .form-row select',
				],
			],
			'inline' => true,
			'small'  => true,
		];

		$this->controls['inputBorder'] = [
			'tab'    => 'content',
			'group'  => 'fields',
			'type'   => 'border',
			'label'  => esc_html__( 'Border', 'max-addons' ),
			'css'    => [
				[
					'property' => 'border',
					'selector' => '.form-row input:not([type=submit]), .form-row textarea, .form-row select',
				],
			],
			'inline' => true,
			'small'  => true,
		];

		$this->controls['inputBoxShadow'] = [
			'tab'    => 'content',
			'group'  => 'fields',
			'label'  => esc_html__( 'Box shadow', 'max-addons' ),
			'type'   => 'box-shadow',
			'css'    => [
				[
					'property' => 'box-shadow',
					'selector' => '.form-row input:not([type=submit]), .form-row textarea, .form-row select',
				],
			],
			'inline' => true,
			'small'  => true,
		];

		$this->controls['inputPadding'] = [
			'tab'   => 'content',
			'group' => 'fields',
			'label' => esc_html__( 'Padding', 'max-addons' ),
			'type'  => 'spacing',
			'css'   => [
				[
					'property' => 'padding',
					'selector' => '.form-row input:not([type=submit]), .form-row textarea, .form-row select',
				],
			],
		];
	}

	// Set labels controls
	public function set_labels_controls() {
		$this->controls['hideLabels'] = [
			'tab'      => 'content',
			'group'    => 'labels',
			'label'    => esc_html__( 'Hide labels', 'max-addons' ),
			'type'     => 'checkbox',
			'css'      => [
				[
					'selector' => 'label',
					'property' => 'display',
					'value'    => 'none',
				],
			],
			'rerender' => true,
		];

		$this->controls['labelsTypography'] = [
			'tab'      => 'content',
			'group'    => 'labels',
			'type'     => 'typography',
			'label'    => esc_html__( 'Labels typography', 'max-addons' ),
			'css'      => [
				[
					'property' => 'font',
					'selector' => '.form-row label',
				],
			],
			'inline'   => true,
			'small'    => true,
			'required' => [ 'hideLabels', '=', '' ],
		];

		$this->controls['requiredTypography'] = [
			'tab'      => 'content',
			'group'    => 'labels',
			'type'     => 'typography',
			'label'    => esc_html__( 'Required typography', 'max-addons' ),
			'css'      => [
				[
					'property' => 'font',
					'selector' => '.form-row label abbr[title]',
				],
			],
			'exclude'  => [
				'font-family',
				'font-style',
				'text-align',
				'text-transform',
				'line-height',
				'letter-spacing',
				'font-variation-settings',
				'white-space',
				'text-wrap',
			],
			'inline'   => true,
			'small'    => true,
			'required' => [ 'hideLabels', '=', '' ],
		];

		$this->controls['labelsMargin'] = [
			'tab'         => 'content',
			'group'       => 'labels',
			'type'        => 'spacing',
			'label'       => esc_html__( 'Margin', 'max-addons' ),
			'css'         => [
				[
					'property' => 'margin',
					'selector' => '.form-row label',
				],
			],
			'placeholder' => [
				'top'    => 0,
				'right'  => 0,
				'bottom' => 5,
				'left'   => 0,
			],
			'required'    => [ 'hideLabels', '=', '' ],
		];
	}

	public function render() {
		$settings = $this->settings;
		$field    = [];

		$checkout = WC()->checkout();

		if ( ! $checkout ) {
			return;
		}

		$this->render_attributes( '_root', 'class', 'woocommerce-checkout' );
		?>
		<div <?php echo $this->render_attributes( '_root' ); ?>>
			<?php
				$fields = $settings['fields'];

				foreach ( $fields as $field_details ) {
					$key = ! empty( $field_details['field'] ) ? $field_details['field'] : '';

					if ( 'custom' === $key ) {
						$key  = isset( $field_details['custom_field_key'] ) ? $field_details['custom_field_key'] : '';
						$type = isset( $field_details['custom_field_type'] ) ? $field_details['custom_field_type'] : 'text';

						if ( $key ) {
							$billing_fields[ $key ]['type'] = $type;

							if ( ( 'checkbox' === $type || 'radio' === $type ) && ! empty( $field_details['options'] ) ) {
								$billing_fields[ $key ]['type'] = 'radio';
								$billing_fields[ $key ]['class'] = 'mab-woo-checkout-options-field';
								$options = \Bricks\Helpers::parse_textarea_options( $field_details['options'] );

								foreach ( $options as $option_key => $option_value ) {
									$billing_fields[ $key ]['options'][ $option_key ] = $option_value;
								}
							}
						}
					} else {
						$billing_fields = WC()->checkout()->get_checkout_fields( 'billing' );
					}

					if ( ! empty( $billing_fields[$key] ) ) {
						$field = $billing_fields[$key];

						if ( isset( $field_details['label'] ) ) {
							$field['label'] = $field_details['label'];
						}

						if ( isset( $field_details['description'] ) ) {
							$field['description'] = $field_details['description'];
						}

						if ( isset( $field_details['placeholder'] ) ) {
							$field['placeholder'] = $field_details['placeholder'];
						}

						$required = ( isset( $field_details['required'] ) ) ? 1 : 0;
						$field['required'] = $required;

						woocommerce_form_field( $key, $field, $checkout->get_value( $key ) );
					}
				}
			?>
		</div>
		<?php
	}
}
