<?php
namespace MaxAddonsPro\Features;

use Bricks\Elements as Bricks_Elements;

/**
 * Handles logic for Background slideshow.
 *
 * @package MaxAddons
 * @since 1.12.0
 */

/**
 * Exit if accessed directly.
 */
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * MAB_Background_Slideshow.
 */
class MAB_Background_Slideshow extends MAB_Feature_Base {

	/**
	 * Initializing features.
	 *
	 * @since 1.12.0
	 */
	public static function init() {
		if ( ! self::is_enabled() ) {
			return;
		}

		$elements = Bricks_Elements::$elements;

		foreach( $elements as $element_key => $element ) {
			if ( 'container' === $element_key || 'section' === $element_key ) {
				add_filter( "bricks/elements/$element_key/control_groups", array( __CLASS__, 'mab_background_slideshow_group' ) );
				add_filter( "bricks/elements/$element_key/controls", array( __CLASS__, 'mab_background_slideshow_controls' ) );
			}
		}

		add_filter( 'bricks/element/set_root_attributes', array( __CLASS__, 'mab_background_slideshow_render_attr' ), 10, 2 );
	}

	/**
	 * Is enabled.
	 *
	 * Check if feature is enabled or not.
	 *
	 * @since 1.12.0
	 *
	 * @return boolean true or false
	 */
	public static function is_enabled() {
		$enabled_features = get_option('max_bricks_features');

		if ( 'disabled' != $enabled_features ) {
			if ( is_array( $enabled_features ) && in_array( 'background-slideshow', $enabled_features ) ) {
				return true;
			}
		}

		return false;
	}

	/**
	 * Adds Background slideshow group
	 *
	 * @since 1.12.0
	 *
	 * @return array
	 */
	public static function mab_background_slideshow_group( $control_groups ) {
		$enabled_features = get_option('max_bricks_features');

		if ( is_array( $enabled_features ) && 'disabled' != $enabled_features ) {
			if ( in_array( 'background-slideshow', $enabled_features ) ) {
				$control_groups['mabBackgroundSlideshow'] = [
					'tab'   => 'style',
					'title' => esc_html__( 'Background slideshow', 'max-addons' ),
				];
			}
		}

		return $control_groups;
	}

	/**
	 * Adds Background slideshow controls
	 *
	 * @since 1.12.0
	 *
	 * @return array
	 */
	public static function mab_background_slideshow_controls( $controls ) {
		$controls['maxEnableBgSlideshow'] = [
			'tab'   => 'style',
			'group' => 'mabBackgroundSlideshow',
			'label' => esc_html__( 'Background slideshow', 'max-addons' ),
			'type'  => 'checkbox',
		];

		$controls['bgSlideshowInfo'] = [
			'tab'      => 'style',
			'group'    => 'mabBackgroundSlideshow',
			'content'  => esc_html__( 'Background Slideshow doesn\'t work in the editor. Please save the page and check it on the frontend.', 'max-addons' ),
			'type'     => 'info',
			'required' => [ 'maxEnableBgSlideshow', '!=', '' ],
		];

		$controls['bgSlideshowGallery'] = [
			'tab'  		=> 'style',
			'group'		=> 'mabBackgroundSlideshow',
			'type' 		=> 'image-gallery',
			'required'  => [ 'maxEnableBgSlideshow', '!=', '' ],
		];

		$controls['bgSlideshowBackgroundSize'] = [
			'tab'         => 'style',
			'group'       => 'mabBackgroundSlideshow',
			'label'       => esc_html__( 'Background size', 'max-addons' ),
			'placeholder' => esc_html__( 'Cover', 'max-addons' ),
			'type'        => 'select',
			'options'     => [
				'contain'    => esc_html__( 'Contain', 'max-addons' ),
				'cover'      => esc_html__( 'Cover', 'max-addons' ),
				'fill'       => esc_html__( 'Fill', 'max-addons' ),
				'scale-down' => esc_html__( 'Scale down', 'max-addons' ),
				'none'       => esc_html__( 'None', 'max-addons' ),
			],
			'inline'      => true,
			'clearable'   => true,
			'css'         => [
				[
					'property' => 'background-size',
					'selector' => '.max-bg-slideshow-slide-image',
				],
			],
			'required'    => [ 'maxEnableBgSlideshow', '!=', '' ],
		];

		$controls['bgSlideshowBackgroundPosition'] = [
			'tab'         => 'style',
			'group'       => 'mabBackgroundSlideshow',
			'label'       => esc_html__( 'Background position', 'max-addons' ),
			'placeholder' => esc_html__( 'Center center', 'max-addons' ),
			'type'        => 'select',
			'options'     => [
				'top left'      => esc_html__( 'Top left', 'max-addons' ),
				'top center'    => esc_html__( 'Top center', 'max-addons' ),
				'top right'     => esc_html__( 'Top right', 'max-addons' ),

				'center left'   => esc_html__( 'Center left', 'max-addons' ),
				'center center' => esc_html__( 'Center center', 'max-addons' ),
				'center right'  => esc_html__( 'Center right', 'max-addons' ),

				'bottom left'   => esc_html__( 'Bottom left', 'max-addons' ),
				'bottom center' => esc_html__( 'Bottom center', 'max-addons' ),
				'bottom right'  => esc_html__( 'Bottom right', 'max-addons' ),
			],
			'inline'      => true,
			'clearable'   => true,
			'css'         => [
				[
					'property' => 'background-position',
					'selector' => '.max-bg-slideshow-slide-image',
				],
			],
			'required'    => [ 'maxEnableBgSlideshow', '!=', '' ],
		];

		$controls['bgSlideshowSlideDuration'] = [
			'tab'         => 'style',
			'group'       => 'mabBackgroundSlideshow',
			'label'       => esc_html__( 'Duration in ms', 'max-addons' ),
			'type'        => 'number',
			'placeholder' => 5000,
			'required'    => [ 'maxEnableBgSlideshow', '!=', '' ],
		];

		$controls['bgSlideshowTransition'] = [
			'tab'         => 'style',
			'group'       => 'mabBackgroundSlideshow',
			'label'       => esc_html__( 'Transition', 'max-addons' ),
			'placeholder' => esc_html__( 'Fade', 'max-addons' ),
			'type'        => 'select',
			'options'     => [
				'fade'        => esc_html__( 'Fade', 'max-addons' ),
				'slide_left'  => esc_html__( 'Slide left', 'max-addons' ),
				'slide_right' => esc_html__( 'Slide right', 'max-addons' ),
				'slide_up'    => esc_html__( 'Slide up', 'max-addons' ),
				'slide_down'  => esc_html__( 'Slide down', 'max-addons' ),
			],
			'inline'      => true,
			'clearable'   => true,
			'required'    => [ 'maxEnableBgSlideshow', '!=', '' ],
		];

		$controls['bgSlideshowTransitionDuration'] = [
			'tab'         => 'style',
			'group'       => 'mabBackgroundSlideshow',
			'label'       => esc_html__( 'Transition duration in ms', 'max-addons' ),
			'type'        => 'number',
			'placeholder' => 500,
			'required'    => [ 'maxEnableBgSlideshow', '!=', '' ],
		];

		$controls['bgSlideshowInfinite'] = [
			'tab'      => 'style',
			'group'    => 'mabBackgroundSlideshow',
			'label'    => esc_html__( 'Loop', 'max-addons' ),
			'type'     => 'checkbox',
			'required' => [ 'maxEnableBgSlideshow', '!=', '' ],
		];

		$controls['bgSlideshowKenBurns'] = [
			'tab'      => 'style',
			'group'    => 'mabBackgroundSlideshow',
			'label'    => esc_html__( 'Ken burns effect', 'max-addons' ),
			'type'     => 'checkbox',
			'required' => [ 'maxEnableBgSlideshow', '!=', '' ],
		];

		$controls['bgSlideshowKenBurnsDirection'] = [
			'tab'         => 'style',
			'group'       => 'mabBackgroundSlideshow',
			'label'       => esc_html__( 'Ken burns direction', 'max-addons' ),
			'placeholder' => esc_html__( 'In', 'max-addons' ),
			'type'        => 'select',
			'options'     => [
				'in'  => esc_html__( 'In', 'max-addons' ),
				'out' => esc_html__( 'Out', 'max-addons' ),
			],
			'inline'      => true,
			'clearable'   => true,
			'required'    => [
				[ 'maxEnableBgSlideshow', '!=', '' ],
				[ 'bgSlideshowKenBurns', '!=', '' ]
			],
		];

		$controls['bgSlideshowZindex'] = [
			'tab'         => 'style',
			'group'       => 'mabBackgroundSlideshow',
			'label'       => 'Z-index',
			'type'        => 'number',
			'large'       => true,
			'placeholder' => -1,
			'css'         => [
				[
					'property' => 'z-index',
					'selector' => '.max-background-slideshow',
				],
			],
			'required' => [ 'maxEnableBgSlideshow', '!=', '' ],
		];
    
        return $controls;
	}

	public static function get_normalized_image_settings( $element ) {
		$settings = $element->settings;
		$items    = $settings['bgSlideshowGallery'] ?? [];
		$size     = $items['size'] ?? BRICKS_DEFAULT_IMAGE_SIZE;

		// Dynamic data
		if ( ! empty( $items['useDynamicData'] ) ) {
			$items['images'] = [];

			$images = $element->render_dynamic_data_tag( $items['useDynamicData'], 'image' );

			if ( is_array( $images ) ) {
				foreach ( $images as $image_id ) {
					$items['images'][] = [
						'id'   => $image_id,
						'full' => wp_get_attachment_image_url( $image_id, 'full' ),
						'url'  => wp_get_attachment_image_url( $image_id, $size )
					];
				}
			}
		}

		// Old data structure (images were saved as one array directly on $items)
		if ( ! isset( $items['images'] ) ) {
			$images = ! empty( $items ) ? $items : [];

			unset( $items );

			$items['images'] = $images;
		}

		// Get 'size' from first image if not set
		$first_image_size = ! empty( $items['images'][0]['size'] ) ? $items['images'][0]['size'] : false;
		$size             = empty( $items['size'] ) && $first_image_size ? $first_image_size : $size;

		// Get image 'url' for requested $size
		foreach ( $items['images'] as $key => $image ) {
			if ( ! empty( $image['id'] ) ) {
				$items['images'][ $key ]['url'] = wp_get_attachment_image_url( $image['id'], $size );
			}
		}

		$settings['bgSlideshowGallery']         = $items;
		$settings['bgSlideshowGallery']['size'] = $size;

		return $settings;
	}

	public static function mab_background_slideshow_render_attr( $attributes, $element ) {
		$settings = self::get_normalized_image_settings( $element );

		if ( 'container' === $element->name || 'section' === $element->name ) {
			if ( isset( $settings['maxEnableBgSlideshow'] ) && true === $settings['maxEnableBgSlideshow'] ) {
				if ( isset( $settings['bgSlideshowGallery'] ) ) {
					wp_enqueue_script( 'bricks-swiper' );
					wp_enqueue_style( 'bricks-swiper' );
					wp_enqueue_script( 'mab-background-slideshow' );
					wp_enqueue_style( 'mab-background-slideshow' );

					$attributes['class'][] = 'max-bg-slideshow';

					$images = $settings['bgSlideshowGallery']['images'] ?? false;

					if ( ! $images ) {
						return;
					}

					$image_size        = isset( $settings['bgSlideshowGallery']['size'] ) ? $settings['bgSlideshowGallery']['size'] : 'full';
					$transition        = isset( $settings['bgSlideshowTransition'] ) ? $settings['bgSlideshowTransition'] : 'fade';
					$autoplay_speed    = isset( $settings['bgSlideshowSlideDuration'] ) ? $settings['bgSlideshowSlideDuration'] : 5000;
					$speed             = isset( $settings['bgSlideshowTransitionDuration'] ) ? $settings['bgSlideshowTransitionDuration'] : 500;
					$loop              = isset( $settings['bgSlideshowInfinite'] );
					$kenBurns          = isset( $settings['bgSlideshowKenBurns'] );
					$kenBurnsDirection = isset( $settings['bgSlideshowKenBurnsDirection'] ) ? $settings['bgSlideshowKenBurnsDirection'] : 'in';
					$images_arr        = [];
					
					foreach ( $images as $key => $value ) {
						$image_id = $value['id'];
						$images_arr[] = esc_url( wp_get_attachment_image_url( $image_id, $image_size ) );
					}

					$swiperOptions = array(
						'slides'         => $images_arr,
						'loop'           => $loop,
						'transition'     => esc_attr( $transition ),
						'autoplay_speed' => intval( $autoplay_speed ),
						'speed'          => intval( $speed ),
						'ken_burns'      => $kenBurns,
					);

					if ( $kenBurns ) {
						$swiperOptions['ken_burns_direction'] = $kenBurnsDirection;
					}

					$attributes['data-max-bg-slideshow'] = json_encode( $swiperOptions );
				}
			}
		}

		return $attributes;
	}
}

// Initialize the class.
MAB_Background_Slideshow::init();
