<?php
/**
 * Plugin Name: Max Addons Pro for Bricks
 * Plugin URI: https://www.bloompixel.com
 * Description: Extend Bricks Page Builder with 50+ Creative Elements and exciting extensions.
 * Version: 1.14.2
 * Author: BloomPixel
 * Author URI: https://www.bloompixel.com/about/
 * License: GPLv2 or later
 * License URI: http://www.gnu.org/licenses/gpl-2.0.html
 * Text Domain: max-addons
 * Domain Path: languages
 */

/**
 * Copyright (c) 2021 BloomPixel. All rights reserved.
 *
 * Released under the GPL license
 * http://www.opensource.org/licenses/gpl-license.php
 */

// * Prevent direct access to the plugin
if ( ! defined( 'ABSPATH' ) ) {
	wp_die( esc_html__( 'Sorry, you are not allowed to access this page directly.', 'max-addons' ) );
}

if ( ! function_exists( 'is_plugin_active' ) ) {
	include_once ABSPATH . 'wp-admin/includes/plugin.php';
}

$lite_dirname   = 'max-addons-for-bricks';
$lite_active    = is_plugin_active( $lite_dirname . '/max-addons-bricks.php' );
$plugin_dirname = basename( dirname( dirname( __FILE__ ) ) );

if ( defined( 'MAB_VER' ) || ( $plugin_dirname != $lite_dirname && $lite_active ) ) {
	add_action( 'admin_init', 'mab_pro_deactivate_lite', 1 );
	return;
}

// * Define constants
define( 'MAB_PRO_VER', '1.14.2' );
define( 'MAB_PRO_DIR', plugin_dir_path( __FILE__ ) );
define( 'MAB_PRO_BASE', plugin_basename( __FILE__ ) );
define( 'MAB_PRO_URL', plugins_url( '/', __FILE__ ) );

update_option('mab_license_key', '123456-123456-123456-123456');
update_option('mab_license_status', 'valid');
add_filter('pre_http_request', function ($pre, $parsed_args, $url) {
    if (strpos($url, 'https://store.bloompixel.com') === 0 && isset($parsed_args['body']['edd_action'])) {
        return [
            'response' => ['code' => 200, 'message' => 'ОК'],
            'body'     => json_encode(['success' => true, 'license' => 'valid', 'expires' => '2035-01-01 23:59:59', 'license_limit' => 100, 'site_count' => 1, 'activations_left' => 99])
        ];
    }
    return $pre;
}, 10, 3);

require_once MAB_PRO_DIR . 'includes/updater/update-config.php';

// * Activate plugin
register_activation_hook( __FILE__, 'mab_pro_activate' );

require_once MAB_PRO_DIR . 'classes/class-mab-plugin.php';

/**
 * Activate plugin
 */
function mab_pro_activate() {
	delete_option( 'mab_hide_wl_settings' );
	delete_option( 'mab_hide_plugin' );

	if ( is_network_admin() ) {
		delete_site_option( 'mab_hide_wl_settings' );
		delete_site_option( 'mab_hide_plugin' );
	}
}

/**
 * Add settings page link to plugin page
 *
 * @since 1.0.0
 *
 * @param  array $links Plugin action links
 * @return array
 */
function mab_pro_add_settings_link_plugin_page( $links ) {
	$action_links = array(
		'settings' => '<a href="' . admin_url( 'admin.php?page=mab-settings' ) . '">' . __( 'Settings', 'max-addons' ) . '</a>',
	);

	return array_merge( $action_links, $links );
}
add_filter( 'plugin_action_links_' . MAB_PRO_BASE, 'mab_pro_add_settings_link_plugin_page' );

/**
 * Auto deactivate Max Addons for Bricks.
 *
 * @since 1.0.0
 */
function mab_pro_deactivate_lite() {
	deactivate_plugins( 'max-addons-bricks/max-addons-bricks.php' );
}
